#!/usr/bin/env python

import os,re,fnmatch,sys
import math,shutil
import pickle
from datetime import datetime

### regular expressions for numbers
int_re = r'\d+'
real_dn = r'-?\d+\.\d*|\d*\.\d+'
real_sn = r'-?\d+\.\d*[Ee][+\-]\d\d?'
real = r'(' + real_sn + '|' + real_dn + '|' + int_re + r')'


#######################################################################
### Read file backwards
def BackwardsReader(filename,BLKSIZE=4096):
    """Read a file line by line, backwards"""
    buf=""
    #print filename
    filename.seek(-1,os.SEEK_END) #set file's current position to the last line
    lastchar = filename.read(1)   #read the last character
    trailing_newline = (lastchar == "\n")
    

    while 1:
        newline_pos = buf.rfind("\n") 
        pos = filename.tell() #return file's current position
        if newline_pos != -1:
            # Found a newline
            line = buf[newline_pos+1:]
            buf = buf[:newline_pos]
            if pos or newline_pos or trailing_newline:
                line += "\n"
            yield line
        elif pos:
            # Need to fill buffer
            toread = min(BLKSIZE,pos)
            filename.seek(-toread,os.SEEK_CUR) # set file's current position to current position-toread 
            buf = filename.read(toread) + buf  # add file content until "toread" to buffer
            filename.seek(-toread,os.SEEK_CUR) # set file's current position to current position-toread
            if pos == toread:
                buf = "\n"+buf
        else:
            # Start-of-file
            return


#######################################################################
### Reading in single result file
class FileReader:
    def __init__(self,path,filename):
        self.header=[]
        self.as=0.0
        self.muR=0.0
        self.mtop=0.0
        self.mbot=0.0
        self.mhiggs=0.0
        self.Val=0.0
        self.Err=0.0
        self.Prob=0.0
        self.Found=False
        self.Hist={}
        self.xsecPDF=[]
        self.path=path
        self.filename=filename
        self.isFirst=True
    def GetRes(self):
        filepath=os.path.join(self.path,self.filename)
        if os.path.isfile(filepath):
            self.Found=True
            ftmp=open(filepath)
            lines = ftmp.readlines()
            ftmp.close()
            isHisto=False
            lastpart=False
            if self.isFirst:
                firstpart=True
            else:
                firstpart=False
            for line in lines:
                match = re.search(r'Higgs boson mass',line)
                if match:
                    match2 = re.search(r'=\s*(' + real + ')',line)
                    if match2:
                        val=match2.group(1)
                        self.mhiggs=float(val)
                match = re.search(r'Renormalization scale',line)
                if match:
                    match2 = re.search(r'=\s*(' + real + ')',line)
                    if match2:
                        val=match2.group(1)
                        self.muR=float(val)
                match = re.search(r'Top-quark mass',line)
                if match:
                    match2 = re.search(r'=\s*(' + real + ')',line)
                    if match2:
                        val=match2.group(1)
                        self.mtop=float(val)
                match = re.search(r'Bottom-quark mass',line)
                if match:
                    match2 = re.search(r'=\s*(' + real + ')',line)
                    if match2:
                        val=match2.group(1)
                        self.mbot=float(val)
                match = re.search(r'Strong coupling',line)
                if match:
                    match2 = re.search(r'=\s*(' + real + ')',line)
                    if match2:
                        val=match2.group(1)
                        self.as=float(val)
                match=re.search('Sig',line)
                if match:
                    firstpart=False
                    #print line
                    #print 'Sig matched'
                    match2 = re.search(r'=\s*(' + real + ')',line)
                    if match2:
                        val=match2.group(1)
                        self.Val=float(val)
                        self.Found=True
                        #print val
                    continue
                else:
                    if firstpart:
                        if self.isFirst:
                            self.header.append(line)
                if self.Found:
                    match=re.search(r'Error',line)
                    if match:
                        #print 'Error matched'
                        match2 = re.search(r'=\s*(' + real + ')',line)
                        if match2:
                            val=match2.group(1)
                            self.Err=float(val)
                        continue
                    match=re.search(r'chi',line)
                    if match:
                        #print 'chi matched'
                        match2 = re.search(r'=\s*(' + real + ')',line)
                        if match2:
                            val=match2.group(1)
                            self.Prob=float(val)
                        continue
                if not(lastpart):
                    match = re.search(r'\*\*\*\*\*\*',line)
                    if match:
                        lastpart=True
                match = re.search(r'----',line)
                if match:
                    isHisto=True
                    match2 = re.search(r'([A-Za-z0-9_]+\s*[A-Za-z0-9_]+)',line)
                    if match2:
                        title=match2.group(1)
                        self.Hist[title]=[]
                        #print "found Hist title: ",self.Hist.keys()
                    continue
                
                m = re.findall(real,line)
                if len(m)==4:
                    bin=float(m[0])
                    val=float(m[1])
                    err=float(m[2])
                    prob=float(m[3])
                    self.Hist[title].append([bin,val,err,prob])
                match = re.search(r'----',line)
                if match:
                    isHisto=False
                match = re.search(r'\*\*\*',line)
                if match:
                    isHisto=False
                match = re.search(r'\+\+\+',line)
                if match:
                    isHisto=False
                if lastpart:
                    m = re.findall(real,line)
                    if len(m)==2:
                        iset=int(m[0])
                        if self.isFirst:
                            self.xsecPDF.append(float(m[1]))
                        else:
                            self.xsecPDF[iset]=self.xsecPDF[iset]+float(m[1])
        #print "Filereader: " , self.Hist.keys()
        

#######################################################################
### Get status of a certain sector
class Monitor:
    def __init__(self,path):
        self.path=path
        self.Neval=-100
        self.Msgs=[]
        self.StatMsg=0
        self.StatTransl={0: None,   1: 'Accuracy reached                  ', 
                                    2: 'Job finished, accuracy not reached', 
                                    3: 'Job unfinished (still running)    '}
        self.MsgTransl={0: None, 1: 'No log file found', 2: 'No result file found'}
        self.error = 'error: Unknown'
        self.accuracy_reached=False
        self.job_finished = False
        self.sigma = 'unknown'
        self.chi_2 = 'unknown'
        self.iterations='unknown'
        #print "Monitor:path=",self.path
        #print "found=",(os.path.isfile(self.path) and (os.path.getsize(self.path)>0.0))
        if os.path.isfile(self.path) and (os.path.getsize(self.path)>0.0):
            infofound=False
            nevalfound=False
            #:avoiding the Backward Reader
            #:
            fff = open(self.path)
            reversed_lines= fff.readlines()
            read_info_flag = False
            iteration_counter=1
            #:---- the section below reads the job_output.txt for each sector
            #:---- and prints out sigma, errors and chi^2 per iteration
            #:---- useful for controlling while running or for checking the 
            #:---- history of badly converging sectors
            #:---- It is by default switched off.
            #:---- To switch it on set the detailed_info flag to True
            detailed_info = False
            if (detailed_info):
                print "reading from ",self.path
                for  line_index, xline in enumerate(reversed_lines):
                    match = re.search(r'\s+Iteration\s*\#(.*):',xline)
                    if match:
                        iteration_counter=int(match.group(1))
                        newsigma_line = reversed_lines[line_index+1]
                        oldsigma_line = reversed_lines[line_index+2]
                        error_line = reversed_lines[line_index+3]
                        chi_line = reversed_lines[line_index+4]
                        match_broken = re.search(r'.*Inititalizing PDFs\.\.\..*',newsigma_line)
                        if match_broken: 
                            print "iteration broken" 
                            break
                        match_broken = re.search(r'.*Inititalizing PDFs\.\.\..*',oldsigma_line)
                        if match_broken: 
                            print "iteration broken" 
                            break
                        match_broken = re.search(r'.*Inititalizing PDFs\.\.\..*',error_line)
                        if match_broken: 
                            print "iteration broken" 
                            break
                        match_broken = re.search(r'.*Inititalizing PDFs\.\.\..*',chi_line)
                        if match_broken: 
                            print "iteration broken" 
                            break
                        evaluations_line = reversed_lines[line_index+5]
                        match1 = re.search(r'\s+sigma\s+ = (.*)\+/-(.*)',newsigma_line)
                        newsigma = float(match1.group(1))
                        newerror = float(match1.group(2))
                        match2 = re.search(r'\s+<sigma>\s+ = (.*)\+/-(.*)',oldsigma_line)
                        avsigma = float(match2.group(1))
                        averror = float(match2.group(2))
                        match3 = re.search(r'\s+rel. error\s+ = (.*)%',error_line)
                        rel_error = float(match3.group(1))
                        match4 = re.search(r'\s+chi\^2\s+=(.*)',chi_line)
                        chi2 = float(match4.group(1))
                        match5 = re.search(r'\s+evaluations\s+ = (.*)',evaluations_line)
                        nevals = float(match5.group(1))
                        print iteration_counter,nevals,newsigma,newerror,avsigma,averror,rel_error,chi2
                    #: end of if match
                #end of for xline in reversed_lines'
            #end of detailed info
                    
                    
            reversed_lines.reverse()
            #:to go back to BackwardsReader remove the above and uncomment the line below
            #for line in BackwardsReader(open(self.path)):
            
            for line in reversed_lines:
                match = re.search(r'neval',line)
                if match and not(nevalfound):
                    match2 = re.search(r'=\s*(' + int_re + ')',line)
                    if match2:
                        val=match2.group(1)
                        self.Neval=int(val)
                        nevalfound=True
                match2 = re.search(r'Accuracy',line)
                if match2:
                    self.accuracy_reached=True
                    infofound=True
                match2 = re.search(r'evaluations',line)
                if match2:
                    infofound=True
                    if not(nevalfound):
                        match3 = re.search(r'=\s*(' + int_re + ')',line)
                        if match3:
                            val=match3.group(1)
                            self.Neval=float(val)
                            
                        
                match2 = re.search(r'rel. error',line)
                if match2:
                    self.error=line[:-1]
                match2 = re.search(r'sigma',line)
                if match2:
                    self.sigma = line[:-1]
                    
                match2 = re.search(r'Probability that the Error estimation is wrong',line)
                if match2:
                    self.job_finished = True
                
                    match2 = re.search(r'evaluations',line)
                match4 = re.search(r'\s+chi\^2\s+=(.*)',line)
                if match4:
                    self.chi_2 = float(match4.group(1))
                match4 = re.search(r'\s+Iteration \#(.*):',line)
                if match4:
                    self.iterations = float(match4.group(1))
                    #print "iteration ",match4.group(1)
                    break
            #end of for loop for reversed lines
            if not(self.job_finished):
                self.StatMsg=self.StatTransl[3]
            else:
                if self.accuracy_reached:
                    self.StatMsg=self.StatTransl[1]
                else:
                    self.StatMsg=self.StatTransl[2]
            #print "In Monitor:", self.StatMsg
        else:
            self.Msgs.append(self.MsgTransl[1])



def has_colours(stream):
    if not hasattr(stream, "isatty"):
        return False
    if not stream.isatty():
        return False # auto color only on TTYs
    try:
        import curses
        curses.setupterm()
        return curses.tigetnum("colors") > 2
    except:
        # guess false in case of error
        return False



#######################################################################
### Get results from an exclusive calculation
class ExclusiveRes:
    def __init__(self,path,hists):
        self.path=path
        self.histnames=hists
        self.totctr={'LO': {'mtinfty': ['sigLOt'], 'top': ['sigLOt'], 'topbottom': ['sigLOt','dsigLOtb']},
                     'NLO': {'mtinfty': ['sigLOt','dsigNLOinfty'], 'top': ['sigLOt','dsigNLOinfty','dsigNLOt'],
                             'topbottom': ['sigLOt','dsigLOtb','dsigNLOinfty','dsigNLOt','dsigNLOtb']},
                     'NNLO': {'mtinfty': ['sigLOt','dsigNLOinfty','dsigNNLOinfty'],
                              'top': ['sigLOt','dsigNLOinfty','dsigNLOt','dsigNNLOinfty'],
                              'topbottom': ['sigLOt','dsigLOtb','dsigNLOinfty','dsigNLOt','dsigNLOtb','dsigNNLOinfty']}}
                     
        self.ctr={'sigLOt': {1: [1]}, 'dsigLOtb': {1: [4]}, 'dsigNLOinfty': {1: [2]}, 'dsigNLOt': {1: [5]},
                  'dsigNLOtb': {1: [6]}, 'dsigNNLOinfty': {1: [3], 2: range(1,22), 3: range(1,17),
                                        5: range(1,49), 8: range(0,1), 9: range(1,3),
                                        10: range(1,3), 12: range(1,2), 13: range(1,2),14: range(1,2)}}
        self.order={'sigLOt': ['LO','NLO','NNLO'], 'dsigLOtb': ['LO','NLO','NNLO'],
                    'dsigNLOinfty': ['NLO','NNLO'], 'dsigNLOt': ['NLO','NNLO'],
                    'dsigNLOtb': ['NLO','NNLO'], 'dsigNNLOinfty': ['NNLO']}
        self.res_sector={'LO': {}, 'NLO': {} ,'NNLO': {}}
        self.sig={'sigLOt': {'LO': 0e0,'NLO': 0e0,'NNLO': 0e0},
                  'dsigLOtb': {'LO': 0e0,'NLO': 0e0,'NNLO': 0e0},
                  'dsigNLOinfty': {'NLO': 0e0,'NNLO': 0e0}, 'dsigNLOt': {'NLO': 0e0,'NNLO': 0e0},
                  'dsigNLOtb': {'NLO': 0e0,'NNLO': 0e0}, 'dsigNNLOinfty': {'NNLO': 0e0}}
        self.err_sq={'sigLOt': {'LO': 0e0,'NLO': 0e0,'NNLO': 0e0},
                  'dsigLOtb': {'LO': 0e0,'NLO': 0e0,'NNLO': 0e0},
                  'dsigNLOinfty': {'NLO': 0e0,'NNLO': 0e0}, 'dsigNLOt': {'NLO': 0e0,'NNLO': 0e0},
                  'dsigNLOtb': {'NLO': 0e0,'NNLO': 0e0}, 'dsigNNLOinfty': {'NNLO': 0e0}}
        self.err={'sigLOt': {'LO': 0e0,'NLO': 0e0,'NNLO': 0e0},
                  'dsigLOtb': {'LO': 0e0,'NLO': 0e0,'NNLO': 0e0},
                  'dsigNLOinfty': {'NLO': 0e0,'NNLO': 0e0}, 'dsigNLOt': {'NLO': 0e0,'NNLO': 0e0},
                  'dsigNLOtb': {'NLO': 0e0,'NNLO': 0e0}, 'dsigNNLOinfty': {'NNLO': 0e0}}
        self.hist={'sigLOt': {'LO': {},'NLO': {},'NNLO': {}},
                  'dsigLOtb': {'LO': {},'NLO': {},'NNLO': {}},
                  'dsigNLOinfty': {'NLO': {},'NNLO': {}}, 'dsigNLOt': {'NLO': {},'NNLO': {}},
                  'dsigNLOtb': {'NLO': {},'NNLO': {}}, 'dsigNNLOinfty': {'NNLO': {}}}
        self.xsecPDF={'sigLOt': {'LO': [],'NLO': [],'NNLO': []},
                  'dsigLOtb': {'LO': [],'NLO': [],'NNLO': []},
                  'dsigNLOinfty': {'NLO': [],'NNLO': []}, 'dsigNLOt': {'NLO': [],'NNLO': []},
                  'dsigNLOtb': {'NLO': [],'NNLO': []}, 'dsigNNLOinfty': {'NNLO': []}}

        self.c0=0e0
        self.c1=0e0
        self.c2=0e0
        self.pi=math.pi
        self.muR=0e0
        self.mhiggs=0e0
        self.mtop=0e0
        self.mbottom=0e0
        self.muF=0e0
        self.as={'LO': 0e0, 'NLO': 0e0, 'NNLO': 0e0}
        self.header={'LO': [], 'NLO': [], 'NNLO': []}
        
    def GetSector(self,order,cont,sect):
        self.res_sector[order][cont,sect]={}
        self.res_sector[order][cont,sect]['Msg']=[]
        self.res_sector[order][cont,sect]['Stat']=0
        self.res_sector[order][cont,sect]['neval']=0
        #thissec=os.path.basename(self.path)
        secpath='sector_%s_%s' %(cont,sect)
        thispath=os.path.join(self.path,secpath)
        #logfile=secpath+'.txt'
        logfile = 'job_output.txt'
        logpath=os.path.join(thispath,logfile)
        MyInfo=Monitor(logpath)
        self.res_sector[order][cont,sect]['neval']=MyInfo.Neval
        self.res_sector[order][cont,sect]['Stat']=MyInfo.StatMsg
        self.res_sector[order][cont,sect]['Msg']=MyInfo.Msgs
        thisfile='%s.result.txt' %(order)
        thisresult=FileReader(thispath,thisfile)
        thisresult.GetRes()
                
        if not (thisresult.Found):
            self.res_sector[order][cont,sect]['Msg'].append('No result file found '+ thisfile)
        else:
            output="* \tsector "+str(cont)+" - "+str(sect)+"\t:\t"+str(MyInfo.StatMsg)+"\tafter "+str(MyInfo.Neval)+" evaluations / "+str(MyInfo.error)+str(MyInfo.sigma)+" | chi_2="+str(MyInfo.chi_2)+"/"+str(MyInfo.iterations)
            color_code=""
            if (MyInfo.chi_2/MyInfo.iterations)>1.0:   
                color_code="\x1b[01;31m"
                red_blink="\x1b[01;05;30;31m"
            if  has_colours(sys.stdout):
                sys.stdout.write(color_code +output + "\x1b[00m")
            else:
                sys.stdout.write(output)
            print ""
        self.res_sector[order][cont,sect]['Res']=thisresult

 
    def Loop(self):
         for key in self.ctr.keys():           
            for korder in self.order[key]:
                sigtot=0e0
                errtot=0e0
                xsecPDF=[]
                isFirst=True
                tmphist={}
                #print "key = ",key,"   in   ",self.ctr.keys()
                #print "korder=",korder
                for cont in self.ctr[key].keys():
                    #print "cont=",cont
                    for sect in self.ctr[key][cont]:
                        #print "now in sector ",cont,"_",sect
                        self.GetSector(korder,cont,sect)
                        self.as[korder]=self.res_sector[korder][cont,sect]['Res'].as
                        self.muR=self.res_sector[korder][cont,sect]['Res'].muR
                        self.mtop=self.res_sector[korder][cont,sect]['Res'].mtop
                        self.mbottom=self.res_sector[korder][cont,sect]['Res'].mbot
                        self.mhiggs=self.res_sector[korder][cont,sect]['Res'].mhiggs
                        self.header[korder]=self.res_sector[korder][cont,sect]['Res'].header
                        sigtot=sigtot+self.res_sector[korder][cont,sect]['Res'].Val
                        errtot=errtot+self.res_sector[korder][cont,sect]['Res'].Err**2
                        thisxsecPDF=self.res_sector[korder][cont,sect]['Res'].xsecPDF
                        for k in range(0,len(thisxsecPDF)):
                            if isFirst:
                                xsecPDF.append(thisxsecPDF[k])
                            else:
                                xsecPDF[k]=xsecPDF[k]+thisxsecPDF[k]
                        for dist in self.histnames:
                            if isFirst:
                                tmphist[dist]={}
                                self.hist[key][korder][dist]=[]
                            #print "ooo ooo : ",self.res_sector[korder][cont,sect]['Res'].Hist.keys()
                            #print "* * * : ", self.histnames
                            for xxx in self.res_sector[korder][cont,sect]['Res'].Hist[dist]:
                                if isFirst:
                                    tmphist[dist][xxx[0]]=[0.0,0.0]
                                tmphist[dist][xxx[0]][0]=tmphist[dist][xxx[0]][0]+xxx[1]
                                tmphist[dist][xxx[0]][1]=tmphist[dist][xxx[0]][1]+xxx[2]**2
                        isFirst=False
                self.xsecPDF[key][korder]=xsecPDF
                for dist in self.histnames:
                    for kkk in tmphist[dist].keys():
                        self.hist[key][korder][dist].append([kkk,tmphist[dist][kkk][0],tmphist[dist][kkk][1]])
                    self.hist[key][korder][dist].sort()
                self.sig[key][korder]=sigtot
                self.err_sq[key][korder]=errtot
       
       
    def GetResult(self):
        self.Loop()
        
    def DefineWilsonCoeff(self,c0,c1,c2):
        self.c0=c0
        self.c1=c1
        self.c2=c2

    def Sig(self,order,ctr):
        Lr=math.log((self.muR/self.mhiggs)**2)
        Lt=math.log((self.mhiggs/self.mtop)**2)
        Nf=5e0
        as=self.as[order]
        c1q=as/self.pi*11e0/4e0
        c2q=(as/self.pi)**2*(2777e0/288e0+19e0/16e0*(Lr+Lt)+Nf*(-67e0/96e0 +1e0/3e0*(Lr+Lt)))
        c0=self.c0
        c1=as/self.pi*self.c1
        c2=(as/self.pi)**2*self.c2
        
        coeff={}
        coeff['sigLOt']=(c0+1e0)**2+(-(2*(1e0+c0))*(c1q*c0-c1))+((-4*c1*c0*c1q+2*c2-2*c2q*c0+3*c0**2*c1q**2+2*c0*c1q**2-2*c1q*c1+2*c2*c0+c1**2-2*c0**2*c2q))
        coeff['dsigNLOinfty']=(1e0+c0)**2+(-(2*(1e0+c0))*(c1q*c0-c1))
        coeff['dsigNNLOinfty']=(1e0+c0)**2
        coeff['dsigLOtb']=1e0
        coeff['dsigNLOt']=1e0
        coeff['dsigNLOtb']=1e0
        sig=0e0
        for cc in self.totctr[order][ctr]:
            sig=sig+coeff[cc]*self.sig[cc][order]
        return sig

    def As(self,order):
        return self.as[order]
        
    def Err(self,order,ctr):
        Lr=math.log((self.muR/self.mhiggs)**2)
        Lt=math.log((self.mhiggs/self.mtop)**2)
        Nf=5e0
        as=self.as[order]
        c1q=as/self.pi*11e0/4e0
        c2q=(as/self.pi)**2*(2777e0/288e0+19e0/16e0*(Lr+Lt)+Nf*(-67e0/96e0 +1e0/3e0*(Lr+Lt)))
        c0=self.c0
        c1=as/self.pi*self.c1
        c2=(as/self.pi)**2*self.c2
        
        coeff={}
        coeff['sigLOt']=(c0+1e0)**2+(-(2*(1e0+c0))*(c1q*c0-c1))+((-4*c1*c0*c1q+2*c2-2*c2q*c0+3*c0**2*c1q**2+2*c0*c1q**2-2*c1q*c1+2*c2*c0+c1**2-2*c0**2*c2q))
        coeff['dsigNLOinfty']=(1e0+c0)**2+(-(2*(1e0+c0))*(c1q*c0-c1))
        coeff['dsigNNLOinfty']=(1e0+c0)**2
        coeff['dsigLOtb']=1e0
        coeff['dsigNLOt']=1e0
        coeff['dsigNLOtb']=1e0
        err=0e0
        for cc in self.totctr[order][ctr]:
            err=err+coeff[cc]**2*self.err_sq[cc][order]
        return math.sqrt(err)

    def Hist(self,dist,order,ctr):
        Lr=math.log((self.muR/self.mhiggs)**2)
        Lt=math.log((self.mhiggs/self.mtop)**2)
        Nf=5e0
        as=self.as[order]
        c1q=as/self.pi*11e0/4e0
        c2q=(as/self.pi)**2*(2777e0/288e0+19e0/16e0*(Lr+Lt)+Nf*(-67e0/96e0 +1e0/3e0*(Lr+Lt)))
        c0=self.c0
        c1=as/self.pi*self.c1
        c2=(as/self.pi)**2*self.c2
        
        coeff={}
        coeff['sigLOt']=(c0+1e0)**2+(-(2*(1e0+c0))*(c1q*c0-c1))+((-4*c1*c0*c1q+2*c2-2*c2q*c0+3*c0**2*c1q**2+2*c0*c1q**2-2*c1q*c1+2*c2*c0+c1**2-2*c0**2*c2q))
        coeff['dsigNLOinfty']=(1e0+c0)**2+(-(2*(1e0+c0))*(c1q*c0-c1))
        coeff['dsigNNLOinfty']=(1e0+c0)**2
        coeff['dsigLOtb']=1e0
        coeff['dsigNLOt']=1e0
        coeff['dsigNLOtb']=1e0
        thishist=[]
        isfirst=True
        for cc in self.totctr[order][ctr]:
            for k in range(0,len(self.hist[cc][order][dist])):
                xxx=self.hist[cc][order][dist][k]
                if isfirst:
                    thishist.append([xxx[0],coeff[cc]*xxx[1],(coeff[cc]**2*xxx[2])])
                else:
                    thishist[k]=[thishist[k][0],thishist[k][1]+coeff[cc]*xxx[1],thishist[k][2]+(coeff[cc]**2*xxx[2])]
            isfirst=False
        for k in range(0,len(thishist)):
            thishist[k]=[thishist[k][0],thishist[k][1],math.sqrt(thishist[k][2])]
        return thishist

    def PDFerror(self,order,ctr):
        Lr=math.log((self.muR/self.mhiggs)**2)
        Lt=math.log((self.mhiggs/self.mtop)**2)
        Nf=5e0
        as=self.as[order]
        c1q=as/self.pi*11e0/4e0
        c2q=(as/self.pi)**2*(2777e0/288e0+19e0/16e0*(Lr+Lt)+Nf*(-67e0/96e0 +1e0/3e0*(Lr+Lt)))
        c0=self.c0
        c1=as/self.pi*self.c1
        c2=(as/self.pi)**2*self.c2
        
        coeff={}
        coeff['sigLOt']=(c0+1e0)**2+(-(2*(1e0+c0))*(c1q*c0-c1))+((-4*c1*c0*c1q+2*c2-2*c2q*c0+3*c0**2*c1q**2+2*c0*c1q**2-2*c1q*c1+2*c2*c0+c1**2-2*c0**2*c2q))
        coeff['dsigNLOinfty']=(1e0+c0)**2+(-(2*(1e0+c0))*(c1q*c0-c1))
        coeff['dsigNNLOinfty']=(1e0+c0)**2
        coeff['dsigLOtb']=1e0
        coeff['dsigNLOt']=1e0
        coeff['dsigNLOtb']=1e0
        pdferror=0.0
        pdferrorP=0.0
        pdferrorM=0.0
        for cc in self.totctr[order][ctr]:
            xsecPDF=self.xsecPDF[cc][order]
            if (len(xsecPDF)>2):
                pdferror=pdferror+coeff[cc]**2*(xsecPDF[1]-xsecPDF[0])**2
                for j in range(1,len(xsecPDF)/2):
                    pdferror=pdferror+coeff[cc]**2*(xsecPDF[2*j]-xsecPDF[2*j-1])**2
                    pdferrorP=pdferrorP+coeff[cc]**2*max(xsecPDF[2*j]-xsecPDF[0],xsecPDF[2*j-1]-xsecPDF[0],0.0)**2
                    pdferrorM=pdferrorM+coeff[cc]**2*max(-xsecPDF[2*j]+xsecPDF[0],-xsecPDF[2*j-1]+xsecPDF[0],0.0)**2
        pdferror=0.5e0*math.sqrt(pdferror)
        pdferrorP=math.sqrt(pdferrorP)
        pdferrorM=math.sqrt(pdferrorM)
        return [pdferror,pdferrorP,pdferrorM]
        
    def Save(self,fname):
        dirname=os.path.dirname(fname)
        if (len(dirname)>0):
            if not os.path.exists(dirname):
                os.makedirs(dirname)
        fid = open(fname,'w')
        pickle.dump(self,fid)
        fid.close()

    def Load(self,fname):
        fid = open(fname,'r')
        selfloaded=pickle.load(fid)
        fid.close()
        self.path=selfloaded.path
        self.histnames=selfloaded.histnames
        self.totctr=selfloaded.totctr
        self.ctr=selfloaded.ctr           
        self.order=selfloaded.order
        self.res_sector=selfloaded.res_sector
        self.sig=selfloaded.sig
        self.err_sq=selfloaded.err_sq
        self.err=selfloaded.err
        self.hist=selfloaded.hist
        self.xsecPDF=selfloaded.xsecPDF
        self.c0=selfloaded.c0
        self.c1=selfloaded.c1
        self.c2=selfloaded.c2
        self.pi=selfloaded.pi
        self.muR=selfloaded.muR
        self.mhiggs=selfloaded.mhiggs
        self.mtop=selfloaded.mtop
        self.mbottom=selfloaded.mbottom
        self.muF=selfloaded.muF
        self.as=selfloaded.as
        self.header=selfloaded.header

    def Write(self,order,ctr,fname):
        dirname=os.path.dirname(fname)
        if (len(dirname)>0):
            if not os.path.exists(dirname):
                os.makedirs(dirname)
        fid=open(fname,'w')
        dt = datetime.now()
        fid.write(dt.strftime("%A, %d. %B %Y %I:%M %p"))
        for line in self.header[order]:
            fid.write(line)
        fid.write('\n')
        fid.write('Wilson coefficient        = \n')
        fid.write('   1 + C0 + as/pi*(C1q+C1w+C1) + (as/pi)**2*(C2q+C2w+C2) \n')
        fid.write('                       C0 = %s \n' %(self.c0))
        fid.write('                       C1 = %s \n' %(self.c1))
        fid.write('                       C2 = %s \n' %(self.c2))
        fid.write('\n\n')
        fid.write('Approximation:              %s \n' %(ctr))
        fid.write('\n\n')
        sig=self.Sig(order,ctr)
        err=self.Err(order,ctr)
        [pdferror,pdferrorP,pdferrorM]=self.PDFerror(order,ctr)
        allhists={}
        for dist in self.histnames:
            allhists[dist]=self.Hist(dist,order,ctr)
        fid.write(' Sigma                    =     %15.6F  fb \n' %(sig))
        fid.write(' Error                    = +/- %15.6F  fb \n' %(err))
        fid.write('\n')
        fid.write(' Symmetric PDF error      = +/- %7.4F %% \n' %(pdferror/max(sig,1e-9)*100e0))
        fid.write(' Assymmetric PDF error    = + %7.4F %% / - %7.4F %% \n' %(pdferrorP/max(sig,1e-9)*100e0,pdferrorM/max(sig,1e-9)*100e0))
        fid.write('\n\n')
        fid.write('==================\n')
        fid.write('= Histograms     =\n')
        fid.write('==================\n')
        fid.write('\n')
        for dist in self.histnames:
            fid.write('\n')
            fid.write('--- %s ---\n' %(dist))
            fid.write('\n')
            for xxx in allhists[dist]:
                fid.write('%7.4F \t %15.6F \t %15.6F\n' %(xxx[0],xxx[1],xxx[2]))
        fid.close()
        #for dist in self.histnames:
        #    f2=open(fname+str(dist)+"_data.txt",'w')
        #    for xxx in allhists[dist]:
        #        f2.write('%7.4F \t %15.6F \t %15.6F\n' %(xxx[0],xxx[1],xxx[2]))
        #    f2.close()

    def Print(self,fname):
        None

    def Plot(self):
        None

#######################################################################
### Get results from an inclusive calculation
class InclusiveRes:
    def __init__(self,path):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        self.path=path
        self.order=['LO','NLO','NNLO']
        self.res_sector={'LO': {}, 'NLO': {} ,'NNLO': {}}
        self.sig={'LO': 0e0, 'NLO': 0e0 ,'NNLO': 0e0}
        self.err_sq={'LO': 0e0, 'NLO': 0e0 ,'NNLO': 0e0}
        self.err={'LO': 0e0, 'NLO': 0e0 ,'NNLO': 0e0}
        self.xsecPDF={'LO': 0e0, 'NLO': 0e0 ,'NNLO': 0e0}
        self.muR=0e0
        self.mhiggs=0e0
        self.mtop=0e0
        self.mbottom=0e0
        self.muF=0e0
        self.as={'LO': 0e0, 'NLO': 0e0, 'NNLO': 0e0}
        self.header={'LO': [], 'NLO': [], 'NNLO': []}
    def GetSector(self,order):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        self.res_sector[order]['Msg']=[]
        self.res_sector[order]['Stat']=0
        self.res_sector[order]['neval']=0
        thisfile=os.path.basename(self.path)
        thispath=self.path
        logfile=thisfile+'.txt'
        logpath=os.path.join(thispath,logfile)
        MyInfo=Monitor(logpath)
        self.res_sector[order]['neval']=MyInfo.Neval
        self.res_sector[order]['Stat']=MyInfo.StatMsg
        self.res_sector[order]['Msg']=MyInfo.Msgs
        resfile='%s.result.txt' %(order)
        thisresult=FileReader(thispath,resfile)
        thisresult.GetRes()        
        if not (thisresult.Found):
            print 'No result file found for: ' + resfile
            self.res_sector[order]['Msg'].append('No result file found for: ' +  resfile)
        self.res_sector[order]['Res']=thisresult
  
    def Loop(self):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        for korder in self.order:
            sigtot=0e0
            errtot=0e0
            xsecPDF=[]
            tmphist={}
            self.GetSector(korder)
            self.as[korder]=self.res_sector[korder]['Res'].as
            self.muR=self.res_sector[korder]['Res'].muR
            self.mtop=self.res_sector[korder]['Res'].mtop
            self.mbottom=self.res_sector[korder]['Res'].mbot
            self.mhiggs=self.res_sector[korder]['Res'].mhiggs
            self.header[korder]=self.res_sector[korder]['Res'].header
            sigtot=sigtot+self.res_sector[korder]['Res'].Val
            errtot=errtot+self.res_sector[korder]['Res'].Err**2
            thisxsecPDF=self.res_sector[korder]['Res'].xsecPDF
            for k in range(0,len(thisxsecPDF)):
                xsecPDF.append(thisxsecPDF[k])
            self.xsecPDF[korder]=xsecPDF
            self.sig[korder]=sigtot
            self.err_sq[korder]=errtot
            #print self.sig[korder]
       
    def GetResult(self):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        self.Loop()
        
    def Sig(self,order):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        return self.sig[order]

    def Err(self,order):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        return math.sqrt(self.err_sq[order])

    def As(self,order):
        return self.as[order]
        
    def PDFerror(self,order):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        pdferror=0.0
        pdferrorP=0.0
        pdferrorM=0.0
        xsecPDF=self.xsecPDF[order]
        #counter=0
        #for zz in xsecPDF: 
        #    print counter,zz
        #    counter = counter+1
        if (len(xsecPDF)>2):
            pdferror=pdferror+(xsecPDF[1]-xsecPDF[0])**2
            #print "*****"
            #print 0,(xsecPDF[1]-xsecPDF[0])**2
            for j in range(1,len(xsecPDF)/2+1):
                pdferror=pdferror+(xsecPDF[2*j]-xsecPDF[2*j-1])**2
                pdferrorP=pdferrorP+max(xsecPDF[2*j]-xsecPDF[0],xsecPDF[2*j-1]-xsecPDF[0],0.0)**2
                pdferrorM=pdferrorM+max(-xsecPDF[2*j]+xsecPDF[0],-xsecPDF[2*j-1]+xsecPDF[0],0.0)**2
                #print 2*j,max(xsecPDF[2*j]-xsecPDF[0],xsecPDF[2*j-1]-xsecPDF[0],0.0),max(-xsecPDF[2*j]+xsecPDF[0],-xsecPDF[2*j-1]+xsecPDF[0],0.0)
        pdferror=0.5e0*math.sqrt(pdferror)
        pdferrorP=math.sqrt(pdferrorP)
        pdferrorM=math.sqrt(pdferrorM)
        return [pdferror,pdferrorP,pdferrorM]
        
    def Save(self,fname):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        dirname=os.path.dirname(fname)
        if (len(dirname)>0):
            if not os.path.exists(dirname):
                os.makedirs(dirname)
        fid = open(fname,'w')
        pickle.dump(self,fid)
        fid.close()

    def Load(self,fname):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        fid = open(fname,'r')
        selfloaded=pickle.load(fid)
        fid.close()
        self.path=selfloaded.path
        self.order=selfloaded.order
        self.res_sector=selfloaded.res_sector
        self.sig=selfloaded.sig
        self.err_sq=selfloaded.err_sq
        self.err=selfloaded.err
        self.xsecPDF=selfloaded.xsecPDF
        self.muR=selfloaded.muR
        self.mhiggs=selfloaded.mhiggs
        self.mtop=selfloaded.mtop
        self.mbottom=selfloaded.mbottom
        self.muF=selfloaded.muF
        self.as=selfloaded.as
        self.header=selfloaded.header

    def Write(self,order,fname):
        #print 'InclusiveRes::',sys._getframe().f_code.co_name
        dirname=os.path.dirname(fname)
        if (len(dirname)>0):
            if not os.path.exists(dirname):
                os.makedirs(dirname)
        fid=open(fname,'w')
        for line in self.header[order]:
            fid.write(line)
        sig=self.Sig(order)
        err=self.Err(order)
        [pdferror,pdferrorP,pdferrorM]=self.PDFerror(order)
        fid.write(' Sigma                    =     %15.6F  fb \n' %(sig))
        fid.write(' Error                    = +/- %15.6F  fb \n' %(err))
        fid.write('\n')
        fid.write(' Symmetric PDF error      = +/- %7.4F %% \n' %(pdferror/max(sig,1e-9)*100e0))
        fid.write(' Assymmetric PDF error    = + %7.4F %% / - %7.4F %% \n' %(pdferrorP/max(sig,1e-9)*100e0,pdferrorM/max(sig,1e-9)*100e0))
        fid.write('\n\n')
        fid.close()
        

    def Print(self,fname):
        None

    def Plot(self):
        None



#######################################################################
### Process data
class ProcessData:
    def __init__(self):
        self.outdata=[]
        #self.data1=data1
        #self.data2=data2

    def Plus(self,data1,data2):
        self.outdata=[]
        for j in range(0,len(data1)):
            var1=data1[j][0]
            sig1=data1[j][1]
            err1=data1[j][2]
            if (len(data2)==len(data1)):
                var2=data2[j][0]
                sig2=data2[j][1]
                err2=data2[j][2]
            elif (len(data2)==1):
                var2=var1
                sig2=data2[0][0]
                err2=data2[0][1]
            else:
                print "Error: Missmatch of arguments!"
                #print data1
                #print data2
                return 
            if (var1==var2):
                self.outdata.append([var1,sig1+sig2,math.sqrt(err1**2+err2**2)])
        return self.outdata

    def Minus(self,data1,data2):
        self.outdata=[]
        print 'Minus: '
        print len(data1)
        print len(data2)
        for j in range(0,len(data1)):
            var1=data1[j][0]
            sig1=data1[j][1]
            err1=data1[j][2]
            if (len(data2)==len(data1)):
                var2=data2[j][0]
                sig2=data2[j][1]
                err2=data2[j][2]
            elif (len(data2)==1):
                var2=var1
                sig2=data2[0][0]
                err2=data2[0][1]
            else:
                print "Error: Missmatch of arguments!"
                #print data1
                #print data2
                return 
            if (var1==var2):
                self.outdata.append([var1,sig1-sig2,math.sqrt(err1**2+err2**2)])
        return self.outdata
    
    def Divide(self,data1,data2):
        self.outdata=[]
        for j in range(0,len(data1)):
            var1=data1[j][0]
            sig1=data1[j][1]
            err1=data1[j][2]
            if (len(data2)==len(data1)):
                var2=data2[j][0]
                sig2=data2[j][1]
                err2=data2[j][2]
            elif (len(data2)==1):
                var2=var1
                sig2=data2[0][0]
                err2=data2[0][1]
            else:
                print "Error: Missmatch of arguments!"
                #print data1
                #print data2
                return 
            if (var1==var2):
                if (sig1==0.0):
                    if (sig2==0.0):
                        k=0.0
                        dk=0.0
                    else:
                        k=0.0
                        dk=math.sqrt(err1**2/sig2**2+err2**2*(sig1/sig2**2)**2)
                else:
                    if (sig2==0.0):
                        k=10e25
                        dk=0.0
                    else:
                        k=sig1/sig2
                        dk=math.sqrt(err1**2/sig2**2+err2**2*(sig1/sig2**2)**2)
                self.outdata.append([var1,k,dk])
        return self.outdata

    def Multiply(self,data1,data2):
        self.outdata=[]
        for j in range(0,len(data1)):
            var1=data1[j][0]
            sig1=data1[j][1]
            err1=data1[j][2]
            if (len(data2)==len(data1)):
                var2=data2[j][0]
                sig2=data2[j][1]
                err2=data2[j][2]
            elif (len(data2)==1):
                var2=var1
                sig2=data2[0][0]
                err2=data2[0][1]
            else:
                print "Error: Missmatch of arguments!"
                #print data1
                #print data2
                return 
            if (var1==var2):
                self.outdata.append([var1,sig1*sig2,math.sqrt((sig2*err1)**2+(sig1*err2)**2)])
        return self.outdata


    def Scale(self,data,scale):
        self.outdata=[]
        for j in range(0,len(data)):
            var1=data[j][0]
            sig1=scale*data[j][1]
            err1=scale*data[j][2]
            self.outdata.append([var1,sig1,err1])
        return self.outdata



#######################################################################
#######################################################################
#######################################################################
### Plotting
class Plotting:
    def __init__(self,pave,pleg,paxis,papp,plotexe,plotfile):
        self.Pave=pave
        self.Legend=pleg
        self.Axis=paxis
        self.Appearance=papp
        self.Plotter=plotexe
        self.Outfile=plotfile
        self.xtitle=self.Axis['xtitle']
        self.ytitle=self.Axis['ytitle']
        if ('xmin' in self.Appearance):
            self.xmin=self.Appearance['xmin']
        if ('xmax' in self.Appearance):   
            self.xmax=self.Appearance['xmax']
        self.plegx1=self.Legend['x1']
        self.plegx2=self.Legend['x2']
        self.plegy1=self.Legend['y1']
        self.plegy2=self.Legend['y2']
        self.title=self.Appearance['title']
        self.minimum=self.Appearance['minimum']
        self.maximum=self.Appearance['maximum']
        self.ilogx=self.Appearance['logx']
        self.ilogy=self.Appearance['logy']
        if ('iDivide' in self.Appearance):
            self.iDivide=self.Appearance['iDivide']
            self.xDivide=self.Appearance['xDivide']
            self.yDivide=self.Appearance['yDivide']
        else:
            self.iDivide=False
            
        self.ifirst=True
        self.ifirst2=True

        ofile=self.Outfile
        plotscript= open(self.Plotter,'w')
        plotscript.write("""\
{    
    gROOT->SetStyle("Pub");
    gROOT->ForceStyle();
    gStyle->SetCanvasBorderMode(0);
    gStyle->SetPadBorderMode(0);
    gStyle->SetFrameBorderMode(0);
    gStyle->SetLegendBorderSize(0.5);
    gStyle->SetFillColor(0);
    gStyle->SetOptTitle(0);
    gStyle->SetOptStat(0);
    gStyle->SetCanvasColor(0);
    gStyle->SetPadColor(0);
    gStyle->SetFrameFillColor(0);
    gStyle->SetTitleFillColor(0);
    gStyle->SetPadBottomMargin(0.18);
    gStyle->SetPadLeftMargin(0.16);
    gStyle->SetLabelOffset(0.015,"xy");
    TCanvas *c1 = new TCanvas("c1","%(ofile)s",1);
    Int_t i;
   
    """ %vars())

        if (self.iDivide):
            xDivide=self.xDivide
            yDivide=self.yDivide
            plotscript.write("""
    c1->Divide(%(xDivide)s,%(yDivide)s,0.003,0.003);
    c1->cd(1);
    
    """ %vars())
        else:
            plotscript.write("""
    c1->cd();
    
    """ %vars())
        plotscript.close()

    def ChangePad(self,ix,iy):
        self.ifirst=True
        self.ifirst2=True
        Ncols=self.xDivide
        Nrows=self.yDivide
        Npads=Ncols*Nrows
        kpad=(iy)*Ncols+ix
        
        plotscript= open(self.Plotter,'a')
        plotscript.write("""
    c1->cd(%(kpad)s);

    """ %vars())
        
    def AddAltPlot(self,options,data,datfile):
        UserDrawOpt=False
        plotscript= open(self.Plotter,'a')
        if ('drawopt' in options):
            drawopt=options['drawopt']
            UserDrawOpt=True
        pl=options['name']
        mstyle=options['mstyle']
        lstyle=options['lstyle']
        acolor=options['color']
        fcolor=options['fcolor']
        fstyle=options['fstyle']
        asize=options['size']
        legentr=options['legentr']
        ftempn='%s.pltmp' %pl
        fid=open(ftempn,'w')
        for dat in data:
            fid.write('%s %s %s\n' %(dat[0],dat[1],dat[2]))
        fid.close()
        cmd4='cp %s %s.txt' %(ftempn,datfile) 
        failure = os.system(cmd4)
        if self.ifirst2:
            
            plotscript.write("""\
        
    TPad *pad2 = new TPad("pad2","",0,0,1,1);
    pad2->SetFillStyle(4000);
    """ %vars())
        plotscript.write("""\
    
    gPad->Update();
    Double_t rightmax = 1.1;
    Double_t scale = (gPad->GetUymax()-gPad->GetUxmax())/rightmax;

    pad2->Draw();
    pad2->cd();
    """ %vars())

        plotscript.write("""
    TGraphErrors *%(pl)s = new TGraphErrors("%(ftempn)s",myformat,myoption);
    %(pl)s->SetMarkerStyle(%(mstyle)s);
    %(pl)s->SetLineStyle(%(lstyle)s);
    %(pl)s->SetMarkerColor(%(acolor)s);
    %(pl)s->SetLineColor(%(acolor)s);
    %(pl)s->SetMarkerSize(%(asize)s);
    %(pl)s->SetFillColor(%(fcolor)s);
    %(pl)s->SetFillStyle(%(fstyle)s);
    """ %vars())

        if (UserDrawOpt):
            plotscript.write("""
    %(pl)s->Draw("%(drawopt)s");
    """ %vars())
        else:
            plotscript.write("""
    %(pl)s->Draw("CP");
    """ %vars())
        plotscript.write("""

    leg->AddEntry(%(pl)s,"%(legentr)s","lp");
    leg->Draw();
    
    gPad->Update();

    Double_t pad1x[2], pad1rax[2];
    Double_t xy[4];
    pad1->GetRange(xy[0],xy[1],xy[2],xy[3]);
    pad1x[0] = xy[0];
    pad1x[1] = xy[2];
    pad1->GetRangeAxis(xy[0],xy[1],xy[2],xy[3]);
    pad1rax[0] =  xy[0];
    pad1rax[1] =  xy[2];

    Double_t ymin = 0;
    Double_t ymax = 1.0;
    Double_t dy = (ymax-ymin)/0.6;
    pad2->Range( pad1x[0], ymin-0.2*dy, pad1x[1], ymax+0.2*dy);
    pad2->RangeAxis( pad1rax[0], ymin, pad1rax[1], ymax);
    pad2->Update();

    TGaxis *axis = new TGaxis(gPad->GetUxmax(),
                              gPad->GetUymin(),
                              gPad->GetUxmax(),
                              gPad->GetUymax(),
                              ymin,ymax,510,"+L");

    axis->SetLineColor(kRed);
    axis->SetLabelColor(kRed);
    axis->Draw("G");

    c1->Update();
    """ %vars())
        plotscript.close()
        self.ifirst2=False
        
    def AddNormalPlot(self,options,data,datfile):
        plotscript= open(self.Plotter,'a')
        if self.ifirst:
            plotscript.write("""\

    char *myoption="";
    char *myformat="%%lg %%lg %%lg";
   
    """ %vars())
        UserDrawOpt=False
        if ('drawopt' in options):
            drawopt=options['drawopt']
            UserDrawOpt=True
        pl=options['name']
        mstyle=options['mstyle']
        lstyle=options['lstyle']
        acolor=options['color']
        fcolor=options['fcolor']
        fstyle=options['fstyle']
        asize=options['size']
        legentr=options['legentr']
        ftempn='%s.pltmp' %pl
        fid=open(ftempn,'w')
        for dat in data:
            if len(dat)==3:
                #print "length 3"
                myformat='char *myformat="%lg %lg %lg"'
                fid.write('%s %s %s\n' %(dat[0],dat[1],dat[2]))
            elif len(dat)==4:
                #print "length 4"
                myformat='char *myformat="%lg %lg %lg %lg"'
                fid.write('%s %s %s %s\n' %(dat[0],dat[1],dat[2],dat[3]))
        fid.close()
        cmd4='cp %s %s.txt' %(ftempn,datfile) 
        failure = os.system(cmd4)

        if (self.iDivide):
            if self.ifirst:
                plotscript.write("""
    TPad *pad1 = new TPad("pad1","",0,0,1,1);
    """ %vars())
            plotscript.write("""    
    pad1->Draw();
    pad1->cd();

    """ %vars())
        plotscript.write("""
    %(myformat)s;
    TGraphErrors *%(pl)s = new TGraphErrors("%(ftempn)s",myformat,myoption);
    %(pl)s->SetMarkerStyle(%(mstyle)s);
    %(pl)s->SetLineStyle(%(lstyle)s);
    %(pl)s->SetMarkerColor(%(acolor)s);
    %(pl)s->SetLineColor(%(acolor)s);
    %(pl)s->SetMarkerSize(%(asize)s);
    %(pl)s->SetFillColor(%(fcolor)s);
    %(pl)s->SetFillStyle(%(fstyle)s);
    """ %vars())
        if (self.ifirst):
            tit=self.title
            xtit=self.xtitle
            ytit=self.ytitle
            minimum=self.minimum
            maximum=self.maximum
            plegx1=self.plegx1
            plegx2=self.plegx2
            plegy1=self.plegy1
            plegy2=self.plegy2
            ilogx=self.ilogx
            ilogy=self.ilogy
            
            plotscript.write("""
    gPad->SetLogx(%(ilogx)s);
    gPad->SetLogy(%(ilogy)s);

    %(pl)s->SetTitle("%(tit)s");
    %(pl)s->SetMinimum(%(minimum)s);
    %(pl)s->SetMaximum(%(maximum)s);
    """ %vars())
            if (UserDrawOpt):
                print 'XX'
                plotscript.write("""
    %(pl)s->Draw("A%(drawopt)s");
    """ %vars())
            else:
                plotscript.write("""
    %(pl)s->Draw("ACP");
    """ %vars())
            plotscript.write("""
    

    TAxis *axisX=%(pl)s->GetXaxis();
    TAxis *axisY=%(pl)s->GetYaxis();
    axisX->SetTitle("%(xtit)s");
    """ %vars())
            fsetrange=1.0
            if ('xmin' in self.Appearance):
                xmin=float(self.xmin)                
                fsetrange=fsetrange-1.0
            if ('xmax' in self.Appearance):
                xmax=float(self.xmax)
                fsetrange=fsetrange-1.0
            if (fsetrange < 0.0):                
                plotscript.write("""
    axisX->SetRangeUser(%(xmin)s,%(xmax)s);
    """ %vars())
            plotscript.write("""
    axisX->Draw();
    axisY->SetTitle("%(ytit)s");
    axisY->Draw();

    TLegend *leg = new TLegend(%(plegx1)s,%(plegy1)s,%(plegx2)s,%(plegy2)s,0,"NDC");
    leg->AddEntry(%(pl)s,"%(legentr)s","lp");
    leg->Draw();
    """ %vars())
            for pkey in self.Pave.keys():
                pavex1=self.Pave[pkey]['x1']
                pavex2=self.Pave[pkey]['x2']
                pavey1=self.Pave[pkey]['y1']
                pavey2=self.Pave[pkey]['y2']
            
                plotscript.write("""
    TPaveText *p%(pkey)s = new TPaveText(%(pavex1)s,%(pavey1)s,%(pavex2)s,%(pavey2)s,"NDC");
    p%(pkey)s->SetBorderSize(0);
    """ %vars())
                for pavet in self.Pave[pkey]['text']:
                    plotscript.write("""
    p%(pkey)s->AddText("%(pavet)s");
    """ %vars())
                plotscript.write("""
    p%(pkey)s->Draw();
    
    c1->Update();
    """ %vars())
        else:
            if (UserDrawOpt):
                plotscript.write("""
    %(pl)s->Draw("%(drawopt)s");
    """ %vars())
            else:
                plotscript.write("""
    %(pl)s->Draw("CP");
    """ %vars())
            plotscript.write("""
    leg->AddEntry(%(pl)s,"%(legentr)s","lp");
    leg->Draw();

    c1->Update();
    """ %vars())
        self.ifirst=False
        plotscript.close()

    def AddBandPlot(self,options,data,datfile):
        plotscript= open(self.Plotter,'a')
        if self.ifirst:
            plotscript.write("""\

    char *myoption="";
    char *myformat="%%lg %%lg %%lg";
    char *myformat2="%%lg %%lg";
   
    """ %vars())
        pl=options['name']
        mstyle=options['mstyle']
        lstyle=options['lstyle']
        acolor=options['color']
        asize=options['size']
        legentr=options['legentr']
        fcolor=options['fcolor']
        fstyle=options['fstyle']
        ftempnC='%s.pltmp' %pl
        fid=open(ftempnC,'w')
        for dat in data['central']:
            fid.write('%s %s %s\n' %(dat[0],dat[1],dat[2]))
        fid.close()
        cmd4='cp %s %s.central.txt' %(ftempnC,datfile) 
        failure = os.system(cmd4)
        ftempnP='%sP.pltmp' %pl
        fid=open(ftempnP,'w')
        for dat in data['plus']:
            fid.write('%s %s \n' %(dat[0],dat[1]))
        fid.close()
        cmd4='cp %s %s.plus.txt' %(ftempnP,datfile) 
        failure = os.system(cmd4)
        ftempnM='%sM.pltmp' %pl
        fid=open(ftempnM,'w')
        for dat in data['minus']:
            fid.write('%s %s \n' %(dat[0],dat[1]))
        fid.close()
        cmd4='cp %s %s.minus.txt' %(ftempnM,datfile) 
        failure = os.system(cmd4)
        plotscript.write("""
    TGraphErrors *%(pl)s = new TGraphErrors("%(ftempnC)s",myformat,myoption);
    %(pl)s->SetMarkerStyle(%(mstyle)s);
    %(pl)s->SetLineStyle(%(lstyle)s);
    %(pl)s->SetMarkerColor(%(acolor)s);
    %(pl)s->SetLineColor(%(acolor)s);
    %(pl)s->SetMarkerSize(%(asize)s);

    TGraph *%(pl)sP = new TGraph("%(ftempnP)s",myformat2,myoption);
    TGraph *%(pl)sM = new TGraph("%(ftempnM)s",myformat2,myoption);
    Int_t %(pl)snpoints=%(pl)s->GetN();
    TGraph *%(pl)sshade = new TGraph(2*%(pl)snpoints);
    Double_t x1%(pl)s,y1%(pl)s,x2%(pl)s,y2%(pl)s;
    for (i=0;i<%(pl)snpoints;i++) {
       %(pl)sP->GetPoint(i,x1%(pl)s,y1%(pl)s);
       %(pl)sM->GetPoint(%(pl)snpoints-i-1,x2%(pl)s,y2%(pl)s);
//       cout << i << ' ' << x1%(pl)s << ' ' << y1%(pl)s << ' ' << x2%(pl)s << ' ' << y2%(pl)s << endl;
       %(pl)sshade->SetPoint(i,x1%(pl)s,y1%(pl)s);
       %(pl)sshade->SetPoint(%(pl)snpoints+i,x2%(pl)s,y2%(pl)s);
    }
        """ %vars())
        if (self.ifirst):
            tit=self.title
            xtit=self.xtitle
            ytit=self.ytitle
            minimum=self.minimum
            maximum=self.maximum
            plegx1=self.plegx1
            plegx2=self.plegx2
            plegy1=self.plegy1
            plegy2=self.plegy2
            ilogx=self.ilogx
            ilogy=self.ilogy
            plotscript.write("""
    %(pl)s->SetTitle("%(tit)s");
    %(pl)s->SetMinimum(%(minimum)s);
    %(pl)s->SetMaximum(%(maximum)s);
    %(pl)s->Draw("ACP");
    c1->SetLogx(%(ilogx)s);
    c1->SetLogy(%(ilogy)s);
    %(pl)sshade->SetFillStyle(%(fstyle)s);
    %(pl)sshade->SetFillColor(%(fcolor)s);
    %(pl)sshade->Draw("lF");
    %(pl)sP->SetLineColor(%(acolor)s);
    %(pl)sP->SetLineWidth(0.1);
    %(pl)sP->Draw("l");
    %(pl)sM->SetLineColor(%(acolor)s);
    %(pl)sM->SetLineWidth(0.1);
    %(pl)sM->Draw("l");

    TAxis *axisX=%(pl)s->GetXaxis();
    TAxis *axisY=%(pl)s->GetYaxis();
    axisX->SetTitle("%(xtit)s");
    axisX->Draw();
    axisY->SetTitle("%(ytit)s");
    axisY->Draw();

    TLegend *leg = new TLegend(%(plegx1)s,%(plegy1)s,%(plegx2)s,%(plegy2)s,0,"NDC");
    leg->AddEntry(%(pl)s,"%(legentr)s","lp");
    leg->Draw();
            """ %vars())
            for pkey in self.Pave.keys():
                pavex1=self.Pave[pkey]['x1']
                pavex2=self.Pave[pkey]['x2']
                pavey1=self.Pave[pkey]['y1']
                pavey2=self.Pave[pkey]['y2']
            
                plotscript.write("""
    TPaveText *p%(pkey)s = new TPaveText(%(pavex1)s,%(pavey1)s,%(pavex2)s,%(pavey2)s,"NDC");
    p%(pkey)s->SetBorderSize(0);
                """ %vars())
                for pavet in self.Pave[pkey]['text']:
                    plotscript.write("""
    p%(pkey)s->AddText("%(pavet)s");
                    """ %vars())
                plotscript.write("""
    p%(pkey)s->Draw();
    
    c1->Update();
                """ %vars())
        else:
            plotscript.write("""
    %(pl)s->Draw("CP");
    %(pl)sshade->SetFillStyle(%(fstyle)s);
    %(pl)sshade->SetFillColor(%(fcolor)s);
    %(pl)sshade->Draw("lF");
    %(pl)sP->SetLineColor(%(acolor)s);
    %(pl)sP->SetLineWidth(0.1);
    %(pl)sP->Draw("l");
    %(pl)sM->SetLineColor(%(acolor)s);
    %(pl)sM->SetLineWidth(0.1);
    %(pl)sM->Draw("l");
    leg->AddEntry(%(pl)s,"%(legentr)s","lp");
    leg->Draw();

    c1->Update();
        """ %vars())
        self.ifirst=False
        plotscript.close()


       


    def AddHistBandPlot(self,options,data,datfile):
        plotscript= open(self.Plotter,'a')
        if self.ifirst:
            plotscript.write("""\

    char *myoption="";
    char *myformat="%%lg %%lg";
   
            """ %vars())
            
        pl=options['name']
        mstyle=options['mstyle']
        lstyle=options['lstyle']
        acolor=options['color']
        asize=options['size']
        legentr=options['legentr']
        fcolor=options['fcolor']
        fstyle=options['fstyle']
        ftempnC='%s.pltmp' %pl
        fid=open(ftempnC,'w')
        for dat in data['central']:
            fid.write('%s %s \n' %(dat[0],dat[1],dat[2]))
        fid.close()
        cmd4='cp %s %s.central.txt' %(ftempnC,datfile) 
        failure = os.system(cmd4)
        ftempnP='%sP.pltmp' %pl
        fid=open(ftempnP,'w')
        for dat in data['plus']:
            fid.write('%s %s \n' %(dat[0],dat[1]))
        fid.close()
        cmd4='cp %s %s.plus.txt' %(ftempnP,datfile) 
        failure = os.system(cmd4)
        ftempnM='%sM.pltmp' %pl
        fid=open(ftempnM,'w')
        for dat in data['minus']:
            fid.write('%s %s \n' %(dat[0],dat[1]))
        fid.close()
        cmd4='cp %s %s.minus.txt' %(ftempnM,datfile) 
        failure = os.system(cmd4)
        plotscript.write("""
    TGraph *%(pl)s = new TGraph("%(ftempnC)s",myformat,myoption);
    %(pl)s->SetMarkerStyle(%(mstyle)s);
    %(pl)s->SetLineStyle(%(lstyle)s);
    %(pl)s->SetMarkerColor(%(acolor)s);
    %(pl)s->SetLineColor(%(acolor)s);
    %(pl)s->SetMarkerSize(%(asize)s);

    TGraph *%(pl)sP = new TGraph("%(ftempnP)s",myformat,myoption);
    TGraph *%(pl)sM = new TGraph("%(ftempnM)s",myformat,myoption);
    Int_t %(pl)snpoints=%(pl)s->GetN();
    %(pl)sP->SetMarkerStyle(2);
    %(pl)sP->SetMarkerColor(%(acolor)s);
    %(pl)sM->SetMarkerStyle(5);
    %(pl)sM->SetMarkerColor(%(acolor)s);
    
    TGraph *%(pl)sshade = new TGraph(4*%(pl)snpoints);
    Double_t x1%(pl)s,y1%(pl)s,x2%(pl)s,y2%(pl)s,x3%(pl)s,y3%(pl)s;

    %(pl)sP->GetPoint(0,x1%(pl)s,y1%(pl)s);
    %(pl)sP->GetPoint(1,x2%(pl)s,y2%(pl)s);
    %(pl)sshade->SetPoint(0,x1%(pl)s-(x2%(pl)s-x1%(pl)s)/2.0,y1%(pl)s);
    %(pl)sshade->SetPoint(1,(x2%(pl)s+x1%(pl)s)/2.0,y1%(pl)s);
    
    %(pl)sP->GetPoint(%(pl)snpoints-2,x1%(pl)s,y1%(pl)s);
    %(pl)sP->GetPoint(%(pl)snpoints-1,x2%(pl)s,y2%(pl)s);
    %(pl)sshade->SetPoint(2*%(pl)snpoints-2 ,(x2%(pl)s+x1%(pl)s)/2.0,y2%(pl)s);
    %(pl)sshade->SetPoint(2*%(pl)snpoints-1,x2%(pl)s+(x2%(pl)s-x1%(pl)s)/2.0,y2%(pl)s);
    

    %(pl)sM->GetPoint(0,x1%(pl)s,y1%(pl)s);
    %(pl)sM->GetPoint(1,x2%(pl)s,y2%(pl)s);
    %(pl)sshade->SetPoint(4*%(pl)snpoints-1,x1%(pl)s-(x2%(pl)s-x1%(pl)s)/2.0,y1%(pl)s);
    %(pl)sshade->SetPoint(4*%(pl)snpoints-2,(x2%(pl)s+x1%(pl)s)/2.0,y1%(pl)s);

    %(pl)sM->GetPoint(%(pl)snpoints-2,x1%(pl)s,y1%(pl)s);
    %(pl)sM->GetPoint(%(pl)snpoints-1,x2%(pl)s,y2%(pl)s);
    %(pl)sshade->SetPoint(2*%(pl)snpoints+1 ,(x2%(pl)s+x1%(pl)s)/2.0,y2%(pl)s);
    %(pl)sshade->SetPoint(2*%(pl)snpoints,x2%(pl)s+(x2%(pl)s-x1%(pl)s)/2.0,y2%(pl)s);
    
    for (i=1;i<%(pl)snpoints-1;i++) {
       %(pl)sP->GetPoint(i-1,x1%(pl)s,y1%(pl)s);
       %(pl)sP->GetPoint(i,x2%(pl)s,y2%(pl)s);
       %(pl)sP->GetPoint(i+1,x3%(pl)s,y3%(pl)s);
       %(pl)sshade->SetPoint(2*i,(x1%(pl)s+x2%(pl)s)/2.0,y2%(pl)s);
       %(pl)sshade->SetPoint(2*i+1,(x2%(pl)s+x3%(pl)s)/2.0,y2%(pl)s);

       %(pl)sM->GetPoint(i-1,x1%(pl)s,y1%(pl)s);
       %(pl)sM->GetPoint(i,x2%(pl)s,y2%(pl)s);
       %(pl)sM->GetPoint(i+1,x3%(pl)s,y3%(pl)s);
       %(pl)sshade->SetPoint(4*%(pl)snpoints-2*i-1,(x1%(pl)s+x2%(pl)s)/2.0,y2%(pl)s);
       %(pl)sshade->SetPoint(4*%(pl)snpoints -(2*i+2),(x2%(pl)s+x3%(pl)s)/2.0,y2%(pl)s);
    }
//    cout << " ************ Check " << %(pl)snpoints << endl;
//    for (i=0;i<4*%(pl)snpoints;i++) {
//       %(pl)sshade->GetPoint(i,x1%(pl)s,y1%(pl)s);
//       cout << " " << i << "   " << x1%(pl)s << " / " << y1%(pl)s <<  endl;
//       
//    }
        """ %vars())
        if (self.ifirst):
            tit=self.title
            xtit=self.xtitle
            ytit=self.ytitle
            minimum=self.minimum
            maximum=self.maximum
            plegx1=self.plegx1
            plegx2=self.plegx2
            plegy1=self.plegy1
            plegy2=self.plegy2
            ilogx=self.ilogx
            ilogy=self.ilogy
            plotscript.write("""
    %(pl)s->SetTitle("%(tit)s");
    %(pl)s->SetMinimum(%(minimum)s);
    %(pl)s->SetMaximum(%(maximum)s);
    %(pl)s->Draw("AP");
    c1->SetLogx(%(ilogx)s);
    c1->SetLogy(%(ilogy)s);
    %(pl)sshade->SetFillStyle(%(fstyle)s);
    %(pl)sshade->SetFillColor(%(fcolor)s);
    %(pl)sshade->Draw("LF");
    //%(pl)sP->SetMarkerColor(%(acolor)s);
    //%(pl)sP->SetLineColor(%(acolor)s);
    //%(pl)sP->SetLineWidth(0.5);
    //%(pl)sP->Draw("L");
    //%(pl)sM->SetMarkerColor(%(acolor)s);
    //%(pl)sM->SetLineColor(%(acolor)s);
    //%(pl)sM->SetLineWidth(0.5);
    //%(pl)sM->Draw("L");

    TAxis *axisX=%(pl)s->GetXaxis();
    TAxis *axisY=%(pl)s->GetYaxis();
    axisX->SetTitle("%(xtit)s");
    axisX->Draw();
    axisY->SetTitle("%(ytit)s");
    axisY->Draw();

    TLegend *leg = new TLegend(%(plegx1)s,%(plegy1)s,%(plegx2)s,%(plegy2)s,0,"NDC");
    leg->AddEntry(%(pl)s,"%(legentr)s","p");
    leg->Draw();

    
        """ %vars())
            for pkey in self.Pave.keys():
                pavex1=self.Pave[pkey]['x1']
                pavex2=self.Pave[pkey]['x2']
                pavey1=self.Pave[pkey]['y1']
                pavey2=self.Pave[pkey]['y2']
            
                plotscript.write("""
    TPaveText *p%(pkey)s = new TPaveText(%(pavex1)s,%(pavey1)s,%(pavex2)s,%(pavey2)s,"NDC");
    p%(pkey)s->SetBorderSize(0);
                """ %vars())
                for pavet in self.Pave[pkey]['text']:
                    plotscript.write("""
    p%(pkey)s->AddText("%(pavet)s");
                    """ %vars())
                plotscript.write("""
    p%(pkey)s->Draw();
    
    c1->Update();
                """ %vars())
        else:
            plotscript.write("""
    %(pl)s->Draw("P");
    %(pl)sshade->SetFillStyle(%(fstyle)s);
    %(pl)sshade->SetFillColor(%(fcolor)s);
    %(pl)sshade->Draw("LF");
    //%(pl)sP->SetMarkerColor(%(acolor)s);
    //%(pl)sP->SetLineColor(%(acolor)s);
    //%(pl)sP->SetLineWidth(0.5);
    //%(pl)sP->Draw("L");
    //%(pl)sM->SetMarkerColor(%(acolor)s);
    //%(pl)sM->SetLineColor(%(acolor)s);
    //%(pl)sM->SetLineWidth(0.5);
    //%(pl)sM->Draw("L");
    leg->AddEntry(%(pl)s,"%(legentr)s","p");
    leg->Draw();

    c1->Update();
        """ %vars())


        self.ifirst=False
        plotscript.close()




    def Final(self,mydir):
        plotscript= open(self.Plotter,'a')
        ofile=self.Outfile
        plotscript.write("""
    c1->Print("%(ofile)s");
}
        """ %vars())
        plotscript.close()
        cmd='root -b -q %s' %(self.Plotter)
        failure = os.system(cmd)
        if failure:
            print 'Execution of root failed... abort'
            return 1
        cmd2='rm -f *.pltmp *.C'
        failure = os.system(cmd2)
        if failure:
            print 'Removing of temporary files failed... Attention!'
        #cmd3='mv %s ./plots/' %self.Outfile
        cmd3='mv %s %s' %(self.Outfile,mydir)
        failure = os.system(cmd3)
        if failure:
            print 'Moving of %s to ./plots failed... Attention!' %self.Outfile
        cmd4='mv *.txt %s/txt/' %(mydir) 
        failure = os.system(cmd4)
        if failure:
            print 'Moving of *.txt to ./plots failed... Attention!' 
        return 0

            
