#!/usr/bin/env python

import sys, math, os, commands
import re,time,shutil,copy
import ClassesCollect
import string


######################################################################
### Initialize FeHiPro Project
class FeHiProProject:
#---------------------------------------------------------------------
    def __init__(self):
        print "............................................ fehipro\n"
        #.self.mass_scan=mass_scan
        #.self.factorization_scale_scan=factorization_scale_scan
        #.self.myfixed=myfixed
        #.self.jobdefs=jobdefs
        # setting up the initial suffix
        #print 'in __init__, jobdefs[\'project\'] = ',jobdefs['projdir']+"/"+jobdefs['project'] 
        #.self.fehip_mode='run'
        self.myfixed={}
        self.jobdefs={}
        self.cuts={}
        self.SetDefaults()
        # checks if collect is passed as an argument
        self.ReadCommandLineArg()
        self.input_filename="runcard"
        self.ReadInputFile()
        self.ExtendMyFixed()
        # overwritting the default values with the user input values
        # in self.myfixed
        for key in self.myfixed.keys():
            self.param[key]=self.myfixed[key]
        for key in self.cuts.keys():
            self.param[key]=self.cuts[key]
        if self.jobdefs['fehip_mode']=='run':
            self.SetUpDir()
            if self.param['TevatronSpecialScan']=='true' and self.myfixed['type']=='inclusive':
                self.DoLoopOverScanVariablesTevatronSpecial()
            else:
                self.DoLoopOverScanVariables()
        elif self.jobdefs['fehip_mode']=='collect':
            if self.myfixed['type'] == 'inclusive':
                if self.param['TevatronSpecialScan']=='true':
                    self.CollectInclusiveSpecialTevatron()
                else:
                    self.CollectInclusive()
            else:
                self.CollectExclusive()
        else:
            print "Unrecognized fehip_mode :",self.jobdefs['fehip_mode']
        print "\n............................................ exiting fehipro"
#---------------------------------------------------------------------
    def mypartition(self,mystring,delimeter):
        pos=mystring.find(delimeter)
        if pos == -1:
            return( (mystring,"",""))
        else:
            return((mystring[0:pos],delimeter,mystring[pos+1:]))
#---------------------------------------------------------------------
    def ReadInputFile(self):
        lines = file(self.input_filename).readlines()
        print "reading ",self.input_filename
        for line in lines:
            #line.substitute('#.','')
            line=self.mypartition(line,"#")[0]
            line=self.mypartition(line,"=")
            line=map(string.strip,line)
            #
            #reading myfixed parameters
            #
            possible_myfixed_flags = ['perturbative_order','type','massive_corrections','electroweak_corrections',
                'muFrel','higgs_width_flag','collider','sqrtS','decay','pdfset','calculate_pdf_error',
                'vary_a_s','accRel','accAbs','ntherm','ncalls','nincr','WilsonFile','factorization_scale',
                'factorization_scale_scan','factorization_scale_low_end','factorization_scale_high_end',
                'factorization_scale_scan_step','m_higgs',
                'mass_scan','mass_scan_low_end','mass_scan_high_end','mass_scan_step']
            for key in possible_myfixed_flags:
                if key==line[0]:
                    self.myfixed[key]=line[2]
                    #print "myfixed parameter ",key," set to ",self.myfixed[key]
            #
            #reading jobdefs parameters
            #
            possible_jobdefs_flags = ['results_dir','project','ifcluster','walltime',
                                    'submitts','gridpath','real_run','fehip_mode','root_installed']
            for key in possible_jobdefs_flags:
                if key==line[0]:
                    self.jobdefs[key]=line[2]
                    #print "jobdefs parameter ",key," set to ",self.jobdefs[key]
            #
            #reading cut parameters
            #
            possible_cut_flags = ['PtHiggsCut','PtHiggsMin','PtHiggsMax','EtaHiggsCut','EtaHiggsMin','EtaHiggsMax',
'UseSISCone','JetConeSize','JetDefPtMin','JetDefEtaMax','JetVeto','JetPtMax','JetEtaMax',
'NumVeto','JetNmax','TeVVeto','TeVJetPtMax1','TeVJetPtMax2','EtaV2Cut','EtaV2Min','EtaV2Max',
'MassV1Cut','MassV1Min','MassV1Max','MassV2Cut','MassV2Min','MassV2Max',
'PtV1Cut','PtV1Min','PtV1Max','PtV2Cut','PtV2Min','PtV2Max',
'EtaV1Cut','EtaV1Min','EtaV1Max','EtaV2Cut','EtaV2Min','EtaV2Max',
'LepHardMinPtCut','LepHardPtMinVal','LepHardMaxPtCut','LepHardPtMaxVal',
'LepSoftMinPtCut','LepSoftPtMinVal','LepSoftMaxPtCut','LepSoftPtMaxVal',
'LepHardMaxEtaCut','LepHardEtaMaxVal','LepSoftMaxEtaCut','LepSoftEtaMaxVal',
'LepTriggerSpec','LepPtMin','LepEtaMax',
'PhiLLCut','PhiLLMin','PhiLLMax','PtMissCut','PtMissMin','PtMissMax',
'SpecMETCut','SpecMETMin','SumPtCut','SumPtMin','SumPtMax',
'IsolationCut','IsoConeSize','IsoEnergyFrac','InvMassCut','InvMassMin','InvMassMax',
'cut_on_pt_of_all_leptons','all_leptons_minimum_pt','cut_on_rapidity_of_all_leptons','all_leptons_maximum_y'
]
            for key in possible_cut_flags:
                if key==line[0]:
                    self.cuts[key]=line[2]

#---------------------------------------------------------------------
    def ReadCommandLineArg(self):
        if len(sys.argv)>1:
            if sys.argv[1]=='collect':
                print 'we collect only'
                self.jobdefs['fehip_mode']='collect'
            elif sys.argv[1]=='run':
                print 'we run only'
                self.jobdefs['fehip_mode']='run'
            else:
                print '\nERROR: sorry, unknown command line option: I can\'t recognize ',sys.argv[1]
                print '\nHELP : legal command line options are: collect and  run\n\n'
                self.jobdefs['fehip_mode']='unknown'
            
#---------------------------------------------------------------------
    def SetDefaults(self):
        #print '::',sys._getframe().f_code.co_name
        #default values for all parameters
        self.param={'type':'inclusive'}
        self.param['mh']= '120.0'
        self.param['GammaH']= '0'
        self.param['muFrel']= '0.5'
        self.param['muRrel']= '0.5'
        self.param['icollider']= '0'
        self.param['sqrtS']= '14000'
        self.param['mW']= '80.398'
        self.param['GammaW']= '2.141'
        self.param['mZ']= '91.1876'
        self.param['GammaZ']= '2.4952'
        self.param['mt']= '173.1'
        self.param['mb']= '4.8'
        self.param['imassive']= '1'
        self.param['iewk']= '2'
        self.param['imbscheme']= '0'
        self.param['ytrel']= '1.0'
        self.param['ybrel']= '1.0'
        self.param['idecay']= '0'
        self.param['exclWidth']= '0'
        self.param['BRgg']= '0.0'
        self.param['porder']= '2'
        self.param['iinclusive']= '1'
        self.param['intMethod']= '11'
        self.param['accRel']= '0.1'
        self.param['accAbs']= '0.000001'
        self.param['ntherm']= '10'
        self.param['nrefin']= '5'
        self.param['ncalls']= '50000'
        self.param['nincr']= '0'
        self.param['adaptofirst']= '1'
        self.param['maxtime']= '0'
        self.param['pdfset']= '4.0'
        self.param['pdferror']= '0'
        self.param['whichalphas']= '0'
        self.param['NNLOcont']= []
        self.param['NNLOsect']= {}
        self.param['consflag']= '0.0'
        self.param['PtHiggsCut']= '0.0'
        self.param['PtHiggsMin']= '0.0'
        self.param['PtHiggsMax']= '0.0'
        self.param['EtaHiggsCut']= '0.0'
        self.param['EtaHiggsMin']= '0.0'
        self.param['EtaHiggsMax']= '0.0'
        self.param['JetConeSize']= '0.0'
        self.param['JetDefPtMin']= '0.0'
        self.param['JetDefEtaMax']='100.0'
        self.param['UseSISCone']= '0.0'
        self.param['JetVeto']= '0.0'
        self.param['JetPtMax']= '0.0'
        self.param['JetEtaMax']= '0.0'
        self.param['NumVeto']= '0.0'
        self.param['JetNmax']= '0'
        self.param['TeVVeto']= '0.0'
        self.param['TeVJetPtMax1']= '0.0'
        self.param['TeVJetPtMax2']= '0.0'
        self.param['MassV1Cut']=  '0.0'
        self.param['MassV1Min']= '0.0'
        self.param['MassV1Max']= '0.0'
        self.param['MassV2Cut']=  '0.0'
        self.param['MassV2Min']= '0.0'
        self.param['MassV2Max']= '0.0'
        self.param['PtV1Cut']= '0.0'
        self.param['PtV1Min']= '0.0'
        self.param['PtV1Max']= '0.0'
        self.param['PtV2Cut']= '0.0'
        self.param['PtV2Min']= '0.0'
        self.param['PtV2Max']= '0.0'
        self.param['EtaV1Cut']= '0.0'
        self.param['EtaV1Min']= '0.0'
        self.param['EtaV1Max']= '0.0'
        self.param['EtaV2Cut']= '0.0'
        self.param['EtaV2Min']= '0.0'
        self.param['EtaV2Max']= '0.0'
        self.param['LepHardMinPtCut']= '0.0'
        self.param['LepHardPtMinVal']= '0.0'
        self.param['LepHardMaxPtCut']= '0.0'
        self.param['LepHardPtMaxVal']= '0.0'
        self.param['LepSoftMinPtCut']= '0.0'
        self.param['LepSoftPtMinVal']= '0.0'
        self.param['LepSoftMaxPtCut']= '0.0'
        self.param['LepSoftPtMaxVal']= '0.0'
        self.param['LepHardMaxEtaCut']= '0.0'
        self.param['LepHardEtaMaxVal']= '0.0'
        self.param['LepSoftMaxEtaCut']= '0.0'
        self.param['LepSoftEtaMaxVal']= '0.0'
        self.param['LepTriggerSpec']= '0.0'
        self.param['LepPtMin']= '0.0'
        self.param['LepEtaMax']= '0.0'
        self.param['PhiLLCut']= '0.0'
        self.param['PhiLLMin']= '0.0'
        self.param['PhiLLMax']= '0.0'
        self.param['PtMissCut']= '0.0'
        self.param['PtMissMin']= '0.0'
        self.param['PtMissMax']= '0.0'
        self.param['SpecMETCut']= '0.0'
        self.param['SpecMETMin']= '0.0'
        self.param['SumPtCut']= '0.0'
        self.param['SumPtMin']= '0.0'
        self.param['SumPtMax']= '0.0'
        self.param['IsolationCut']= '0.0'
        self.param['IsoConeSize']= '0.0'
        self.param['IsoEnergyFrac']= '0.0'
        self.param['InvMassCut']= '0.0'
        self.param['InvMassMin']= '0.0'
        self.param['InvMassMax']= '0.0'
        self.param['PhotBasicCut']= '0.0'
        self.param['PhotPtMin1']= '0.0'
        self.param['PhotPtMin2']= '0.0'
        self.param['PhotEtaMax']= '0.0'
        self.param['PhotIsolCut']= '0.0'
        self.param['PhotConeIso']= '0.0'
        self.param['PhotEtmax']= '0.0'
        self.param['PtSoftCut']= '0.0'
        self.param['PtSoftMin']= '0.0'
        self.param['PtmCut']= '0.0'
        self.param['PtmMin']= '0.0'
        self.param['PtmMax']= '0.0'
        self.param['PhotEtaHardCut']= '0.0'
        self.param['PhotEtaHardMin']= '0.0'
        self.param['PhotEtaHardMax']= '0.0'
        self.param['YstarCut']= '0.0'
        self.param['YstarMin']= '0.0'
        self.param['YstarMax']= '0.0'
        self.param['C0user']='0.0'
        self.param['C1user']='0.0'
        self.param['C2user']='0.0'
        self.param['include_higgs_width']='1.0'
        self.param['plot_mu_a']='no'
        self.param['as_scan']=['0']
        self.param['cut_on_pt_of_all_leptons']='0.0'
        self.param['all_leptons_minimum_pt']='0.0'
        self.param['cut_on_rapidity_of_all_leptons']='0.0'
        self.param['all_leptons_maximum_y']='100.0'
        self.param['TevatronSpecialScan']='false'
        self.param['ConfidenceLevel']='68%'
        self.jobdefs['real_run']='yes'
        self.jobdefs['fehip_mode']='run'
        self.jobdefs['exe']='fehip'
        self.jobdefs['walltime']='60'
        self.jobdefs['root_installed']='no'
#---------------------------------------------------------------------
    def ExtendMyFixed(self):
        #mass scan variables
        if self.myfixed['mass_scan']=='no':
            self.mass_scan=[self.myfixed['m_higgs']]
        else:
            self.mass_scan=range(   int(self.myfixed['mass_scan_low_end']),
                                    int(self.myfixed['mass_scan_high_end']) +int( self.myfixed['mass_scan_step']),
                                    int(self.myfixed['mass_scan_step'])
                                    )
        #translate the pdfset variable
        pdftranslate={'MSTW2008':'4.0','MRST2006':'3.0','MRST2004':'2.0','MRST2001':'1.0'}
        self.myfixed['pdfset']=pdftranslate[self.myfixed['pdfset']]
        #defining the mb for the two choices of renormalization scheme
        if 'imbscheme' in self.myfixed.keys():
            if self.myfixed['imbscheme']=='0':self.param['mb']='4.8'
            elif self.myfixed['imbscheme']=='1':self.param['mb']='3.609'
        #if there is a factorization scale scan, we need to have the value 0.5
		#included as a central value for mass scans
        if self.myfixed['factorization_scale_scan']=='yes' and not('factorization_scale_scan_step' in self.myfixed.keys()):
            self.factorization_scale_scan=[ self.myfixed['factorization_scale_low_end'],
                                            self.myfixed['factorization_scale'],
                                            self.myfixed['factorization_scale_high_end']]
        elif self.myfixed['factorization_scale_scan']=='yes' and 'factorization_scale_scan_step' in self.myfixed.keys():
            start=round(float(self.myfixed['factorization_scale_low_end']),5)
            end  =round(float(self.myfixed['factorization_scale_high_end']),5)
            step =round(float( self.myfixed['factorization_scale_scan_step']),5)
            self.factorization_scale_scan=[]
            curx=start
            while curx<end:
                self.factorization_scale_scan=self.factorization_scale_scan+ [round(curx,5)]
                curx=curx+step
            print self.factorization_scale_scan
        else:
            self.factorization_scale_scan=[self.myfixed['factorization_scale']]
        #porder from human readable perturbative order
        translate_porder = {'LO':'0', 'NLO':'1', 'NNLO':'2'}
        self.myfixed['porder']=translate_porder[self.myfixed['perturbative_order']]
        #type
        if self.myfixed['type'] == 'inclusive':self.myfixed['iinclusive']= '1' #-- 0:exclusive 1:inclusive
        else:self.myfixed['iinclusive']= '0' #-- 0:exclusive 1:inclusive
        #massive corrections
        if self.myfixed['massive_corrections']=='none':
            self.myfixed['imassive']='0'
            self.myfixed['ytrel']= '0.0'
            self.myfixed['ybrel']= '0.0'
        else: 
            self.myfixed['imassive']='1'
            if self.myfixed['massive_corrections']=='top-only':
				self.myfixed['ytrel']= '1.0'
				self.myfixed['ybrel']= '0.0'
            if self.myfixed['massive_corrections']=='top-bottom':
				self.myfixed['ytrel']= '1.0'
				self.myfixed['ybrel']= '1.0'
        # varying a_s through the pdfs
        if self.myfixed['vary_a_s']=='yes':
            if self.myfixed['ConfidenceLevel']=='68%':
                self.myfixed['as_scan']= ['4','3','5'] #a_s varying in pdf's - 68% CL 
            elif self.myfixed['ConfidenceLevel']=='90%':
                self.myfixed['as_scan']= ['1','0','2'] #a_s varying in pdf's - 90% CL
            else:
                print "[ExtendMyFixed] ERROR: ConfidenceLevel value not recognized. MSTW2008 support 68% and 90%"
        else:self.myfixed['as_scan']=['0']
        self.param['mu_central']=self.myfixed['factorization_scale']
        # collider definition
        if self.myfixed['collider']=='LHC':self.myfixed['icollider']='0'#lhc
        else:self.myfixed['icollider']='1' #tevatron
        # decay mode definition
        if ('decay' in self.myfixed.keys()):
            decay_mode_translate={'none':'0','WW':'1','WW-ZZ':'2','ZZ':'3','ZZ->l1 l1 l2 l2':'4','gamma gamma':'5'}
            self.param['idecay']=decay_mode_translate[self.myfixed['decay']]
        # executable definition
        if ('decay' in self.myfixed.keys() and self.myfixed['type'] == 'exclusive'):
            executable_translate={'none':'fehip','WW':'fehipWW','WW-ZZ':'fehipWW','ZZ':'fehipZZ',
                                'ZZ->l1 l1 l2 l2':'fehipZZ','gamma gamma':'fehipgg'}
            self.jobdefs['exe']=executable_translate[self.myfixed['decay']]
        else:
            self.myfixed['decay']='none' #default value inserted here.
        # electroweak corrections
        ew_translate={'off':'0','only-virtual':'1','virtual-real':'2'}
        self.myfixed['iewk']=ew_translate[self.myfixed['electroweak_corrections']]
        # pdf corrections
        if self.myfixed['calculate_pdf_error']=='yes':self.myfixed['pdferror']='1'
        else:self.myfixed['pdferror']='0'
        # path fixing
        thisdir = os.getcwd()
        fehipdir=os.path.join(thisdir,'..')
        self.jobdefs['projdir']= os.path.join(thisdir,"results")#-- Path to result directory for this project
        self.jobdefs['exepath']= os.path.join(fehipdir,'bin')		#-- Path to the directory containing the executable
        #self.jobdefs['gridpath']= fehipdir                           #-- Path to the directory containing pdf grids 
        # wilson coefficients
        if os.path.exists(self.myfixed['WilsonFile']):
			ftmp=open(wilsonfile)
			line=ftmp.readline()
			ftmp.close()
			C0BSM,C1BSM,C2BSM=line.split()
			self.myfixed['C0user'] = C0BSM
			self.myfixed['C1user'] = C1BSM
			self.myfixed['C2user'] = C2BSM
        #-- higgs width effects flag
        if self.myfixed['higgs_width_flag']=='yes':
            self.myfixed['include_higgs_width']=1
        else:
            self.myfixed['include_higgs_width']=0
        #-- convert maximum run time to seconds for vegas 
        if ('walltime' in self.jobdefs.keys() and  self.jobdefs['walltime']=='0'):
            self.jobdefs['walltime']='60'
        
        self.myfixed['maxtime']=str(max(int(self.jobdefs['walltime'])*60-15,15))
        #-- distributions
        if self.myfixed['decay']=='none' or self.myfixed['type']=='inclusive':
            self.dists=['PT HIGGS','Y HIGGS']
        elif self.myfixed['decay']=='WW' and self.myfixed['type']=='exclusive':
            self.dists=['PHILL','INV MASS', 'MET', 'PT HARD', 'PT SOFT']#, 'ANN OUT'] # ANN OUT doesn't seem to work
        elif self.myfixed['decay']=='gamma gamma' and self.myfixed['type']=='exclusive':
            self.dists=['PT AVG','Ystar', 'PT LEAD', 'PT HIGGS', 'Y HIGGS']
        elif self.myfixed['decay']=='ZZ' and self.myfixed['type']=='exclusive':
            self.dists=['MT','PT ONE', 'PT TWO', 'PT THREE', 'PT FOUR']
        elif self.myfixed['decay']=='ZZ->l1 l1 l2 l2' and self.myfixed['type']=='exclusive':
            self.dists=['PT ONE', 'PT TWO', 'PT THREE', 'PT FOUR' , 'Y_1' , 'Y_2' , 'Y_3' , 'Y_4']
        self.dists_translate = {'PT HIGGS': 'PT_HIGGS','Y HIGGS':'Y_HIGGS',
                                'PHILL':'PHILL','INV MASS':'INV_MASS', 
                                'MET':'MET', 'PT HARD':'PT_HARD', 'PT SOFT':'PT_SOFT', 'ANN OUT':'ANN_OUT',
                                'PT AVG':'PT_AVG','Ystar':'Ystar', 'PT LEAD':'PT_LEAD', 
                                'MT':'MT','PT ONE':'PT_ONE', 'PT TWO':'PT_TWO', 'PT THREE':'PT_THREE', 'PT FOUR':'PT_FOUR',
                                'Y_1':'Y_1','Y_2':'Y_2','Y_3':'Y_3','Y_4':'Y_4'}
        if (self.myfixed['accAbs']=='0.0'):
            self.myfixed['accAbs']='0.00000001'
            #print "self.myfixed['accAbs'] set to ",self.myfixed['accAbs']
#---------------------------------------------------------------------
    def Give_directory_tree(self):
        res=[]
        newdir='%(projdir)s' %(self.jobdefs)#results_dir/current_run_dir 
        res=res+[newdir]
        #if self.myfixed['icollider'] == '1':#Tevatron
        #    newdir=newdir+"/TeV" #results_dir/current_run_dir/ppbar
        #elif self.myfixed['icollider'] == '0':#LHC
        #    newdir=newdir+"/LHC_"+self.param['sqrtS'] #results_dir/current_run_dir/pp
        ##res=res+[newdir]
        #if self.myfixed['iinclusive']=='1':#inclusive
        #    newdir=newdir+"_INCL"
        #else:#exclusive
        #    if self.myfixed['idecay']=='0':#nodecay
        #        newdir=newdir+"_EXCL_NO_DECAY"
        #    elif self.myfixed['idecay']=='1':#WW
        #        newdir=newdir+"_WW"
        #    elif self.myfixed['idecay']=='2':#WW/ZZ
        #        newdir=newdir+"_WW_ZZ"
        #    elif self.myfixed['idecay']=='3':#ZZ
        #        newdir=newdir+"_ZZ"
        #    elif self.myfixed['idecay']=='4':#ZZ->4l
        #        newdir=newdir+"_ZZ_to_llll"
        #    elif self.myfixed['idecay']=='5':#gamma gamma
        #        newdir=newdir+"_gamma_gamma"    
        #res=res+[newdir]
        return res
#---------------------------------------------------------------------
    def SetUpDir(self):
        if not os.path.isdir("results"):os.makedirs("results")
        self.modedir=os.getcwd()
#---------------------------------------------------------------------
    def GiveScanName(self):
        ##self.suffix='_'+self.myfixed['collider']+self.myfixed['sqrtS']+'_'+self.myfixed['massive_corrections']
        ##if self.myfixed['imbscheme'] == '1':self.suffix=self.suffix+'_MSbar' 
        
        #res_name ="muF_"+str(self.param['muFrel'])
        res_name =""
        # '_'+self.myfixed['collider']+self.myfixed['sqrtS']
        #massive_corr = self.myfixed['massive_corrections']
        #if massive_corr == "none": massive_corr = 'mt_inf'
        #elif massive_corr == "top-only": massive_corr = "top_only"
        #elif massive_corr == "top-bottom": massive_corr = "top_bottom"
        #res_name += '_'+massive_corr
        #res_name += '_ew_'+self.myfixed['iewk']
        ##res_name += '_as_'+self.param['whichalphas']
        #if self.param['imbscheme'] == '1':res_name += '_MSbar'
        return res_name
#---------------------------------------------------------------------
    def GiveName(self):
        res_name ="muF_"+str(self.param['muFrel'])
        res_name += '_as_'+self.param['whichalphas']
        return res_name
#---------------------------------------------------------------------
    def RunRoot_x(self,run_dir_name,input):
        root_fname=run_dir_name+"/"+input+".C"
        cmd='root -b -q '+root_fname
        failure, output = commands.getstatusoutput(cmd)
        if failure:
            print 'failed: ' + output
#---------------------------------------------------------------------
    def CollectExclusive(self):
        if not(self.myfixed['perturbative_order']=='NNLO'):
            print "ERROR: Sorry, collect does not support LO or NLO exclusive runs yet"
        else:
            directory_tree=self.Give_directory_tree()
            for loc_path in directory_tree:
                if not os.path.isdir(loc_path):
                    print "path not found: ",loc_path
            collect_base_path = loc_path
            mass_scan_data=[]
            for msc in self.mass_scan:
                self.param['mh']=msc
                self.collect_path_to_mass_dir = collect_base_path+"/m_"+str(self.param['mh'])
                #:Collect_fscan() performs the scan over mu_F, creates the relevant plot
                #:and returns the PDF error and the mu_F error for the central mu_F value = 0.5*m_H
                #:Collect_ascan performs the scan over a_s, creates the relevant plot 
                #:and returns the da_s error for the central value of a_s ('whichalphas'=0) determined by the pdf set
                curmassdata=self.Collect_fscan_ex()+self.Collect_ascan_ex()
                mass_scan_data += [curmassdata]
            if len(self.mass_scan)>1:
                print mass_scan_data
                mscan_fname=collect_base_path+"/mscan"+self.GiveScanName()+".txt"
                mscan_root = "root_mscan"+self.GiveScanName()
                mscan_eps = "mscan"+self.GiveScanName()+".eps"
                f=open(mscan_fname, 'w') 
                f.write("mass\tsigma\tpdf up\tpdf down\tmu up\tmu down\ta up\ta down");
                for xx in mass_scan_data:
                    f.write("\n"+str(xx[0])+"\t"+str(xx[1])+"\t"+str(xx[2])+"\t"+str(xx[3])+"\t"+str(xx[4])+"\t"+str(xx[5])+"\t"+str(xx[6])+"\t"+str(xx[7]))
                f.close()
                if self.jobdefs['root_installed']=='yes':
                    self.WriteRootFileMH(collect_base_path,mscan_fname,mscan_root,mscan_eps)
                    self.RunRoot_x(collect_base_path,mscan_root)

#---------------------------------------------------------------------
    def Collect_fscan_ex(self):
        msc = self.param['mh']
        factorization_scale_scan_data=[]
        for fsc in self.factorization_scale_scan:
                porder_suffix=['LO','NLO','NNLO']
                cur_porder_suffix=porder_suffix[int(self.myfixed['porder'])]
                self.param['muFrel']=fsc
                self.param['whichalphas']='0'
                dir_for_current_scan_value=self.GiveName()
                rundir=self.collect_path_to_mass_dir+"/"+self.GiveName()
                local_result=ClassesCollect.ExclusiveRes(rundir,self.dists)
                local_result.GetResult()
                local_save_path=rundir+"/"+dir_for_current_scan_value+".sav"
                #local_result.Save(local_save_path)
                for porder in ['LO','NLO','NNLO']:
                    for ctr in ['mtinfty','top','topbottom']:
                        resfile=rundir+"/res_%s_%s.txt" %(ctr,porder)
                        local_result.Write(porder,ctr,resfile)
                if self.myfixed['massive_corrections']=='none':
                    local_control='mtinfty'
                elif self.myfixed['massive_corrections']=='top-only':
                    local_control='top'
                elif self.myfixed['massive_corrections']=='top-bottom':
                    local_control='topbottom'
                local_s= local_result.Sig(cur_porder_suffix,local_control)
                local_er1=local_result.PDFerror(cur_porder_suffix,local_control)[1]
                local_er2=local_result.PDFerror(cur_porder_suffix,local_control)[2]
                curdata=[[fsc,local_s,local_er1,local_er2]]
                factorization_scale_scan_data +=  curdata
                if len(self.factorization_scale_scan)==1 or fsc=='0.5' 'factorization_scale_scan_step' in self.myfixed.keys():
                    masscurdata=[msc,local_s,local_er1,local_er2]
                #------------ plotting 
                for histname in self.dists:
                    proper_histname = self.dists_translate[histname]
                    pert_order = self.myfixed['perturbative_order']
                    mass_aprox = 'topbottom'
                    root_input_fname = "root_"+proper_histname
                    eps_file_name =  proper_histname+".eps"
                    self.WriteHistoDataFiles(local_result.Hist(histname,pert_order,mass_aprox),rundir,histname)
                    if self.jobdefs['root_installed']=='yes':
                        self.WriteRootFileFromData(
                            local_result.Hist(histname,pert_order,mass_aprox),
                            rundir,
                            msc,
                            histname,
                            root_input_fname,
                            eps_file_name)
                        self.RunRoot_x(rundir,root_input_fname)
        dmu_up=0.0
        dmu_down=0.0
        if len(self.factorization_scale_scan)>1:
            fscan_fname=self.collect_path_to_mass_dir+"/fscan_m"+str(msc)+self.GiveScanName()+".txt"
            fscanroot = "root_plot_fscan"+self.GiveScanName()
            fscan_eps = "fscan"+self.GiveScanName()+".eps"
            f=open(fscan_fname, 'w') 
            f.write("fscan file\nmh="+str(msc))
            for xx in factorization_scale_scan_data:
                true_f=float(xx[0])*float(msc)
                f.write("\n"+str(true_f)+"\t"+str(xx[1])+"\t"+str(xx[2])+"\t"+str(xx[3]))
                cur_sigma=float(xx[1])
                if cur_sigma-masscurdata[1]>dmu_up:dmu_up=cur_sigma-masscurdata[1]
                if cur_sigma-masscurdata[1]<dmu_down:dmu_down=cur_sigma-masscurdata[1]
                #print '::',cur_sigma,cur_sigma-masscurdata[1],dmu_up,dmu_down
            f.close()
            if self.jobdefs['root_installed']=='yes':
                self.WriteRootFile_x(self.collect_path_to_mass_dir,fscan_fname,msc,"#mu_{F} (GeV)",fscanroot,fscan_eps)
                self.RunRoot_x(self.collect_path_to_mass_dir,fscanroot)
        return masscurdata+[dmu_up,dmu_down]
#---------------------------------------------------------------------
    def Collect_ascan_ex(self):
        if len(self.param['as_scan'])==1:
            return [0.0,0.0]
        msc = self.param['mh']
        as_data=[]
        for local_as in self.param['as_scan']:
                porder_suffix=['LO','NLO','NNLO']
                cur_porder_suffix=porder_suffix[int(self.myfixed['porder'])]
                self.param['muFrel']=self.param['mu_central']
                self.param['whichalphas']=local_as
                dir_for_current_scan_value=self.GiveName()
                rundir=self.collect_path_to_mass_dir+"/"+self.GiveName()
                #print rundir
                local_result=ClassesCollect.ExclusiveRes(rundir,self.dists)
                local_result.GetResult()
                local_save_path=rundir+"/"+dir_for_current_scan_value+".sav"
                #local_result.Save(local_save_path)
                #print '\nrundir: ',rundir
                for porder in ['LO','NLO','NNLO']:
                    for ctr in ['mtinfty','top','topbottom']:
                        resfile=rundir+"/res_%s_%s.txt" %(ctr,porder)
                        local_result.Write(porder,ctr,resfile)  
                if self.myfixed['massive_corrections']=='none':
                    local_control='mtinfty'
                elif self.myfixed['massive_corrections']=='top-only':
                    local_control='top'
                elif self.myfixed['massive_corrections']=='top-bottom':
                    local_control='topbottom'
                #print '\n path given to Write: ',   local_res_txt_path
                #print 'sigma_',fsc,' = ',local_result.Sig(cur_porder_suffix)
                actual_as = local_result.As(cur_porder_suffix)
                local_s= local_result.Sig(cur_porder_suffix,local_control)
                local_er1=local_result.PDFerror(cur_porder_suffix,local_control)[1]
                local_er2=local_result.PDFerror(cur_porder_suffix,local_control)[2]
                as_data += [[actual_as,local_s,local_er1,local_er2]]
                if len(self.param['as_scan'])==1 or local_as=='0':
                    masscurdata=[msc,local_s,local_er1,local_er2]
                #------------ plotting 
                for histname in self.dists:
                    proper_histname = self.dists_translate[histname]
                    pert_order = self.myfixed['perturbative_order']
                    mass_aprox = 'topbottom'
                    root_input_fname = "root_"+proper_histname
                    eps_file_name =  proper_histname+".eps"
                    self.WriteHistoDataFiles(local_result.Hist(histname,pert_order,mass_aprox),rundir,histname)
                    if self.jobdefs['root_installed']=='yes':
                        self.WriteRootFileFromData(
                            local_result.Hist(histname,pert_order,mass_aprox),
                            rundir,
                            msc,
                            histname,
                            root_input_fname,
                            eps_file_name)
                        self.RunRoot_x(rundir,root_input_fname)
        da_up=0.0
        da_down=0.0
        if len(self.param['as_scan'])>1:
            ascan_fname=self.collect_path_to_mass_dir+"/ascan_m"+str(msc)+self.GiveScanName()+".txt"
            ascan_root = "root_plot_ascan"+self.GiveScanName()
            ascan_eps = "ascan"+self.GiveScanName()+".eps"
            f=open(ascan_fname, 'w') 
            f.write("ascan file\nmh="+str(msc))
            for xx in as_data:
                f.write("\n"+str(xx[0])+"\t"+str(xx[1])+"\t"+str(xx[2])+"\t"+str(xx[3]))
                cur_sigma=float(xx[1])
                if cur_sigma-masscurdata[1]>da_up:da_up=cur_sigma-masscurdata[1]
                if cur_sigma-masscurdata[1]<da_down:da_down=cur_sigma-masscurdata[1]
            f.close()
            if self.jobdefs['root_installed']=='yes':
                self.WriteRootFile_x(self.collect_path_to_mass_dir,ascan_fname,msc,"a_{s} (m_{Z})",ascan_root,ascan_eps)
                self.RunRoot_x(self.collect_path_to_mass_dir,ascan_root)
        return [da_up,da_down]

#---------------------------------------------------------------------
    def Collect_fscan(self,per_ord):
        msc = self.param['mh']
        factorization_scale_scan_data=[]
        for fsc in self.factorization_scale_scan:
                cur_porder_suffix=per_ord
                self.param['muFrel']=fsc
                self.param['whichalphas']='0'
                dir_for_current_scan_value=self.GiveName()
                rundir=self.collect_path_to_mass_dir+"/"+self.GiveName()
                local_result=ClassesCollect.InclusiveRes(rundir)
                local_result.GetResult()
                local_save_path=rundir+"/"+dir_for_current_scan_value+".sav"
                #local_result.Save(local_save_path)
                local_res_txt_path=rundir+"/CollectedResults"+per_ord+".txt"
                local_result.Write(cur_porder_suffix,local_res_txt_path)  
                local_s= local_result.Sig(cur_porder_suffix)
                local_er1=local_result.PDFerror(cur_porder_suffix)[1]
                local_er2=local_result.PDFerror(cur_porder_suffix)[2]
                curdata=[[fsc,local_s,local_er1,local_er2]]
                factorization_scale_scan_data +=  curdata
                if len(self.factorization_scale_scan)==1 or fsc=='0.5' or  'factorization_scale_scan_step' in self.myfixed.keys():
                    masscurdata=[msc,local_s,local_er1,local_er2]
        dmu_up=0.0
        dmu_down=0.0
        if len(self.factorization_scale_scan)>1:
            fscan_fname=self.collect_path_to_mass_dir+"/fscan_m"+str(msc)+self.GiveScanName()+per_ord+".txt"
            fscanroot = "root_plot_fscan"+self.GiveScanName()+per_ord
            fscan_eps = "fscan"+self.GiveScanName()+per_ord+".eps"
            f=open(fscan_fname, 'w') 
            f.write("fscan file\nmh="+str(msc)+"\t"+per_ord)
            for xx in factorization_scale_scan_data:
                true_f=float(xx[0])*float(msc)
                f.write("\n"+str(true_f)+"\t"+str(xx[1])+"\t"+str(xx[2])+"\t"+str(xx[3]))
                cur_delta_sigma=float(xx[1])-masscurdata[1]
                if cur_delta_sigma>dmu_up:dmu_up=cur_delta_sigma
                if cur_delta_sigma<dmu_down:dmu_down=cur_delta_sigma
                #print '::',cur_sigma,cur_sigma-masscurdata[1],dmu_up,dmu_down
            f.close()
            if (self.param['plot_mu_a']=='yes'):
                if self.jobdefs['root_installed']=='yes':
                    self.WriteRootFile_x(self.collect_path_to_mass_dir,fscan_fname,msc,"#mu_{F} (GeV)",fscanroot,fscan_eps)
                    self.RunRoot_x(self.collect_path_to_mass_dir,fscanroot)
        return masscurdata+[dmu_up,-dmu_down]
#---------------------------------------------------------------------
    def Collect_ascan(self,per_ord):
        msc = self.param['mh']
        as_data=[]
        for local_as in self.param['as_scan']:
                cur_porder_suffix=per_ord
                self.param['muFrel']=self.param['mu_central']
                self.param['whichalphas']=local_as
                dir_for_current_scan_value=self.GiveName()
                rundir=self.collect_path_to_mass_dir+"/"+self.GiveName()
                #print rundir
                local_result=ClassesCollect.InclusiveRes(rundir)
                local_result.GetResult()
                local_save_path=rundir+"/"+dir_for_current_scan_value+".sav"
                #local_result.Save(local_save_path)
                #print '\nrundir: ',rundir
                local_res_txt_path=rundir+"/CollectedResults"+per_ord+".txt"
                local_result.Write(cur_porder_suffix,local_res_txt_path)  
                #print '\n path given to Write: ',   local_res_txt_path
                #print 'sigma_',fsc,' = ',local_result.Sig(cur_porder_suffix)
                actual_as = local_result.As(cur_porder_suffix)
                local_s= local_result.Sig(cur_porder_suffix)
                local_er_plus=local_result.PDFerror(cur_porder_suffix)[1]
                local_er_minus=local_result.PDFerror(cur_porder_suffix)[2]
                as_data += [[actual_as,local_s,local_er_plus,local_er_minus]]
                if len(self.param['as_scan'])==1 or local_as=='0':
                    masscurdata=[msc,local_s,local_er_plus,local_er_minus]
        da_up=0.0
        da_down=1e25
        if len(self.param['as_scan'])>1:
            ascan_fname=self.collect_path_to_mass_dir+"/ascan_m"+str(msc)+self.GiveScanName()+per_ord+".txt"
            ascan_root = "root_plot_ascan"+self.GiveScanName()+per_ord
            ascan_eps = "ascan"+self.GiveScanName()+per_ord+".eps"
            f=open(ascan_fname, 'w') 
            f.write("ascan file\nmh="+str(msc)+"\t"+per_ord)
            for xx in as_data:
                cur_sigma_plus=float(xx[1])+float(xx[2]);
                cur_sigma_minus=float(xx[1])-float(xx[3]);
                if cur_sigma_plus>da_up:da_up=cur_sigma_plus
                if cur_sigma_minus<da_down:da_down=cur_sigma_minus
                f.write("\n"+str(xx[0])+"\t"+str(xx[1])+"\t"+str(xx[2])+"\t"+str(xx[3]))
            f.close()
            if (self.param['plot_mu_a']=='yes'):
                if self.jobdefs['root_installed']=='yes':
                    self.WriteRootFile_x(self.collect_path_to_mass_dir,ascan_fname,msc,"a_{s} (m_{Z})",ascan_root,ascan_eps)
                    self.RunRoot_x(self.collect_path_to_mass_dir,ascan_root)
        return [da_up-masscurdata[1],masscurdata[1]-da_down]
#---------------------------------------------------------------------
    def CollectInclusive(self):
        directory_tree=self.Give_directory_tree()
        for loc_path in directory_tree:
            if not os.path.isdir(loc_path):
                print "path not found: ",loc_path
        collect_base_path = loc_path
        if (self.myfixed['perturbative_order']=='NNLO'):
            porder_suffix=['LO','NLO','NNLO']
        else:   
            porder_suffix=[self.myfixed['perturbative_order']]
        for per_ord in porder_suffix:
            mass_scan_data=[]
            for msc in self.mass_scan:
                if len(self.mass_scan)>1:print "reading file for m= ",msc," @",per_ord
                self.param['mh']=msc
                self.collect_path_to_mass_dir = collect_base_path+"/m_"+str(self.param['mh'])
                #:Collect_fscan() performs the scan over mu_F, creates the relevant plot
                #:and returns the PDF error and the mu_F error for the central mu_F value = 0.5*m_H
                #:Collect_ascan performs the scan over a_s, creates the relevant plot 
                #:and returns the da_s error for the central value of a_s ('whichalphas'=0) determined by the pdf set
                curmassdata=self.Collect_fscan(per_ord)+self.Collect_ascan(per_ord)
                mass_scan_data += [curmassdata]
            if len(self.mass_scan)>1:
                print '**',self.GiveScanName(),per_ord
                mscan_fname=collect_base_path+"/mscan"+self.GiveScanName()+per_ord+".txt"
                mscan_fname_tex=collect_base_path+"/mscan"+self.GiveScanName()+per_ord+".tex"
                mscan_root = "root_mscan"+self.GiveScanName()+per_ord
                mscan_eps = "mscan"+self.GiveScanName()+per_ord+".eps"
                f=open(mscan_fname_tex, 'w') 
                f.write(per_ord+"\nmass\tsigma\tcentral PDF + \tcentral PDF -\tmu +\tmu - \t(PDF+a_s) +\t(PDF+a_s) -\t% mu +\t\t% mu - \t% (PDF+a_s) +\t% (PDF+a_s) -");
                for xx in mass_scan_data:
                    f.write("\n"+str(xx[0])+"&\t"+str(round(xx[1],2))
                    +"&\t"+str(round(xx[4]/xx[1]*100.0,2))+"&\t"+str(round(xx[5]/xx[1]*100.0,2))
                    +"&\t"+str(round(xx[6]/xx[1]*100.0,2))+"&\t"+str(round(xx[7]/xx[1]*100.0,2))
                    +"\\\\")
                f.write("\n")
                f.close()
                f=open(mscan_fname,'w')
                f.write(per_ord+"\nmass\tsigma\tcentral PDF + \tcentral PDF -\tmu +\tmu - \t(PDF+a_s) +\t(PDF+a_s) -\t% mu +\t\t% mu - \t% (PDF+a_s) +\t% (PDF+a_s) -");
                for xx in mass_scan_data:
                    f.write("\n"+str(xx[0])
                    +"\t"+str(round(xx[1],2))
                    +"\t"+str(round(xx[2],2))
                    +"\t"+str(round(xx[3],2))
                    +"\t"+str(round(xx[4],2))
                    +"\t"+str(round(xx[5],2))
                    +"\t"+str(round(xx[6],2))
                    +"\t"+str(round(xx[7],2))
                    +"\t"+str(round(xx[4]/xx[1]*100.0,2))
                    +"\t"+str(round(xx[5]/xx[1]*100.0,2))
                    +"\t"+str(round(xx[6]/xx[1]*100.0,2))
                    +"\t"+str(round(xx[7]/xx[1]*100.0,2))
                    )
                f.write("\n")
                f.close()
                if self.jobdefs['root_installed']=='yes':
                    self.WriteRootFileMH(collect_base_path,mscan_fname,mscan_root,mscan_eps)
                    self.RunRoot_x(collect_base_path,mscan_root)

#---------------------------------------------------------------------
    def Collect_scans_SpecialTevatron(self,per_ord):
        msc = self.param['mh']
        mass_scan_data=[]
        for fsc in self.factorization_scale_scan:   
            for local_as in self.param['as_scan']: 
                cur_porder_suffix=per_ord
                self.param['muFrel']=fsc
                self.param['whichalphas']=local_as
                dir_for_current_scan_value=self.GiveName()
                rundir=self.collect_path_to_mass_dir+"/"+self.GiveName()
                local_result=ClassesCollect.InclusiveRes(rundir)
                local_result.GetResult()
                local_save_path=rundir+"/"+dir_for_current_scan_value+".sav"
                #local_result.Save(local_save_path)
                local_res_txt_path=rundir+"/CollectedResults"+per_ord+".txt"
                local_result.Write(cur_porder_suffix,local_res_txt_path)  
                local_s= local_result.Sig(cur_porder_suffix)
                local_er1=local_result.PDFerror(cur_porder_suffix)[1]
                local_er2=local_result.PDFerror(cur_porder_suffix)[2]
                mass_scan_data += [[msc,fsc,local_as,local_s,local_er1,local_er2]]
        return mass_scan_data

#---------------------------------------------------------------------
    def CollectInclusiveSpecialTevatron(self):
        directory_tree=self.Give_directory_tree()
        for loc_path in directory_tree:
            if not os.path.isdir(loc_path):
                print "path not found: ",loc_path
        collect_base_path = loc_path
        porder_suffix=['LO','NLO','NNLO']
        for per_ord in porder_suffix:
            mass_scan_data=[]
            for msc in self.mass_scan:
                if len(self.mass_scan)>1:print "reading file for m= ",msc," @",per_ord
                self.param['mh']=msc
                self.collect_path_to_mass_dir = collect_base_path+"/m_"+str(self.param['mh'])
                #:Collect_fscan() performs the scan over mu_F, creates the relevant plot
                #:and returns the PDF error and the mu_F error for the central mu_F value = 0.5*m_H
                #:Collect_ascan performs the scan over a_s, creates the relevant plot 
                #:and returns the da_s error for the central value of a_s ('whichalphas'=0) determined by the pdf set
                mass_scan_data += self.Collect_scans_SpecialTevatron(per_ord)
                mscan_fname=collect_base_path+"/mscan"+self.GiveScanName()+per_ord+".txt"
                mscan_fname_tex=collect_base_path+"/mscan"+self.GiveScanName()+per_ord+".tex"
                mscan_root = "root_mscan"+self.GiveScanName()+per_ord
                mscan_eps = "mscan"+self.GiveScanName()+per_ord+".eps"
                #f=open(mscan_fname_tex, 'w') 
                #f.write(per_ord+"\nmass\tsigma\tcentral PDF + \tcentral PDF -\tmu +\tmu - \t(PDF+a_s) +\t(PDF+a_s) -\t% mu +\t\t% mu - \t% (PDF+a_s) +\t% (PDF+a_s) -");
                #for xx in mass_scan_data:
                #    f.write("\n"+str(xx[0])+"&\t"+str(round(xx[1],2))
                #    +"&\t"+str(round(xx[4]/xx[1]*100.0,2))+"&\t"+str(round(xx[5]/xx[1]*100.0,2))
                #    +"&\t"+str(round(xx[6]/xx[1]*100.0,2))+"&\t"+str(round(xx[7]/xx[1]*100.0,2))
                #    +"\\\\")
                #f.write("\n")
                #f.close()
                
            f=open(mscan_fname,'w')
            f.write(per_ord+"\nmass\tmuF\ta_s\tsigma\tPDF + \tPDF -");
            for xx in mass_scan_data:
                f.write("\n"+str(xx[0])
                    +"\t"+str(xx[1])
                    +"\t"+str(xx[2])
                    +"\t"+str(round(xx[3],2))
                    +"\t"+str(round(xx[4],2))
                    +"\t"+str(round(xx[5],2))
                    )
            f.write("\n")
            f.close()
            #if self.jobdefs['root_installed']=='yes':
                #self.WriteRootFileMH(collect_base_path,mscan_fname,mscan_root,mscan_eps)
                #self.RunRoot_x(collect_base_path,mscan_root)

#---------------------------------------------------------------------
    def DoLoopOverScanVariables(self):
        #print '::',sys._getframe().f_code.co_name
        for msc in self.mass_scan:
            self.param['mh']=msc
            self.mass_dir = self.modedir+"/results/m_"+str(self.param['mh'])
            if not os.path.isdir(self.mass_dir):os.makedirs(self.mass_dir)
            for fsc in self.factorization_scale_scan: 
                self.param['muFrel']=fsc
                self.param['muRrel']=fsc
                self.param['whichalphas']='0' # central a_s
                self.jobdefs['rundir'] = self.mass_dir+"/"+self.GiveName()
                self.jobdefs['case'] = self.GiveName()
                self.DoSectorLoop()
            for local_as in self.param['as_scan']:
                if local_as=='0' : continue
                self.param['muFrel']=self.param['mu_central'] # central mu_f 
                self.param['muRrel']=self.param['mu_central'] # central mu_r
                self.param['whichalphas']=local_as
                self.jobdefs['rundir'] = self.mass_dir+"/"+self.GiveName()
                self.jobdefs['case'] = self.GiveName()
                self.DoSectorLoop()
#---------------------------------------------------------------------
    def DoLoopOverScanVariablesTevatronSpecial(self):
        #print '::',sys._getframe().f_code.co_name
        for msc in self.mass_scan:
            self.param['mh']=msc
            self.mass_dir = self.modedir+"/m_"+str(self.param['mh'])
            if not os.path.isdir(self.mass_dir):os.makedirs(self.mass_dir)
            for fsc in self.factorization_scale_scan: 
                for local_as in self.param['as_scan']:
                    self.param['muFrel']=fsc
                    self.param['muRrel']=fsc
                    self.param['whichalphas']=local_as
                    self.jobdefs['rundir'] = self.mass_dir+"/"+self.GiveName()
                    self.jobdefs['case'] = self.GiveName()
                    self.DoSectorLoop()
#---------------------------------------------------------------------
    def SetUpSectors(self):
        #print '::',sys._getframe().f_code.co_name
        self.NNLOcont=[1,2,3,5,8,9,10,12,13,14]
        if (self.param['imassive']==0):
            self.NNLOsect={
                1: range(1,4), 
                2: range(1,22), 
                3: range(1,17),
                5: range(1,49), 
                8: range(0,1), 
                9: range(1,3),
                10: range(1,3), 
                12: range(1,2), 
                13: range(1,2),
                14: range(1,2)
                }
        else:            
            self.NNLOsect={
                1: range(1,7), 
                2: range(1,22), 
                3: range(1,17),
                5: range(1,49), 
                8: range(0,1), 
                9: range(1,3),
                10: range(1,3), 
                12: range(1,2), 
                13: range(1,2),
                14: range(1,2)
                }
#---------------------------------------------------------------------
    def DoSectorLoop(self):
        #print '::',sys._getframe().f_code.co_name
        self.SetUpSectors()
        if (int(self.param['iinclusive'])==0 and int(self.param['porder'])==2):
            self.RunWithSectors()
        else:
            self.RunWithoutSectors()
#---------------------------------------------------------------------
    def RunWithoutSectors(self):
        #print '::',sys._getframe().f_code.co_name
        self.param['NNLOcont']='0'
        self.param['NNLOsect']='0'
        self.jobdefs['jobname']=self.jobdefs['case']
        self.jobdefs['outfile']="job_output.txt"
        self.jobdefs['logfile']="job.log"
        self.RunSingleJob()
#---------------------------------------------------------------------
    def RunWithSectors(self):
        #print '::',sys._getframe().f_code.co_name
        for contr in self.NNLOcont:	
            for sec in self.NNLOsect[contr]:
                self.param['NNLOcont']=str(contr)
                self.param['NNLOsect']=str(sec)
                sectorname = str2='_%(NNLOcont)s_%(NNLOsect)s' %(self.param)
                self.jobdefs['rundir'] = self.mass_dir+"/"+self.GiveName()+"/sector"+sectorname
                self.jobdefs['jobname'] =  self.GiveName()+sectorname
                self.jobdefs['outfile']="job_output.txt"
                self.jobdefs['logfile']="job.log"
                self.RunSingleJob()
#---------------------------------------------------------------------
    def RunSingleJob(self):
        #print '::',sys._getframe().f_code.co_name
        if not os.path.isdir(self.jobdefs['rundir']):
            os.makedirs(self.jobdefs['rundir'])
        self.setupcmd=''
        self.WriteInputfile()
        if self.jobdefs['real_run']=='yes':
            if self.jobdefs['ifcluster']=='yes':
                self.RunCluster()
            else:
                self.RunWorkstation()
        else:
            print "dummy job"
#---------------------------------------------------------------------
    def WriteRunScript(self):
        scriptname = self.jobdefs['rundir']+'/run.sh' 
        f = open(scriptname,'w')
        f.write("""\
#!/bin/sh

""")
        f.write('\n\ntouch running.flg\n')
        f.write(self.setupcmd)
        f.write('time %(exepath)s/%(exe)s -i input.txt -o result.txt -t %(rundir)s -p %(gridpath)s >> %(outfile)s' % self.jobdefs)
        f.write('\n\nrm running.flg')
        f.close()
#---------------------------------------------------------------------
    def RunCluster(self):
        #if os.path.isfile(os.path.join(self.jobdefs['rundir'],'running.flg')):
        #    print "ERROR: run aborted - there is a job running currently in this running directory"
        #    print "\t* It might also be that a previous job in this directory was abruptly killed"
        #    print "\t* If you are sure this is the case, please delete the running.flg file located "
        #    print "\t* in the current running directory and rerun "
        #else:
            #print '::',sys._getframe().f_code.co_name
            self.setupcmd = 'cd %(rundir)s;\n' %(self.jobdefs)
            self.WriteRunScript()
            cmd='cd %(rundir)s\n bsub -o %(logfile)s -J %(jobname)s -W %(walltime)s < run.sh' % self.jobdefs
            print cmd
            failure, output = commands.getstatusoutput(cmd)
            if failure:
                print 'failed: ' + output
                # resubmit the job until it ended
            #print self.jobdefs['submitts']
            iter=2
            while (iter < int(self.jobdefs['submitts'])+1):
                print 'submitting job ' + str(iter) + ' time... '
                cmd='cd %(rundir)s;\n bsub -w \'ended(%(jobname)s)\' -o %(logfile)s -J %(jobname)s -W %(walltime)s < run.sh' % self.jobdefs
                #print rundir
                failure, output = commands.getstatusoutput(cmd)
                if failure:
                    print 'failed: ' + output
                iter=iter+1
#---------------------------------------------------------------------
    def RunWorkstation(self):
        #print '::',sys._getframe().f_code.co_name
        cmd1 = 'cd '+ self.jobdefs['rundir']+';\n'
        if os.path.isfile(os.path.join(self.jobdefs['rundir'],'running.flg')):
            print "ERROR: run aborted - there is a job running currently in this running directory"
            print "\t* It might also be that a previous job in this directory was abruptly killed"
            print "\t* If you are sure this is the case, please delete the running.flg file located "
            print "\t* in the current running directory and rerun "
        else:
            cmd2 = ''
            #cmd2 = 'touch running.flg;\n'
            #cmd2 = 'ln -sf '+ self.jobdefs['gridpath'] + ' pdfdat;\n'
            cmd3 = 'time %(exepath)s/%(exe)s -i input.txt -o result.txt -t %(rundir)s -p %(gridpath)s >> %(outfile)s' % self.jobdefs
            #cmd3 = ' %(exepath)s/%(exe)s -i input.txt -o result.txt -t %(rundir)s -p %(gridpath)s' % self.jobdefs 
            #cmd4 = 'rm running.flg'
            cmd4 = ''
            #cmd = cmd1+cmd2+cmd3+cmd4
            cmd = cmd1 + cmd3
            print cmd
            failure, output = commands.getstatusoutput(cmd)
            #os.system(cmd)
#---------------------------------------------------------------------
    def WriteRootFileMH(self,dir_name,input_file,output,epsfile):
        epsfname=dir_name+"/"+epsfile
        #print epsfname
        f=open(dir_name+"/"+output+".C",'w')
        f.write("""\
#include<string>
using namespace std;

void """+output+"""()
{
string inputfilename="""+"\""+input_file+"""";

   
gROOT->Reset();
gStyle->SetOptStat(0);
gStyle->SetPalette(1);
gStyle->SetCanvasColor(33);
gStyle->SetFrameFillColor(18);


ifstream in;
Int_t nlines;	
in.open(inputfilename.c_str());
nlines=0;	
for (int i=0;i<1;i++){string header;getline(in,header);}
double MHin,Sigma,Error_up,Error_down,mu_up,mu_down,a_up,a_down;
vector<double> myMH,mysigma,myerrorup,myerrordown;
while (1) 
	{
	in 
	>>MHin
	>>Sigma	
	>>Error_up
    >>Error_down
    >>mu_up
    >>mu_down
    >>a_up
    >>a_down
    ;
	myMH.push_back(MHin);
    mysigma.push_back(Sigma);
    myerrorup.push_back(a_up);
    myerrordown.push_back(a_down);
    nlines++;
	
	if (!in.good()) break;
	}
in.close();

Double_t MH[100];
Double_t sigma[100];
Double_t errorup[100];
Double_t errordown[100];
Double_t xdown[100];
Double_t xup[100];
for (int i=0;i<nlines;i++)  
    {
    MH[i]=myMH[i];
    sigma[i]=mysigma[i];
    errorup[i]=myerrorup[i];
    errordown[i]=myerrordown[i];
    xdown[i]=0.0;
    xup[i]=0.0;
    }
    
TCanvas *c1 = new TCanvas("c1","c1",0,0,800,600);
c1->SetFillColor(kWhite);
c1->SetBorderMode(0);


gPad->SetLeftMargin(0.15);
gPad->SetLogy();
MHgraph=new TGraphAsymmErrors(nlines,MH,sigma,xdown,xup,errordown,errorup);
//MHgraph->SetMarkerColor(4);
//MHgraph->SetMarkerStyle(21);

MHgraph->SetTitle("#sigma (fb)");
MHgraph->GetYaxis()->SetTitle("#sigma (fb)");
MHgraph->GetXaxis()->SetTitle("m_{H} (GeV)");
MHgraph->SetFillStyle(3003);
MHgraph->SetFillColor(kRed);

//MHgraph->SetMinimum(1000);
//MHgraph->SetMaximum(100000);
MHgraph->Draw("a4");




c1->Update();
c1->GetFrame()->SetFillColor(kWhite);
c1->GetFrame()->SetBorderSize(0);
c1->GetFrame()->SetBorderMode(0);
c1->Modified();
c1->Print(\""""+epsfname+"""\");

 
}""")
        f.close()
#---------------------------------------------------------------------
    def WriteHistoDataFiles(self,thedata,dir_name,x_label):
        proper_histname = self.dists_translate[x_label]
        f2=open(dir_name+"/"+proper_histname+".data.txt",'w')
        for xx in thedata:
            f2.write('\n'+str(xx[0])+'\t'+str(xx[1])+'\t'+str(xx[2]))
        f2.close()
#---------------------------------------------------------------------
    def WriteRootFileFromData(self,thedata,dir_name,mass,x_label,output,epsfile):
        proper_histname = self.dists_translate[x_label]
        epsfname=dir_name+"/"+epsfile
        print epsfname
        f=open(dir_name+"/"+output+".C",'w')
        #finding out histogram max, min and bin size
        N=len(thedata)
        binsize = thedata[1][0] - thedata[0][0]
        xmin = thedata[0][0] - binsize / 2.0
        xmax = thedata[N-1][0]+ binsize / 2.0
        
        f.write("""\
#include<string>
using namespace std;

void """+output+"""()
{
double mh="""+str(mass)+""";
const int N="""+str(N)+""";   
const double xmin="""+str(xmin)+""";
const double xmax="""+str(xmax)+""";
gROOT->Reset();
gStyle->SetOptStat(0);
gStyle->SetPalette(1);
gStyle->SetCanvasColor(33);
gStyle->SetFrameFillColor(18);

Double_t sigma[N];
Double_t x[N];
Double_t error[N];
Double_t ex[N];
for (int i=0;i<N;i++){ex[i]=0.0;}
        """)
        for i,xx in enumerate(thedata):
            newline="\nx["+str(i)+"]="+str(xx[0])+";"+"\nsigma["+str(i)+"]="+str(xx[1])+";"+"\nerror["+str(i)+"]="+str(xx[2])+";"
            f.write(newline)
        f.write("""\


TCanvas *c1 = new TCanvas("c1","c1",0,0,800,600);
c1->SetFillColor(0);
c1->SetBorderMode(0);
c1->GetFrame()->SetFillColor(0);
c1->GetFrame()->SetBorderSize(0);
c1->GetFrame()->SetBorderMode(0);
   
gPad->SetLeftMargin(0.15);

histo1 = new TH1F("histo1","#sigma (fb) for m_{H}="""+str(mass)+"""\",N,xmin,xmax);
histo2 = new TH1F("histo2","#sigma (fb) for m_{H}="""+str(mass)+"""\",N,xmin,xmax);
histo3 = new TH1F("histo3","#sigma (fb) for m_{H}="""+str(mass)+"""\",N,xmin,xmax);
for (int i=0;i<N;i++)
    {
    histo1->Fill(x[i],sigma[i]);
    histo2->Fill(x[i],sigma[i]+error[i]);
    histo3->Fill(x[i],sigma[i]-error[i]);
    }

histo1->SetLineColor(kBlack);
histo2->SetLineColor(40);
histo3->SetLineColor(40);
histo2->SetFillStyle(3001);
histo1->SetFillStyle(3001);
histo2->Draw();
histo3->Draw("same");
histo1->Draw("same");

//histo1->SetMinimum(0.0);
//histo2->SetMinimum(0.0);
//histo3->SetMinimum(0.0);


histo1->GetYaxis()->SetTitleOffset(1.9);

histo1->SetTitle("#sigma (fb) for m_{H}="""+str(mass)+"""\");
histo1->GetYaxis()->SetTitle("#sigma (fb)");
histo1->GetXaxis()->SetTitle(\""""+x_label+"""\");

//gPad->SetLogy();





  
c1->Update();
c1->GetFrame()->SetFillColor(0);
c1->GetFrame()->SetBorderSize(0);
c1->GetFrame()->SetBorderMode(0);
c1->Modified();   
     
c1->Print(\""""+epsfname+"""\");

 
        }""")
        f.close()
#---------------------------------------------------------------------
    def WriteRootFile_x(self,dir_name,input_file,mass,x_label,output,epsfile):
        epsfname=dir_name+"/"+epsfile
        print epsfname
        f=open(dir_name+"/"+output+".C",'w')
        f.write("""\
#include<string>
using namespace std;

void """+output+"""()
{
string inputfilename="""+"\""+input_file+"""";
double mh="""+str(mass)+""";
   
gROOT->Reset();
gStyle->SetOptStat(0);
gStyle->SetPalette(1);
gStyle->SetCanvasColor(33);
gStyle->SetFrameFillColor(18);


ifstream in;
Int_t nlines;	
in.open(inputfilename.c_str());
nlines=0;	
for (int i=0;i<2;i++){string header;getline(in,header);}
double MuF,Sigma,Error_up,Error_down;
vector<double> mymuF,mysigma,myerrorup,myerrordown;
while (1) 
	{
	in 
	>>MuF
	>>Sigma	
	>>Error_up
    >>Error_down;
	mymuF.push_back(MuF);
    mysigma.push_back(Sigma);
    myerrorup.push_back(Error_up);
    myerrordown.push_back(Error_down);
    nlines++;
	
	if (!in.good()) break;
	}
in.close();

Double_t muF[100];
Double_t sigma[100];
Double_t errorup[100];
Double_t errordown[100];
Double_t xdown[100];
Double_t xup[100];
for (int i=0;i<nlines;i++)  
    {
    muF[i]=mymuF[i];
    sigma[i]=mysigma[i];
    errorup[i]=myerrorup[i];
    errordown[i]=myerrordown[i];
    xdown[i]=0.0;
    xup[i]=0.0;
    }
    
TCanvas *c1 = new TCanvas("c1","c1",0,0,800,600);
c1->SetFillColor(0);
c1->SetBorderMode(0);
c1->GetFrame()->SetFillColor(0);
c1->GetFrame()->SetBorderSize(0);
c1->GetFrame()->SetBorderMode(0);
   
gPad->SetLeftMargin(0.15);

muFgraph=new TGraphAsymmErrors(nlines,muF,sigma,xdown,xup,errordown,errorup);
muFgraph->SetMarkerColor(4);
muFgraph->SetMarkerStyle(21);

muFgraph->GetYaxis()->SetTitleOffset(1.9);

muFgraph->SetTitle("#sigma (fb) for m_{H}="""+str(mass)+"""\");
muFgraph->GetYaxis()->SetTitle("#sigma (fb)");
muFgraph->GetXaxis()->SetTitle(\""""+x_label+"""\");

//gPad->SetLogy();

//muFgraph->SetMinimum(0);
//muFgraph->SetMaximum(40000);
muFgraph->Draw("APC");



  
c1->Update();
c1->GetFrame()->SetFillColor(0);
c1->GetFrame()->SetBorderSize(0);
c1->GetFrame()->SetBorderMode(0);
c1->Modified();   
     
c1->Print(\""""+epsfname+"""\");

 
}""")
        f.close()

#---------------------------------------------------------------------
    def WriteInputfile(self):
        #print '::',sys._getframe().f_code.co_name
        self.param['muF']=str(float(self.param['muFrel'])*float(self.param['mh']))
        self.param['muR']=str(float(self.param['muRrel'])*float(self.param['mh']))
        f=open('%(rundir)s/input.txt' %(self.jobdefs), 'w') 
        f.write("""\
-------------------  SECTOR CHOICE (only NNLO) -----------
'NNLO contribution		    = ' %(NNLOcont)s
'NNLO sector			    = ' %(NNLOsect)s
-------------------  MASSES and WIDTH  -------------------
'Mass of the Higgs boson (GeV)      = ' %(mh)s
'Width Higgs (GeV) (0d0=Spira)	    = ' %(GammaH)s
'Mass of the W boson (GeV)          = ' %(mW)s
'Width of the W boson (GeV)         = ' %(GammaW)s
'Mass of the Z boson (GeV)          = ' %(mZ)s
'Width of the Z boson (GeV)         = ' %(GammaZ)s
'Top-quark mass (GeV)               = ' %(mt)s
'Bottom-quark mass (GeV)            = ' %(mb)s
'Bottom renormalization, 0=Pole, 1=MSbar =' %(imbscheme)s				    
'Fermi constant                     = ' 0.0000116639d0
----------------------  COLLIDER  ------------------------
'Collider (pp=0, ppbar=1)           = ' %(icollider)s
'CM collision energy (GeV)          = ' %(sqrtS)s
-----------------------  SCALES  -------------------------
'Factorization scale  (GeV)         = ' %(muF)s
'Renormalization scale  (GeV)       = ' %(muR)s
-----------------  MODEL PARAMETERS ----------------------
'ytrel=                             = '  %(ytrel)s
'ybrel=                             = '  %(ybrel)s
'0th order Wilson coefficion     C0 = '  %(C0user)s
'1st order Wilson coefficion     C1 = '  %(C1user)s
'2nd order Wilson coefficion     C2 = '  %(C2user)s
-----------------------  DECAY  --------------------------
'Fast inclusive comp. (0=no,1=yes)  = ' %(iinclusive)s
'Final-state (0=no, 1=e e nu_e nubar_e (WW only),'
' 2=e e nu_e nubar_e (WW/ZZ) , 3=eeee (ZZ), '
'      4= e e mu mu, 5= 2gamma )    = ' %(idecay)s
'Pert. Order (0=LO, 1=NLO, 2=NNLO)  = ' %(porder)s
'Massive corrections (0=none,1=yes) = ' %(imassive)s
'Electroweak corr.   (0=none,1=virtual,'
'2=virtual+real)                    = ' %(iewk)s
'Exclude W/Z width (0=no, 1=yes)    = ' %(exclWidth)s
'Higgs width Effects(0=no, 1=yes)   = ' %(include_higgs_width)s
------------------  VEGAS PARAMETER  ---------------------
(1=nvegas,11=Cuba Vegas, 12=Cuba Suave,13=Cuba Divonne,14=Cuba Cuhre) 
'Integration method                 = ' %(intMethod)s
'Relative accuracy (in %%)          = ' %(accRel)s
'Absolute accuracy                  = ' %(accAbs)s
'Number of therm. iterations        = ' %(ntherm)s
'Number of refin. iterations        = ' %(nrefin)s
'Number of calls per iteration      = ' %(ncalls)s
'Number of increase calls per iter. = ' %(nincr)s
'Max. run-time in sec (0=no limit)  = ' %(maxtime)s
'1=Adapt only to 1st int. 0=adapt to all (Cuba) = ' %(adaptofirst)s
----------------------- CONSTRAINT -----------------------
 consflag=0.0 (old para) =1.0 (new para)
'constraint flag		    = ' %(consflag)s
----------------------- PDF INFO ------------------------
'1.0: MRST2001, 2.0: MRST2004,        '
'3.0: MRST2006, 4.0: MSTW2008       = ' %(pdfset)s
'Only MSTW2008: 0.0: alphaS best fit, 1.0: alphas-da(90CL), 2.0: alphas+da(90CL)'
'3.0: alphas-sigma(68CL), 4.0: alphas+sigma(68CL) = ' %(whichalphas)s
'1: PDF error / 0: no PDF error     = ' %(pdferror)s
-------------------- JET DEFINITION ----------------------
' Cone size			    = ' %(JetConeSize)s
' Pt min			    = ' %(JetDefPtMin)s
' |eta| max			    = ' %(JetDefEtaMax)s
' use SISCone                       = ' %(UseSISCone)s
------------------------  CUTS  --------------------------
 Cut on  (active=1d0, inactive=0d0)
' lepton trigger specification      = ' %(LepTriggerSpec)s
'                           pt min  = ' %(LepPtMin)s
'                         |eta| max = ' %(LepEtaMax)s
' cut min pT of harder lepton	    = ' %(LepHardMinPtCut)s
'			     value  = ' %(LepHardPtMinVal)s
' cut max pT of harder lepton	    = ' %(LepHardMaxPtCut)s
'			     value  = ' %(LepHardPtMaxVal)s	
' cut min pT of softer lepton	    = ' %(LepSoftMinPtCut)s
'			     value  = ' %(LepSoftPtMinVal)s	
' cut max pT of softer lepton	    = ' %(LepSoftMaxPtCut)s
'			     value  = ' %(LepSoftPtMaxVal)s
' cut max |eta| for harder lepton   = ' %(LepHardMaxEtaCut)s
'			     value  = ' %(LepHardEtaMaxVal)s
' cut max |eta| for softer lepton   = ' %(LepSoftMaxEtaCut)s
'			     value  = ' %(LepSoftEtaMaxVal)s
' Isolation			    = '	%(IsolationCut)s
'			 cone size  = ' %(IsoConeSize)s
'	           energy fraction  = ' %(IsoEnergyFrac)s
' Jet-Veto			    = ' %(JetVeto)s	
'		     jet cone size  = ' %(JetConeSize)s
'		        jet pt max  = ' %(JetPtMax)s
'		     jet |eta| max  = ' %(JetEtaMax)s
' Num-Veto			    = ' %(NumVeto)s	
'		     max num. Jets  = ' %(JetNmax)s
' TeV-Veto			    = ' %(TeVVeto)s
'		       ptMax Jet 1  = ' %(TeVJetPtMax1)s
'		       ptMax Jet 2  = ' %(TeVJetPtMax2)s
' Cut on PhiLL			    = ' %(PhiLLCut)s
'			 min angle  = '	%(PhiLLMin)s
'			 max angle  = '	%(PhiLLMax)s
' Cut on missing Pt		    = '	%(PtMissCut)s
'		   min. missing pt  = ' %(PtMissMin)s
'		   max. missing pt  = ' %(PtMissMax)s
' Cut on Inv. mass		    = '	%(InvMassCut)s
'		    min. inv. mass  = ' %(InvMassMin)s
'		    max. inv. mass  = ' %(InvMassMax)s
' Cut on Sum Pt	    		= '	%(SumPtCut)s
'		    min. inv. mass  = ' %(SumPtMin)s
'		    max. inv. mass  = ' %(SumPtMax)s
' Cut on Special MET		= '	%(SpecMETCut)s
'		    min. spec. MET  = ' %(SpecMETMin)s
--------------------- MORE CUTS --------------------------
' Cut on mass V1		    = '	%(MassV1Cut)s
'                      min mass V2  = ' %(MassV1Min)s
'                      max mass V2  = ' %(MassV1Max)s
' Cut on mass V2		    = '	%(MassV2Cut)s
'                      min mass V2  = ' %(MassV2Min)s
'                      max mass V2  = ' %(MassV2Max)s
' Cut on pT VB1			    = '	%(PtV1Cut)s
'                       min pT VB1  = ' %(PtV1Min)s
'                       max pT VB1  = ' %(PtV1Max)s
' Cut on pT VB2			    = '	%(PtV2Cut)s
'                       min pT VB2  = ' %(PtV2Min)s
'                       max pT VB2  = ' %(PtV2Max)s
' Cut on eta VB1		    = '	%(EtaV1Cut)s
'                      min eta VB1  = ' %(EtaV1Min)s
'                      max eta VB1  = ' %(EtaV1Max)s
' Cut on eta VB2		    = '	%(EtaV2Cut)s
'                      min eta VB2  = ' %(EtaV2Min)s
'                      max eta VB2  = ' %(EtaV2Max)s
' Cut on pT Higgs		    = '	%(PtHiggsCut)s
'                     min pT Higgs  = ' %(PtHiggsMin)s
'                     max pT Higgs  = ' %(PtHiggsMax)s
' Cut on eta Higgs	  	    = '	%(EtaHiggsCut)s
'                    min eta Higgs  = ' %(EtaHiggsMin)s
'                    max eta Higgs  = ' %(EtaHiggsMax)s
=========== CUTS FOR H->GAMMA GAMMA =========
' Basic photon cut on pt, eta       = ' %(PhotBasicCut)s
'                    min pT gamma 1 = ' %(PhotPtMin1)s
'                    min pT gamma 2 = ' %(PhotPtMin2)s
'                    max eta diff   = ' %(PhotEtaMax)s
' Isolation Cut for photons         = ' %(PhotIsolCut)s
'                    Isolation cone = ' %(PhotConeIso)s
'                             Etmax = ' %(PhotEtmax)s
' Cut on ptm	 	  	    = '	%(PtmCut)s
'                    min ptm	    = ' %(PtmMin)s
'                    max ptm        = ' %(PtmMax)s
' Cut on ystar	 	  	    = '	%(YstarCut)s
'                    min ystar	    = ' %(YstarMin)s
'                    max ystar	    = ' %(YstarMax)s
' Cut on min pt soft 	  	    = '	%(PtSoftCut)s
'                    min ptsoft	    = ' %(PtSoftMin)s
' Cut on eta of harder photon       = ' %(PhotEtaHardCut)s
'                           min eta = ' %(PhotEtaHardMin)s
'                           max eta = ' %(PhotEtaHardMax)s
' BRANCHING RATIO H->gg		    = ' %(BRgg)s
=========== Cuts for ZZ-> e e mu mu
'       cut_on_pt_of_all_leptons = ' %(cut_on_pt_of_all_leptons)s
'         all_leptons_minimum_pt = ' %(all_leptons_minimum_pt)s
' cut_on_rapidity_of_all_leptons = ' %(cut_on_rapidity_of_all_leptons)s
'          all_leptons_maximum_y = ' %(all_leptons_maximum_y)s
""" % self.param)
        f.close()


