#include "TMVAnalysis_MLP.class.h"


ReadMLP::ReadMLP( std::vector<std::string>& theInputVars ) 
  : IClassifierReader(),
    fClassName( "ReadMLP" ),
    fStatusIsClean( true ),
    fNvars( 5 ),
    fIsNormalised( true )
{      
  // the training input variables
  const char* inputVars[] = { "pthard", "ptsoft", "mll", "met", "phill" };
  
  // sanity checks
  if (theInputVars.size() <= 0) {
    std::cout << "Problem in class \"" << fClassName << "\": empty input vector" << std::endl;
         fStatusIsClean = false;
  }
  
  if (theInputVars.size() != fNvars) {
    std::cout << "Problem in class \"" << fClassName << "\": mismatch in number of input values: "
	      << theInputVars.size() << " != " << fNvars << std::endl;
    fStatusIsClean = false;
  }
  
  // validate input variables
  for (size_t ivar = 0; ivar < theInputVars.size(); ivar++) {
    if (theInputVars[ivar] != inputVars[ivar]) {
      std::cout << "Problem in class \"" << fClassName << "\": mismatch in input variable names" << std::endl
		<< " for variable [" << ivar << "]: " << theInputVars[ivar].c_str() << " != " << inputVars[ivar] << std::endl;
      fStatusIsClean = false;
    }
  }
  
  // initialize min and max vectors (for normalisation)
  fVmin[0] = 20.0100002288818;
  fVmax[0] = 329.829986572266;
  fVmin[1] = 10;
  fVmax[1] = 289.200012207031;
  fVmin[2] = 16;
  fVmax[2] = 774.200012207031;
  fVmin[3] = 25;
  fVmax[3] = 297.670013427734;
  fVmin[4] = 0;
  fVmax[4] = 3.14000010490417;
  
  
  // initialize input variable types
  fType[0] = 'F';
  fType[1] = 'F';
  fType[2] = 'F';
  fType[3] = 'F';
  fType[4] = 'F';
  
  // initialize constants
  Initialize();
  
}

// destructor
ReadMLP::~ReadMLP() {
  Clear(); // method-specific
}

inline void ReadMLP::Initialize()
{

  // build network structure
  fLayers = 4;
  fLayerSize[0] = 6; fWeights[0] = new double[6]; 
  fLayerSize[1] = 7; fWeights[1] = new double[7]; 
  fLayerSize[2] = 6; fWeights[2] = new double[6]; 
  fLayerSize[3] = 1; fWeights[3] = new double[1]; 
  // weight matrix from layer 0 to 1
   fWeightMatrix0to1[0][0] = 2.13183738260833;
   fWeightMatrix0to1[1][0] = 2.03568810230241;
   fWeightMatrix0to1[2][0] = -1.80519378970817;
   fWeightMatrix0to1[3][0] = -0.983905878030222;
   fWeightMatrix0to1[4][0] = -1.89965693941526;
   fWeightMatrix0to1[5][0] = -9.37694497676898;
   fWeightMatrix0to1[0][1] = -3.6389418473014;
   fWeightMatrix0to1[1][1] = 3.00994636980396;
   fWeightMatrix0to1[2][1] = -6.46219157770388;
   fWeightMatrix0to1[3][1] = -14.2905357848742;
   fWeightMatrix0to1[4][1] = -3.3857250362435;
   fWeightMatrix0to1[5][1] = 0.216601233254802;
   fWeightMatrix0to1[0][2] = -2.74568992050789;
   fWeightMatrix0to1[1][2] = 0.635638309193244;
   fWeightMatrix0to1[2][2] = 8.74718626436912;
   fWeightMatrix0to1[3][2] = 7.48600764485621;
   fWeightMatrix0to1[4][2] = -10.7056395602625;
   fWeightMatrix0to1[5][2] = -4.04473672061449;
   fWeightMatrix0to1[0][3] = 2.39275298907234;
   fWeightMatrix0to1[1][3] = 1.22902561658285;
   fWeightMatrix0to1[2][3] = -0.701068811522203;
   fWeightMatrix0to1[3][3] = -2.14269307492161;
   fWeightMatrix0to1[4][3] = -1.78404143354339;
   fWeightMatrix0to1[5][3] = -6.47382615165004;
   fWeightMatrix0to1[0][4] = -0.132023721678352;
   fWeightMatrix0to1[1][4] = -0.478292132558779;
   fWeightMatrix0to1[2][4] = 1.26156151303706;
   fWeightMatrix0to1[3][4] = -0.899673119953077;
   fWeightMatrix0to1[4][4] = 1.0803056386317;
   fWeightMatrix0to1[5][4] = 0.226498434440235;
   fWeightMatrix0to1[0][5] = -0.40194202566066;
   fWeightMatrix0to1[1][5] = -0.390724743793674;
   fWeightMatrix0to1[2][5] = 0.286850262189489;
   fWeightMatrix0to1[3][5] = -8.81134167060212;
   fWeightMatrix0to1[4][5] = -15.0461856052312;
   fWeightMatrix0to1[5][5] = -14.5774951517011;
   // weight matrix from layer 1 to 2
   fWeightMatrix1to2[0][0] = -0.38455779248316;
   fWeightMatrix1to2[1][0] = 1.72343573646472;
   fWeightMatrix1to2[2][0] = -1.13540951020243;
   fWeightMatrix1to2[3][0] = 0.542716169176011;
   fWeightMatrix1to2[4][0] = -0.584555298662726;
   fWeightMatrix1to2[0][1] = -0.339412897739418;
   fWeightMatrix1to2[1][1] = 1.53199627581411;
   fWeightMatrix1to2[2][1] = 1.60652820756524;
   fWeightMatrix1to2[3][1] = -0.233416459513534;
   fWeightMatrix1to2[4][1] = -1.8260279669613;
   fWeightMatrix1to2[0][2] = 0.532092829420055;
   fWeightMatrix1to2[1][2] = -0.697721609225707;
   fWeightMatrix1to2[2][2] = -0.408059548265484;
   fWeightMatrix1to2[3][2] = -0.0144031237266305;
   fWeightMatrix1to2[4][2] = 1.11084840787449;
   fWeightMatrix1to2[0][3] = 2.0753649650008;
   fWeightMatrix1to2[1][3] = 0.0763709125322826;
   fWeightMatrix1to2[2][3] = -0.0858726926590183;
   fWeightMatrix1to2[3][3] = 0.305523723818626;
   fWeightMatrix1to2[4][3] = -0.256630649003158;
   fWeightMatrix1to2[0][4] = -1.20204953026915;
   fWeightMatrix1to2[1][4] = -0.763849779249615;
   fWeightMatrix1to2[2][4] = 3.01005442231239;
   fWeightMatrix1to2[3][4] = -0.00934075261691859;
   fWeightMatrix1to2[4][4] = -1.38166028623962;
   fWeightMatrix1to2[0][5] = -0.149352695493157;
   fWeightMatrix1to2[1][5] = 1.57734635654089;
   fWeightMatrix1to2[2][5] = 0.192897238769754;
   fWeightMatrix1to2[3][5] = 0.229627516889678;
   fWeightMatrix1to2[4][5] = -1.71874389671975;
   fWeightMatrix1to2[0][6] = 1.91592798546713;
   fWeightMatrix1to2[1][6] = -1.2242887970315;
   fWeightMatrix1to2[2][6] = 0.394161869952418;
   fWeightMatrix1to2[3][6] = -0.806664839522145;
   fWeightMatrix1to2[4][6] = -0.200586388494685;
   // weight matrix from layer 2 to 3
   fWeightMatrix2to3[0][0] = -0.324348215111288;
   fWeightMatrix2to3[0][1] = 0.270083742211988;
   fWeightMatrix2to3[0][2] = -0.310582496418718;
   fWeightMatrix2to3[0][3] = 0.105248347183458;
   fWeightMatrix2to3[0][4] = -0.535354987585187;
   fWeightMatrix2to3[0][5] = -0.114473366130915;

}

inline double ReadMLP::GetMvaValue__( const std::vector<double>& inputValues ) const
{
   if (inputValues.size() != (unsigned int)fLayerSize[0]-1) {
      std::cout << "Input vector needs to be of size " << fLayerSize[0]-1 << std::endl;
      return 0;
   }

   for (int l=0; l<fLayers; l++)
      for (int i=0; i<fLayerSize[l]; i++) fWeights[l][i]=0;

   for (int l=0; l<fLayers-1; l++)
      fWeights[l][fLayerSize[l]-1]=1;

   for (int i=0; i<fLayerSize[0]-1; i++)
      fWeights[0][i]=inputValues[i];

   // layer 0 to 1
   for (int o=0; o<fLayerSize[1]-1; o++) {
      for (int i=0; i<fLayerSize[0]; i++) {
         double inputVal = fWeightMatrix0to1[o][i] * fWeights[0][i];
         fWeights[1][o] += inputVal;
      }
      fWeights[1][o] = ActivationFnc(fWeights[1][o]);
   }
   // layer 1 to 2
   for (int o=0; o<fLayerSize[2]-1; o++) {
      for (int i=0; i<fLayerSize[1]; i++) {
         double inputVal = fWeightMatrix1to2[o][i] * fWeights[1][i];
         fWeights[2][o] += inputVal;
      }
      fWeights[2][o] = ActivationFnc(fWeights[2][o]);
   }
   // layer 2 to 3
   for (int o=0; o<fLayerSize[3]; o++) {
      for (int i=0; i<fLayerSize[2]; i++) {
         double inputVal = fWeightMatrix2to3[o][i] * fWeights[2][i];
         fWeights[3][o] += inputVal;
      }
   }

   return fWeights[3][0];
}

double ReadMLP::ActivationFnc(double x) const
{
   // hyperbolic tan
   return tanh(x);
}

// Clean up
inline void ReadMLP::Clear() 
{
  // nothing to clear
  delete[] fWeights[0];
  delete[] fWeights[1];
  delete[] fWeights[2];
  delete[] fWeights[3];
}

inline double ReadMLP::GetMvaValue( const std::vector<double>& inputValues ) const
{
  // classifier response value
  double retval = 0;

   // classifier response, sanity check first
   if (!fStatusIsClean) {
      std::cout << "Problem in class \"" << fClassName << "\": cannot return classifier response"
                   << " because status is dirty" << std::endl;
      retval = 0;
   }
   else {
      if (IsNormalised()) {
         // normalise variables
         std::vector<double> iV;
         int ivar = 0;
         for (std::vector<double>::const_iterator varIt = inputValues.begin();
              varIt != inputValues.end(); varIt++, ivar++) {
            iV.push_back(NormVariable( *varIt, fVmin[ivar], fVmax[ivar] ));
         }
         retval = GetMvaValue__( iV );
      }
      else {
         retval = GetMvaValue__( inputValues );
      }
   }

   return retval;
}
