C =============================
      subroutine filllsfuncs(theta,acl1,acl2,als1,als2,als3,
     $                       alsc1,alsc2,alsc3,alslsc1,alslsc2,alslsc3)
C fills arrays with values for Cl,Ls,Lsc and LsLsc functions
C acl1 = Cl(1,theta)
C acl2 = Cl(1,theta-Pi)
C als1(j,k) = Ls(j,k,theta)
C als2(j,k) = Ls(j,k,theta-Pi)
C als3(j,k) = Ls(j,k,2*theta)
C alsc1(i,j) = Lsc(i,j,theta)
C alsc2(i,j) = Lsc(i,j,theta-Pi)
C alsc3(i,j) = Lsc(i,j,2*theta)
      implicit none
      integer i,j,k,dummy,maxiter                       
      double precision clausen1,lsc,lsjk,lslsc112
      
      include 'auxini.f'
      
      maxiter=1000
      acl1=clausen1(theta)
      acl2=clausen1(theta-PI)
      do i=2,4
         do k=0,i-2
            als1(i,k)=lsjk(i,k,theta,maxiter,dummy)
            als2(i,k)=lsjk(i,k,theta-pi,maxiter,dummy)
            als3(i,k)=lsjk(i,k,2.D0*theta,maxiter,dummy)
         enddo
      enddo
      do i=2,3
         do j=2,3
            alsc1(i,j)=lsc(i,j,theta,maxiter,dummy)
            alsc2(i,j)=lsc(i,j,theta-pi,maxiter,dummy)
            alsc3(i,j)=lsc(i,j,2.D0*theta,maxiter,dummy)
         enddo
      enddo
      alslsc1=lslsc112(theta,maxiter,dummy)
      return
      end
C =============================
      subroutine filloneloopintT(s,t,u,mh,mq,bubs,bubt,bubu,bubmh,
     &     trians,triant,trianu,trianmh,boxTst,boxTsu,boxTtu)

C     input: s,t,u,mh,mq
C     output: bubs,bubt,bubu,bubmh,trians,triant,trianu,trianmh,
C             boxTst,boxTsu,boxTtu
C     bubs=Bub1(s,mq**2)
C     bubt=Bub1(t,mq**2)
C     bubu=Bub1(u,mq**2)
C     bubmh=Bub1(mh**2,mq**2)
C     and similar for trian
C     boxTst=BoxT(s,t)
C     boxTsu=BoxT(s,u)
C     boxTtu=BoxT(t,u)
      implicit none
      double precision s,t,u
      double precision mh,mq
      double complex bubs,bubt,bubu,bubmh
      double complex trians,triant,trianu,trianmh
      double complex boxTst,boxTsu,boxTtu
      double complex Bub1,TriT2,BoxT

      external Bub1,TriT2,BoxT


      bubs=-Bub1(s,mq**2)
      bubt=-Bub1(t,mq**2)
      bubu=-Bub1(u,mq**2)
      bubmh=-Bub1(mh**2,mq**2)
      
      trians=-TriT2(s,mq**2)
      triant=-TriT2(t,mq**2)
      trianu=-TriT2(u,mq**2)
      trianmh=-TriT2(mh**2,mq**2)  

      boxTst=BoxT(s,t,mh**2,mq**2)
      boxTsu=BoxT(s,u,mh**2,mq**2)
      boxTtu=BoxT(t,u,mh**2,mq**2)


c$$$      write(6,*)'================='
c$$$      write(6,*)bubs,bubt
c$$$      write(6,*)bubu,bubmh
c$$$      write(6,*)trians,triant
c$$$      write(6,*)trianu,trianmh
c$$$      write(6,*)boxTst,boxTsu,boxTtu

      return
      end





C =============================
      subroutine filloneloopintV(s,t,u,mh,mv,bub1mh,bub2s,bub2t,bub2u,
     &     tris,trit,triu,boxst,boxsu,boxtu)
C     input: s,t,u,mh,mv
      implicit none
      double precision s,t,u
      double precision mh,mv
      double complex bub1mh,bub2s,bub2t,bub2u
      double complex tris,trit,triu
      double complex boxst,boxsu,boxtu

      double complex Bub1,Bub2,Tri,Box
      external Bub1,Bub2,Tri,Box


      bub1mh=Bub1(mh**2,mv**2)

      bub2s=Bub2(s,mv**2)
      bub2t=Bub2(t,mv**2)
      bub2u=Bub2(u,mv**2)

      tris=Tri(s,mh**2,mv**2)
      trit=Tri(t,mh**2,mv**2)
      triu=Tri(u,mh**2,mv**2)

      boxst=Box(s,t,mh**2,mv**2)
      boxsu=Box(s,u,mh**2,mv**2)
      boxtu=Box(t,u,mh**2,mv**2)

      return
      end












C =============================     
C     Daleo's Li2 functions, modified by S. Bucherer
      double complex function bLi2(z)
      implicit none
      double complex z,zz,zsig,zxn,zxni


      double precision PI
      parameter( PI = 3.14159265358979323846264d0 )

      double complex myLi2s
      external myLi2s

      zsig=dcmplx(1d0,0d0)

      bLi2=dcmplx(0d0,0d0)
      if ((dreal(z).eq.1d0).and.(dimag(z).eq.0d0)) then
         bLi2=PI**2/6d0
         goto 210
      elseif ((dreal(z).eq.-1d0).and.(dimag(z).eq.0d0)) then
         bLi2=-PI**2/12d0
         goto 210
      elseif (dreal(z).gt.1d0) then
         bLi2=PI**2/3d0-1d0/2d0*zlog(z)**2
         zz=1d0/z
         zsig=dcmplx(-1d0,0d0)
         if (dimag(z).gt.0d0) then
            bLi2=bLi2+dcmplx(0d0,1d0)*PI*zlog(z)
         else
            bLi2=bLi2-dcmplx(0d0,1d0)*PI*zlog(z)
         endif
      elseif ((zabs(z).gt.1d0)) then
         bLi2=-PI**2/6d0-1d0/2d0*zlog(-z)**2
         zz=1d0/z
         zsig=dcmplx(-1d0,0d0)
      else
         zz=z
         zsig=dcmplx(1d0,0d0)
      endif

      if(dreal(zz).gt.0.5d0) then
         bLi2=bLi2+zsig*(PI**2/6d0-zlog(zz)*zlog(1d0-zz))
         zz=1d0-zz
         zsig=dcmplx(-1d0,0d0)*zsig
      endif

      bLi2=bLi2+zsig*myLi2s(zz)
     
 210  continue

      return
      end

C =============================     
      double complex function myLi2s(z)
      implicit none
      double complex z,zxn,zz

      double precision coeff(38)
      integer i,nmax
      parameter(nmax=16)

      DATA coeff / 
     .  1d0,
     .  -0.25d0,
     .  0.0277777777777777777777778d0,
     .  0d0,
     .  -0.000277777777777777777777778d0,
     .  0d0,
     .  4.72411186696900982615268d-6,
     .  0d0,
     .  -9.18577307466196355085244d-8,
     .  0d0,
     .  1.89788699889709990720092d-9,
     .  0d0,
     .  -4.06476164514422552680591d-11,
     .  0d0,
     .  8.92169102045645255521799d-13,
     .  0d0,
     .  -1.99392958607210756872364d-14,
     .  0d0,
     .  4.51898002961991819165048d-16,
     .  0d0,
     .  -1.03565176121812470144834d-17,
     .  0d0,
     .  2.39521862102618674574028d-19,
     .  0d0,
     .  -5.58178587432500933628307d-21,
     .  0d0,
     .  1.30915075541832128581231d-22,
     .  0d0,
     .  -3.08741980242674029324228d-24,
     .  0d0,
     .  7.31597565270220342035791d-26,
     .  0d0,
     .  -1.74084565723400074098906d-27,
     .  0d0,
     .  4.1576356446138997196179d-29,
     .  0d0,
     .  -9.96214848828462210319401d-31,
     .  0d0/

  
      zxn=-zlog(1d0-z)
      myLi2s=coeff(nmax)
      do i=nmax-1,1,-1
         myLi2s=zxn*myLi2s+coeff(i)
      enddo
      myLi2s=zxn*myLi2s
      return
      end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCC Loop Ints CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      double complex function Bub1(s,m2)
      double precision s,m2
      double precision pi,tau

C      pi = 3.1415926535d0
      pi = 3.1415926535897932385d0

      if (s.lt.0d0) then
         tau=dsqrt(1+4*m2/dabs(s))
         Bub1=dcmplx(-2+tau*dlog((tau+1)/(tau-1)),0d0) 
      elseif (s.lt.(4d0*m2)) then
         tau = dsqrt(4*m2/s-1)
         Bub1 = dcmplx(-2+2*tau*datan(1/tau),0d0)
      else
         tau = dsqrt(1-4*m2/s)
         Bub1 = dcmplx(-2-tau*dlog((1-tau)/(1+tau)),
     &            -pi*tau)
      endif


C      write(6,*)'Bub1= ',s,m2,Bub1

      RETURN
      END

      double complex function Bub2(t,m2)
      double precision t,m2
      double complex cm2
      double complex bLi2
      external bLi2

CCC   Account for the analytic continuation by giving m2
CCC   a small negative imaginary part

      cm2 = dcmplx(m2,-0.000000001d0)

      Bub2 = 4-(1-t/cm2)/t*cm2*zlog(1-t/cm2)**2/2+2*(1-t/cm2)/t*cm2*zlog
     $(1-t/cm2)+(1-t/cm2)/t*cm2*bLi2((-t/cm2)/(1-t/cm2))

C      write(6,*)'Bub2= ',t,m2,Bub2

      RETURN
      END

      double complex function Tri(t,mh2,mv2)
      double precision t,mh2,mv2
      double complex cmv2,tt1,tt2,tt3,a,b,xp,xm
      double complex term1,term2,term3,term4,term5,a1
      double precision pi,tol
      double complex bLi2
      external bLi2

CCC   Account for the analytic continuation by giving m2
CCC   a small negative imaginary part

      pi = 3.1415926535897932385d0
      tol=0.01d0  

      cmv2 = dcmplx(mv2,-0.0000000001d0)
   
      tt1 = bLi2((mh2-t)/cmv2)

      tt2 = zlog(t*(mh2-t-cmv2)/cmv2/(mh2-t))**2/2-dlog(-t/(mh2-t))**2/2
     $-bLi2(1/cmv2*(cmv2*mh2-t*mh2+t**2)/t)+bLi2(-(cmv2*mh2-t*mh2+t**2)/
     $t/(mh2-t-cmv2))

c$$$      write(6,*)'1: ',zlog(t*(mh2-t-cmv2)/cmv2/(mh2-t))**2/2
c$$$      write(6,*)'2: ',dlog(-t/(mh2-t))**2/2,-t/(mh2-t)
c$$$      write(6,*)'3: ',bLi2(1/cmv2*(cmv2*mh2-t*mh2+t**2)/t)
c$$$      write(6,*)'4: ',bLi2(-(cmv2*mh2-t*mh2+t**2)/t/(mh2-t-cmv2))

      b = cmv2/(mh2-t)
      a = mh2/cmv2
      xp = 0.5d0*(1+zsqrt(1-4d0/a))
      xm = 0.5d0*(1-zsqrt(1-4d0/a))

      term1=zlog(a*b**2-a*b+1)*zlog((b-1)/b)

      a1=(1-b)/(xm-b)
      term2=bLi2(a1)
      
      a1=(1-b)/(xp-b)
      term3=bLi2(a1)

      a1=-b/(xm-b)
      term4=bLi2(a1)

      a1=-b/(xp-b)
      term5=bLi2(a1)

      tt3 = -(term1-term2-term3+term4+term5)

      Tri=1d0/(mh2-t)*(tt1+tt2+tt3)

      
C      write(6,*)'Tri= ',t,mh2,mv2,Tri
C      write(6,*)'     ',tt1,tt2,tt3
      RETURN
      END

      double complex function Box(t,u,mh2,m2)
      double precision t,u,mh2,m2
      double complex cm2,tt1,tt2,tt31,tt32u,tt32t,a,b,xp,xm
      double complex term1,term2,term3,term4,term5,a1
      double precision pi,tol
      double complex bLi2
      external bLi2
 
      pi = 3.1415926535897932385d0
      tol=0.01d0  

CCC   Account for the analytic continuation by giving m2
CCC   a small negative imaginary part

      cm2 = dcmplx(m2,-0.0000000001d0)

      tt1 = (bLi2(-t/(-t+cm2))-bLi2(u/cm2))
      
      tt2 = -zlog(1-t/cm2)-zlog(1-u/cm2)-0.5d0*zlog(1-t/cm2)**2-0.5d0
     $*zlog(1-u/cm2)**2-bLi2(-u**2/(u*t-cm2*(u+t)))-bLi2(-t**2/(u*t-cm2*
     $(u+t)))+2d0*bLi2(u*t/(u*t-cm2*(u+t)))

      tt31 = zlog(1-u/cm2)*zlog(u/(cm2*t-u*t+cm2*u)*cm2)-zlog(1-u/cm2)**
     $2/2+bLi2(t*(cm2-u)/(cm2*t-u*t+cm2*u))-bLi2(1/(cm2*t-u*t+cm2*u)*cm2
     $*t)

      b=cm2/u
      a = mh2/cm2
      xp = 0.5d0*(1+zsqrt(1-4d0/a))
      xm = 0.5d0*(1-zsqrt(1-4d0/a))

      term1=zlog(a*b**2-a*b+1)*zlog((b-1)/b)

      a1=(1-b)/(xm-b)
      term2=bLi2(a1)

      a1=(1-b)/(xp-b)
      term3=bLi2(a1)

      a1=-b/(xm-b)
      term4=bLi2(a1)

      a1=-b/(xp-b)
      term5=bLi2(a1)

      tt32u = (term1-term2-term3+term4+term5)

      b=cm2/t
      a = mh2/cm2
      xp = 0.5d0*(1+zsqrt(1-4d0/a))
      xm = 0.5d0*(1-zsqrt(1-4d0/a))

      term1=zlog(a*b**2-a*b+1)*zlog((b-1)/b)

      a1=(1-b)/(xm-b)
      term2=bLi2(a1)

      a1=(1-b)/(xp-b)
      term3=bLi2(a1)

      a1=-b/(xm-b)
      term4=bLi2(a1)

      a1=-b/(xp-b)
      term5=bLi2(a1)

      tt32t = (term1-term2-term3+term4+term5)

      Box=1d0/(cm2*t-u*t+cm2*u)*(tt1+tt2+tt31+tt32u+tt32t)

C      write(6,*)'Box= ',t,u,mh2,m2,Box

      RETURN
      END
 

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCC   Triangles for the top quark piece

      double complex function TriT(s,mt2,mh2)
      double precision s,mt2,mh2
      double complex cmt2,taus,tauh,xp1,xm1,xp2,xm2
      double complex bLi2
      external bLi2

CCC   Account for the analytic continuation by giving mt2
CCC   a small negative imaginary part
      cmt2 = dcmplx(mt2,-0.0000000001d0)
      taus = zsqrt(4*cmt2/s-1)
      tauh = zsqrt(4*cmt2/mh2-1)

      xp1 = 0.5d0*(1+dcmplx(0d0,1d0)*taus)
      xm1 = 0.5d0*(1-dcmplx(0d0,1d0)*taus)
      xp2 = 0.5d0*(1+dcmplx(0d0,1d0)*tauh)
      xm2 = 0.5d0*(1-dcmplx(0d0,1d0)*tauh)

      TriT = -1d0/(mh2-s)*(bLi2(1d0/xp1)+bLi2(1d0/xm1)
     &  -bLi2(1d0/xp2)-bLi2(1d0/xm2))

C      write(6,*)'TriT= ',s,mt2,mh2,TriT

      RETURN
      END

      double complex function TriT2(s,mt2)
      double precision s,mt2
      double complex cmt2,taus,xp,xm
      double complex bLi2
      external bLi2

CCC   Account for the analytic continuation by giving mt2
CCC   a small negative imaginary part
      cmt2 = dcmplx(mt2,-0.0000000001d0)
      taus = zsqrt(4*cmt2/s-1)

      xp = 0.5d0*(1+dcmplx(0d0,1d0)*taus)
      xm = 0.5d0*(1-dcmplx(0d0,1d0)*taus)

      TriT2 = 1d0/s*(bLi2(1/xp)+bLi2(1/xm))
C      write(6,*)'TriT2= ',s,mt2,TriT2
      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCC   Box for the top quark piece
CCC   Valid for s,t either both negative or one positive

      double complex function BTaux(s,t,mh2,mt2,gam)
      double precision s,t,mt2,mh2
      double precision u,gam,a,sigm,sigp,pi
      double complex cmt2,x3m,x3p,t1a,t2a,t3a,t4a,t5a,t6a,t7a,t8a,
     & yp,ym,sp,tp
      double complex bLi2
      external bLi2

      pi = 3.14159265358979d0

CCC   Account for the analytic continuation by giving mt2
CCC   a small negative imaginary part
      cmt2 = dcmplx(mt2,-0.000000001d0)

      u = -s-t+mh2

      x3m = -t/2/u*(1+sqrt(1+4*u*cmt2/t/s))
      x3p = -t/2/u*(1-sqrt(1+4*u*cmt2/t/s))

      a = t/s
      sigm = DIMAG(x3m)/dabs(DIMAG(x3m))
      sigp = DIMAG(x3p)/dabs(DIMAG(x3p))


CCC   Terms with gam do not contribute

      t1a = zlog((1+a)*(1-x3m))**2/2-zlog(-x3m*(1+a))**2/2+bLi2((a-x3m*(
     $1+a))/(1+a)/(1-x3m))-bLi2(1-a/(1+a)/x3m)-zlog(1-x3m)**2/2+zlog(-x3
     $m)**2/2-bLi2(-x3m/(1-x3m))+0.3141592653589793D1**2/6

      if (a.gt.0d0) then
         t1a=t1a+dcmplx(0,pi)*(sigm-0*gam)*zlog(1-a/(1+a)/x3m)
      endif

C      if (a.gt.0d0) then
C         t1a=t1a+0.5d0*(zlog(-a+x3m*(1+a))**2-zlog(x3m*(1+a))**2
C     $      -zlog(a-(1+a)*x3m)**2+zlog(-x3m*(1+a))**2)
C      endif


      t2a = -(
     $zlog((1+a)*(1-x3p))**2/2-zlog(-x3p*(1+a))**2/2+bLi2((a-x3p*(1
     $+a))/(1+a)/(1-x3p))-bLi2(1-a/(1+a)/x3p)-zlog(1-x3p)**2/2+zlog(-x3p 
     $)**2/2-bLi2(-x3p/(1-x3p))+0.3141592653589793D1**2/6)

      if (a.gt.0d0) then
         t2a=t2a-dcmplx(0,pi)*(sigp-0*gam)*zlog(1-a/(1+a)/x3p)
      endif

C      if (a.gt.0d0) then
C         t2a=t2a-0.5d0*(zlog(-a+x3p*(1+a))**2-zlog(x3p*(1+a))**2
C     $      -zlog(a-(1+a)*x3p)**2+zlog(-x3p*(1+a))**2)
C      endif

      yp = cmt2/2/mh2*(mh2/cmt2+sqrt(mh2**2/cmt2**2-4*mh2/cmt2))
      ym = cmt2/2/mh2*(mh2/cmt2-sqrt(mh2**2/cmt2**2-4*mh2/cmt2))

      t5a = -zlog(mh2/cmt2)*zlog(-(1-x3m)/x3m)-zlog(x3m-yp)*zlog(1-1/x3m
     $)+bLi2((1-x3m)/(yp-x3m))-bLi2(-x3m/(yp-x3m))-zlog(x3m-ym)*zlog(1-1
     $/x3m)+bLi2((1-x3m)/(ym-x3m))-bLi2(-x3m/(ym-x3m))

      yp = cmt2/2/s*(s/cmt2+sqrt(s**2/cmt2**2-4*s/cmt2))
      ym = cmt2/2/s*(s/cmt2-sqrt(s**2/cmt2**2-4*s/cmt2))

      t6a = zlog(s/cmt2)*zlog(-(1-x3m)/x3m)+zlog(x3m-yp)*zlog(1-1/x3m)-b
     $Li2((1-x3m)/(yp-x3m))+bLi2(-x3m/(yp-x3m))+zlog(x3m-ym)*zlog(1-1/x3
     $m)-bLi2((1-x3m)/(ym-x3m))+bLi2(-x3m/(ym-x3m))

      yp = cmt2/2/mh2*(mh2/cmt2+sqrt(mh2**2/cmt2**2-4*mh2/cmt2))
      ym = cmt2/2/mh2*(mh2/cmt2-sqrt(mh2**2/cmt2**2-4*mh2/cmt2))

      t7a = zlog(mh2/cmt2)*zlog(-(1-x3p)/x3p)+zlog(x3p-yp)*zlog(1-1/x3p)
     $-bLi2((1-x3p)/(yp-x3p))+bLi2(-x3p/(yp-x3p))+zlog(x3p-ym)*zlog(1-1/
     $x3p)-bLi2((1-x3p)/(ym-x3p))+bLi2(-x3p/(ym-x3p))

      yp = cmt2/2/s*(s/cmt2+sqrt(s**2/cmt2**2-4*s/cmt2))
      ym = cmt2/2/s*(s/cmt2-sqrt(s**2/cmt2**2-4*s/cmt2))

      t8a = -zlog(s/cmt2)*zlog(-(1-x3p)/x3p)-zlog(x3p-yp)*zlog(1-1/x3p)+
     $bLi2((1-x3p)/(yp-x3p))-bLi2(-x3p/(yp-x3p))-zlog(x3p-ym)*zlog(1-1/x
     $3p)+bLi2((1-x3p)/(ym-x3p))-bLi2(-x3p/(ym-x3p))

      BTaux=t1a+t2a+t5a+t6a+t7a+t8a

      RETURN
      END

C      double complex function BoxT(s,t,mh2,mt2)
C      double precision s,t,mt2,mh2
C      double precision u,pref,gam
C      double complex BTaux
C      external BTaux
CCC Need this flag because the analytic continuation undergoes a slight
CCC change when I call BTaux with flipped arguments; see notes

C      u = -s-t+mh2
C      pref=-1d0/s/t/dsqrt(1+4*u*mt2/t/s)

C      BoxT = pref*(BTaux(s,t,mh2,mt2,1d0)+BTaux(t,s,mh2,mt2,-1d0))

C      RETURN
C      END

CCCCC Evaluation based on 0902.0325

      double complex function BoxT(s,t,mh2,mt2)
      double precision s,t,mh2,mt2
      double precision u,delt,pref
      double complex cmt2,Iaux1,Iaux2,y1,y2
      external Iaux1,Iaux2

      
      delt=0.0000000001d0
      cmt2=dcmplx(mt2,-delt)

      u=-t-s+mh2
      pref=1d0/dsqrt(t*s*(t*s+4d0*u*mt2))
      
      y1=(t*s-sqrt(t*s*(t*s+4*u*cmt2)))/2d0/t/s
      y2=(t*s+sqrt(t*s*(t*s+4*u*cmt2)))/2d0/t/s

      BoxT=pref*(Iaux2(cmt2,s,y1)-Iaux1(cmt2,y1)
     $+Iaux2(cmt2,t,y1)-Iaux2(cmt2,mh2,y1)
     $-Iaux2(cmt2,s,y2)+Iaux1(cmt2,y2)-Iaux2(cmt2,t,y2)
     $+Iaux2(cmt2,mh2,y2))



C      write(6,*)'BoxT= ',s,t,mh2,mt2,BoxT

      RETURN
      END

CCC
      double complex function Iaux1(a,y)
      double complex a,y

      Iaux1=log(a)*log((1-y)/(-y))

      RETURN
      END

CCC   
      double complex function Iaux2(m2,x,y)
      double complex m2,y
      double precision x,delt,epy
      double complex z1,z2,eta,t1,t2,t3,t4,t5,t6,t7,
     $t8,t9,t10,t11,bLi2,arg1,arg2
      external eta,bLi2

      delt=0.0000000001d0

      epy=-delt*DIMAG(y)/abs(DIMAG(y))

      z1=0.5d0*(1d0+sqrt(x**2-4d0*x*m2)/x)
      z2=0.5d0*(1d0-sqrt(x**2-4d0*x*m2)/x)

C      write(*,*) 'Imag part:',-3.1415926535d0*log((z1-y)/(z2-y))

      t1=(log(dcmplx(x,-delt))+eta(-z1,-z2))
     $*log((1-y)/(-y))

C      t1=log(REALPART(x*(y-z1)*(y-z2))-dcmplx(0d0,delt))
C     $*log((1-y)/(-y))

      arg1=y-z1
      arg2=y-z2
      if (DREAL(arg1).lt.0d0) then
          arg1=arg1-dcmplx(0d0,delt)
      endif
      if (DREAL(arg2).lt.0d0) then
          arg2=arg2-dcmplx(0d0,delt)
      endif

CCC   z1 terms

      t2=eta(-z1,1d0/arg1)*log(y/arg1)
      t3=-eta(1-z1,1d0/arg1)*log((y-1)/arg1)
      t4=bLi2(y/arg1)
      t5=-bLi2((y-1)/arg1)
      t10=log(arg1)*log((1-y)/(-y))      

CCC   z2 terms

      t6=eta(-z2,1d0/arg2)*log(y/arg2)
      t7=-eta(1-z2,1d0/arg2)*log((y-1)/arg2)
      t8=bLi2(y/arg2)
      t9=-bLi2((y-1)/arg2)
      t11=log(arg2)*log((1-y)/(-y))    
 

      Iaux2=t1+t2+t3+t4+t5+t6+t7+t8+t9+t10+t11

      RETURN
      END

CCC
      double complex function eta(a,b)
      double complex a,b
      double complex c
      double precision rpa,rpb,rpc
      double precision pi,Heavi
      external Heavi

      pi = 3.1415926535897932385d0

      c=a*b
      rpa=DIMAG(a)
      rpb=DIMAG(b)
      rpc=DIMAG(c)

      eta=dcmplx(0d0,2d0*pi)*(Heavi(-rpa)*Heavi(-rpb)*Heavi(rpc)
     $-Heavi(rpa)*Heavi(rpb)*Heavi(-rpc))

      RETURN
      END

CCC
      double precision function Heavi(x)
      double precision x

      if (x.ge.0d0) then
          Heavi = 1d0
      else
          Heavi = 0d0
      endif

      RETURN
      END


