CC-----------------------------------------------------------
CC 
CC Routines for evaluating the generalized log-sine-function
CC 
CC-----------------------------------------------------------

C =============================
C log-sine-log-sine-cosine function LsLsc_{1,1,2}
C map argument to the region 0 < theta < pi/2
      function lslsc112(x,Nmax,lmax)
      implicit none
      double precision lslsc112,lslsc2,x,xx
      double precision lsjk,pi
      double precision tmp1,tmp2,lslsc112pi
      integer Nmax,lmax,dummy
      pi   = 3.14159265358979324d0
      lslsc112=0.d0
      lslsc112pi=0.6490877578859336D0                
      xx=x
      if ((x.lt.0.d0).and.(x.ge.-2.d0*pi)) then
C         write(6,*) 'x < 0'
         xx=-x
      endif
      if ((xx.gt.3.d0*pi/2.d0).and.(xx.le.2.d0*pi)) then
         lslsc112=lslsc2(2.d0*pi-xx,lmax)
C         write(6,*) 'range IV:  lslsc112= ',lslsc112
      elseif ((xx.gt.pi).and.(xx.le.3.d0*pi/2.d0)) then
         tmp1=lsjk(2,0,2d0*pi-xx,Nmax,dummy)
         tmp1=tmp1*lsjk(2,0,pi-xx,Nmax,dummy)
         tmp2=lslsc2(xx-pi,lmax)
         lslsc112=-tmp1-tmp2+lslsc112pi
C         write(6,*) 'range III:  lslsc112= ',lslsc112      
      elseif ((xx.gt.pi/2.d0).and.(xx.le.pi)) then
         tmp1=lslsc2(pi-xx,lmax)
         tmp2=lsjk(2,0,xx,Nmax,dummy)*lsjk(2,0,pi-xx,Nmax,dummy)
         lslsc112=tmp2-tmp1+lslsc112pi
C         write(6,*) 'range II:  lslsc112= ',lslsc112
      elseif ((xx.ge.0.d0).and.(xx.le.pi/2.d0)) then
         lslsc112=lslsc2(xx,lmax)
C         write(6,*) 'range I:  lslsc112= ',lslsc112
      else
         print 999
         write(6,*) x
         goto 11
      endif
C      write(6,*)'lslsc112= ',lslsc112
      return
 11   stop
 999  format('FAILED: x is outside range -2*pi < x < 2*pi!!!!!! x:')
      end


C =============================
C log-sine-cosine function
C map argument to the region 0 < theta < pi/2
C so far implemented for: (i,j) = (2,2),(2,3),(3,2),(3,3)
      function lsc(i,j,x,Nmax,lmax)
      implicit none
      double precision lsc,lsc2,lscpi
      double precision x,sig,xx,pi
      integer i,j,Nmax,lmax
      dimension lscpi(2:4,2:4)
      pi   = 3.14159265358979324d0
C      write(6,*)'***************'
C      write(6,*)'This is lsc: i= ',i,' j= ',j,' x= ',x
      lscpi(2,2) = 0.1291928195012492D1
      lscpi(2,3) = -0.9440932840407697D0
      lscpi(2,4) = 0.4462787069785354D1
      lscpi(3,2) = -0.9440932840407697D0
      lscpi(3,3) = -0.1912623029908009D1
      lscpi(3,4) = 0.2215710152207518D1
      lscpi(4,2) = 0.4462787069785354D1
      lscpi(4,3) = 0.2215710152207518D1
      lscpi(4,4) = 0.3974846983186666D1
C      write(6,*)'pi = ',pi,' 4*datan(1.d0)= ',4.d0*datan(1.d0)
      sig=1.d0
      lsc=0.d0
      xx=x
C      write(6,*)'in lsc: x= ',x,' xx= ',xx
C      write(6,*)'pi = ',pi,' pi/2.d0 = ',pi/2.d0
      if ((x.lt.0.d0).and.(x.ge.-2.d0*pi)) then
C         write(6,*) 'x < 0'
         sig=-1.d0
         xx=-x
      endif
      if ((xx.gt.3.d0*pi/2.d0).and.(xx.le.2.d0*pi)) then
         lsc=lsc2(i,j,2.d0*pi-xx,Nmax,lmax)
         lsc=sig*(2.d0*lscpi(i,j) - lsc)
C         write(6,*) 'range IV:  lsc= ',lsc
      elseif ((xx.gt.pi).and.(xx.le.3.d0*pi/2.d0)) then
         lsc=lsc2(j,i,xx-pi,Nmax,lmax)
         lsc=sig*(lscpi(i,j)+lsc)
C;         lsc=lsc2(i,j,pi-xx,Nmax,lmax)
C;         lsc=sig*(lscpi(i,j) + lsc)
C         write(6,*) 'range III:  lsc= ',lsc      
      elseif ((xx.gt.pi/2.d0).and.(xx.le.pi)) then
         lsc=lsc2(j,i,pi-xx,Nmax,lmax)
         lsc=sig*(lscpi(i,j) - lsc)
C         write(6,*) 'range II:  lsc= ',lsc
      elseif ((xx.gt.0.d0).and.(xx.le.pi/2.d0)) then
         lsc=lsc2(i,j,xx,Nmax,lmax)
         lsc=sig*lsc
C         write(6,*) 'range I:  lsc= ',lsc
C         write(6,*) lsc2(i,j,xx,Nmax,lmax)
      elseif (xx.eq.0.d0) then
         lsc=0.d0
C         write(6,*) 'range 0:  lsc= ',lsc
      else
         print 999
         write(6,*) x
         goto 11
      endif
C      write(6,*) 'lsc = ',lsc
      return
 11   stop
 999  format('FAILED: x is outside range -2*pi < x < 2*pi!!!!!! x:')
      end

C =============================
C logsine function
C map argument to the region 0 < theta < pi
C implemented are: (j,k) = (2,0),(3,0),(3,1),(4,0),(4,1),(4,2)
      function lsjk(j,k,x,Nmax,lmax)
      implicit none
      double precision lsjk2,lsjk,lsatpi,pi
      double precision x,ls2pi,tmp,mybinomial
      integer j,k,Nmax,lmax,l
      dimension ls2pi(2:4,0:2)
      pi   = 3.14159265358979324d0
C      write(6,*)'in lsjk: pi = ',pi
      lsjk=0.d0
      ls2pi(2,0) = 0.D0
      ls2pi(2,1) = 0.D0
      ls2pi(2,2) = 0.D0
      ls2pi(3,0) = -0.516771278004996D1
      ls2pi(3,1) = 0.D0
      ls2pi(3,2) = 0.D0
      ls2pi(4,0) = 0.113291194084892D2
      ls2pi(4,1) = -0.16234848505667D2
      ls2pi(4,2) = 0.151054925446522D2
C      if ((x.ge.0).and.(x.le.pi)) then
      if ((x.ge.0.d0).and.(x.le.2.5d0)) then
         lsjk=0.d0
         lsjk=lsjk2(j,k,x,Nmax,lmax)
      elseif ((x.gt.2.5d0).and.(x.le.pi)) then
         lsjk=lsatpi(j,k,x)
      elseif ((x.lt.0.d0).and.(x.gt.-2.5d0)) then
         lsjk=0.d0
         tmp=lsjk2(j,k,-x,Nmax,lmax)
         lsjk=(-1.d0)**(k+1)*tmp
      elseif ((x.le.-2.5d0).and.(x.gt.-3.7d0)) then
         lsjk=0.d0
         tmp=lsatpi(j,k,-x)
         lsjk=(-1.d0)**(k+1)*tmp
      elseif ((x.le.-3.7d0).and.(x.gt.-2.d0*pi)) then
         lsjk=0.d0
         tmp=lsjk2(j,k,-x,Nmax,lmax)
         lsjk=(-1.d0)**(k+1)*tmp
      elseif ((x.gt.pi).and.(x.lt.2.d0*pi)) then
         lsjk=0.d0
         do l=0,k
            if (x.gt.2d0*pi-2.5d0) then
               tmp=lsjk2(j-l,k-l,2.d0*pi-x,Nmax,lmax)
            else
               tmp=lsatpi(j-l,k-l,2.d0*pi-x)
            endif
            tmp=tmp-ls2pi(j-l,k-l)
            tmp=(-2.d0*pi)**l*mybinomial(k,l)*tmp
            lsjk=lsjk+tmp
         enddo
         lsjk=(-1.d0)**(k+1)*lsjk   
      else 
         print 999
         write(6,*) x
         goto 11
      endif
C      write(6,*)'lsjk= ',lsjk
      return
 11   stop
 999  format('FAILED: x is outside range -2*pi < x < 2*pi!!!!!! x:')
      end
C =============================
C log-sine-log-sine-cosine function
      function lslsc2(x,lastmax)
      implicit none
      double precision lslsc2,coeff,lseven
      double precision x,pi
      double precision term1,term2
      double precision res,tmp,termp
      double precision tsinx,lnpow,lnsn,tcosx,tlncs,ddd
      double precision lncs,xpow,xpown
      integer Nmax,lastmax,dummy,n,p
      dimension coeff(0:60,0:1)
      dimension lnpow(0:2)
      pi   = 3.14159265358979324d0
      Nmax=60
      lastmax=0
      lslsc2=0.d0
      tsinx=2.d0*dsin(x/2.d0)
      tcosx=2.d0*dcos(x/2.d0)
      lnsn=dlog(tsinx)
      tlncs=dlog(tcosx**2)
      term2=0.d0
      lnpow(0)=1.d0
      xpow=tsinx/dsqrt(dsqrt(10.d0))
      call lslslsccoeff(coeff)
      if ((x.gt.0).and.(x.le.pi/2.d0)) then
         term1=x*lseven(2,0,x,Nmax,dummy)/2.d0
         term1=term1+x**2*lnsn/4.d0
         term1=term1*tlncs
         do p=0,1
            tmp=1.d0
            termp=0.d0
            ddd=dabs(termp-tmp)
            if (p.eq.0) then
               n=1
            elseif (p.eq.1) then
               n=2
            else
               n=0
            endif
            xpown=xpow**(2*n)
            do while((ddd.gt.1.d-20).and.(n.lt.Nmax))
               tmp=termp
               termp=termp+coeff(n,p)*xpown
               ddd=dabs(lnpow(p)*(termp-tmp))
               xpown=xpow**2*xpown
               n=n+1
            enddo
            if (n.gt.lastmax) lastmax=n
            term2=term2+lnpow(p)*termp 
            lnpow(p+1)=lnsn**(p+1)
         enddo
         lslsc2=term1+term2
      endif
      return
      end

C =============================
C log-sine-cosine function
      function lsc2(i,j,x,Nmax,lastmax)
      implicit none
      double precision lsc2
      double precision x
      double precision res,tmp,pf,termp,term1
      double precision tsinx,lnpow,lnsn,lnsnp,ddd
      double precision lncs,f1,f2
      double precision anj,bnj
      double precision ifact,myfactorial,mybinomial
      integer n,i,j,k,Nmax,lastmax,p
      dimension lnpow(0:i)
C      write(6,*) '================= lsc2 =================== '
C      write(6,*) 'This is lsc2: i=',i,' j= ',j
      lastmax = 0
      tsinx=2.d0*dsin(x/2.d0)
      ifact=myfactorial(i-1)
      pf=(-1.d0)**(i-2)*2.d0**(1-j)*ifact
      lncs=dlog(4.d0*dcos(x/2.d0)**2)
      term1=-(2.d0**(1-j))*x*dlog(tsinx)**(i-1)*lncs**(j-1)
C      write(6,*)'lncs= ',lncs,'dlog= ',dlog(tsinx)
C      write(6,*)'term1 = ',term1
C      write(6,*)'2^(1-j)= ',2.d0**(1-j)
      lnsnp=1.d0
      lnpow(0)=1.d0
      res=0.d0
      lnsn=-dlog(tsinx)
      do p=0,i-1
C         write(6,*) '+++++++++++++   p= ',p,' +++++++++++++'
         n=0
         if (p.eq.i-1) then
            n=1
         endif
         tmp=1.d0
         termp=0.d0
         ddd=dabs(termp-tmp)
         do while((ddd.gt.1.d-20).and.(n.lt.Nmax))
C            write(6,*) 'n= ',n
            tmp=termp
            f1=(tsinx/2.d0)**(2*n+1)
C            write(6,*) 'f1= ',f1
            if (p.lt.i-1) then
               f2=anj(n,j)
            else
               f2=0.d0
            endif
C            write(6,*) f2
            f2=f2 + (1.d0*j-1.d0)/(2.d0*n+1.d0)*bnj(n-1,j)
            f2=1.d0/(2.d0*n+1.d0)**(i-1-p)*f2
C            write(6,*) 1/(2.d0*n+1.d0)**p
C            write(6,*) 2.d0*(1.d0*j-1.d0)/(2.d0*n+1.d0)**(p+1) 
C            write(6,*) 'anj= ',anj(n,j), '  bnj= ', bnj(n-1,j)
C            write(6,*) 'f2= ',f2
            termp=termp+f1*f2
            ddd=dabs(lnpow(p)*(termp-tmp))
            n=n+1
         enddo
         if (n.gt.lastmax) lastmax=n
         res=res+lnpow(p)*termp 
C         write(6,*) 'lnpow(p)= ',lnpow(p)
         lnsnp=lnsnp*lnsn
         lnpow(p+1)=lnsnp/myfactorial(p+1)
      enddo
C      write(6,*) 'term1= ',term1,' pf= ',pf,' res=',res
      lsc2=term1+pf*res
C      write(6,*) 'lsc2= ',lsc2
      return
      end



C =============================
C logsine function
      function lsjk2(j,k,x,Nmax,lmax)
      implicit none
      double precision lsjk2,lsodd,lseven
      double precision x,pi
      integer j,k,Nmax,lmax
      pi   = 3.14159265358979324d0
      lsjk2=0.d0
      if ((x.gt.0).and.(x.lt.pi)) goto 101
      if (x.eq.0.d0) then
         lsjk2=0.d0
         return
      else
         print 1001
         write(6,*) x
      endif
      stop
 101  if (k.eq.2*(k/2)) then
         lsjk2=lseven(j,k/2,x,nmax,lmax)
      else
         lsjk2=lsodd(j,k/2,x,nmax,lmax)
      endif
      return
 1001 format('FAILED: x is outside range 0 <= x < pi!!!!!! x:')
      end


C =============================
C logsine function for 2*k+1
      function lsodd(j,k,x,Nmax,lastmax)
      implicit none
      double precision x
      double precision lsodd,res,tmp,pf,f1,termp,f2
      double precision tsinx,lnpow,lnsn,lnsnp,ddd,lsbcoeff
      double precision kfact,jkfact,myfactorial,mybinomial
      double precision lsbb,beta
      integer l1,l2,j,k,Nmax,lastmax   
      dimension lsbb(1:1001)
      dimension lnpow(0:j-2*k-1)
      lastmax=0
      call beta2(lsbb)
      tsinx=2.d0*dsin(x/2.d0)
      kfact=myfactorial(2*k+1)
      jkfact=myfactorial(j-2*k-2)
      pf=(-1)**(j-1)*2.d0**(2*k+2)*kfact*jkfact
      lnsnp=1.d0
      lnpow(0)=1
      res=0.d0
      lnsn=-dlog(tsinx)
      do l1=0,j-2*k-2
         l2=k+1
         tmp=1.d0
         termp=0.d0
         ddd=dabs(termp-tmp)
         do while((ddd.gt.1.d-20).and.(l2.lt.Nmax))
            tmp=termp
            f1=1.d0/(2.d0*l2)**(j-2*k-l1)
            if (l2.lt.451) goto 203
            f1=f1/beta(l2)
            goto 204
 203        f1=f1/lsbb(l2+1)
 204        f2=(tsinx/2.d0)**(2*l2)*lsbcoeff(l2,k+1)
            termp=termp+f1*f2
            ddd=dabs(lnpow(l1)*(termp-tmp))
            l2=l2+1
         enddo
         if (l2.gt.lastmax) lastmax=l2
         res=res+lnpow(l1)*termp 
         lnsnp=lnsnp*lnsn
         lnpow(l1+1)=lnsnp/myfactorial(l1+1)
      enddo
      lsodd=pf*res
      return
      end      

C =============================
C logsine function for 2*k
      function lseven(j,k,x,Nmax,lastmax)
      implicit none
      double precision x
      double precision lseven,res,tmp,pf,f1,termp,f2
      double precision tsinx,lnpow,lnsn,lnsnp,ddd,lsccoeff
      double precision kfact,jkfact,myfactorial,mybinomial
      double precision lsbb,beta,dum
      integer l1,l2,j,k,Nmax,lastmax
      dimension lsbb(1:1001)
      dimension lnpow(0:j-2*k)
      lastmax = 0
      call beta2(lsbb)
      tsinx=2.d0*dsin(x/2.d0)
      kfact=myfactorial(2*k)
      jkfact=myfactorial(j-2*k-1)
      pf=(-1)**j*tsinx*2.d0**(2*k)*kfact*jkfact
      lnsnp=1.d0
      lnpow(0)=1
      res=0.d0
      lnsn=-dlog(tsinx)
      do l1=0,j-2*k-1
         l2=k
         tmp=1.d0
         termp=0.d0
         ddd=dabs(termp-tmp)
         do while((ddd.gt.1.d-20).and.(l2.lt.Nmax))
            tmp=termp
            f1=1.d0/(2.d0*l2+1.d0)**(j-2*k-l1)
            if (l2.lt.451) goto 303
            f1=f1*beta(l2)
            goto 304
 303        f1=f1*lsbb(l2+1)
 304        f2=(tsinx/2.d0)**(2*l2)*lsccoeff(l2,k+1)
            termp=termp+f1*f2
            ddd=dabs(lnpow(l1)*(termp-tmp))
            l2=l2+1
         enddo
         if (l2.gt.lastmax) lastmax=l2
         res=res+lnpow(l1)*termp 
         lnsnp=lnsnp*lnsn
         lnpow(l1+1)=lnsnp/myfactorial(l1+1)
      enddo
      lseven=pf*res
      return
      end

C =============================
      function anj(n,j)
      implicit none
      double precision anj,tmp
      double precision lsacoeff,lsccoeff,dfact
      integer n,j,l1
C      write(6,*)'This is anj: '
      dfact=1.d0
      anj=2.d0*lsccoeff(0,1)*lsacoeff(n,j-1)
      tmp=0.d0
      if (n.gt.0) then
         do l1=1,n
C     write(6,*) 'l1= ',l1
            dfact=dfact*(2.d0*l1-1.d0)/(l1*2.d0)
            tmp=dfact/(2.d0*l1+1.d0)
            anj=anj+2.d0*tmp*lsccoeff(l1,1)*lsacoeff(n-l1,j-1)
         enddo   
      endif
C         write(6,*) 'dfact= ',dfact,' tmp= ',tmp,' anj= ',anj
      return
      end
C =============================
C Clausen's function of order 1 (Cl_1(x))
      function clausen1(x)
      implicit none
      double precision x,clausen1,tsinx
      tsinx=dabs(2.d0*dsin(x/2.d0))
      clausen1=-dlog(tsinx)
      return
      end
C =============================
      function bnj(n,j)
      implicit none
      double precision bnj,tmp
      double precision lsacoeff,lsccoeff,dfact
      integer n,j,l1,l2
      dfact=1.d0
      bnj=0.d0
      tmp=0.d0
C      write(6,*) '+++++++++++++++++++'
      if (n.eq.0) then
         bnj=4.d0*lsccoeff(0,1)*lsacoeff(0,j-2)
      elseif (n.gt.0) then
         do l1=0,n
            if (l1.gt.0) then
               dfact=dfact*(2.d0*l1-1.d0)/(l1*2.d0)
            endif
            tmp=0.d0
            do l2=0,n-l1
C               write(6,*)'l2= ',l2,' lsacoeff= ',lsacoeff(l2,j-2)
               tmp=tmp+lsacoeff(l2,j-2)
            enddo
C            write(6,*) 'sum of lsacoeff= ',tmp
            tmp=tmp*dfact/(2.d0*l1+1.d0)
C            write(6,*)'l1= ',l1, 'tmp= ',tmp,' dfact= ',dfact
            bnj=bnj+4.d0*tmp*lsccoeff(l1,1)
         enddo
      endif   
      return
      end

C =============================
      function lsacoeff(m,n)
      implicit none
      double precision lsacoeff,tmp
      integer n,m,l
      lsacoeff=0.d0
      if (n.lt.3) goto 405
         print 1101,m
         stop
 405  if (n.eq.0) then
         if (m.eq.0) then
            lsacoeff=1.d0
         endif
      elseif (n.eq.1) then
         if (m.eq.0) then
            lsacoeff=dlog(4.d0)
         else
            lsacoeff=-1.d0/m
         endif
      elseif (n.eq.2) then
         if (m.eq.0) then
            lsacoeff=dlog(4.d0)**2
         else
            lsacoeff=-2.d0*dlog(4.d0)/m
            tmp=0.d0
            do l=1,m-1
               tmp=tmp+1.d0/l
            enddo
            lsacoeff=lsacoeff+2.d0/m*tmp
         endif
      endif    
      return
 1101 format('lsacoeff(',I5,',n) not implemented')      
      
      end

C =============================
      function lsbcoeff(m,n)
      implicit none
      integer m,n
      double precision lsbcoeff,lsbb1,lsbb2,lsbb3,lsbb4
      if (n.lt.5) goto 105
         print 1005,n
         stop
 105  if (m.lt.n-1) then
         lsbcoeff=0.d0
      elseif (n.eq.1) then
         lsbcoeff=lsbb1(m)
      elseif (n.eq.2) then
         lsbcoeff=lsbb2(m)
      elseif (n.eq.3) then
         lsbcoeff=lsbb3(m)
      else 
         lsbcoeff=lsbb4(m)
      endif
      return
 1005 format('no coefficients implemented for ',I5)      
      end

C =============================
      function lsccoeff(m,n)
      implicit none
      integer m,n
      double precision lsccoeff,lscc1,lscc2,lscc3,lscc4
      if (n.lt.5) goto 106
      print 1006,n
      stop
 106  if (m.lt.n-1) then
         lsccoeff=0.d0
      return
      elseif (n.eq.1) then
         lsccoeff=lscc1(m)
      elseif (n.eq.2) then
         lsccoeff=lscc2(m)
      elseif (n.eq.3) then
         lsccoeff=lscc3(m)
      else 
         lsccoeff=lscc4(m)
      endif
      return
 1006 format('no coefficients implemented for ',I5)      
      end




C =============================
      function lsbb1(m)
      implicit none
      integer m
      double precision lsbb1
      lsbb1=1.d0
      return
      end

C =============================
      function lsbb2(m)
      implicit none
      integer m
      double precision lsbb2,ssum
      lsbb2=ssum(m,1)
      return
      end

C =============================
      function lsbb3(m)
      implicit none
      integer m
      double precision lsbb3,ssum
      lsbb3=ssum(m,1)
      lsbb3=lsbb3**2
      lsbb3=lsbb3-ssum(m,2)
      lsbb3=0.5d0*lsbb3
      return
      end

C =============================
      function lsbb4(m)
      implicit none
      integer m
      double precision lsbb4,ssum
      lsbb4=ssum(m,1)
      lsbb4=lsbb4**3- 3*ssum(m,1)*ssum(m,2) + 2*ssum(m,3)
      lsbb4=1.d0/6.d0*lsbb4 
      return
      end

C =============================
      function lscc1(m)
      implicit none
      integer m
      double precision lscc1
      lscc1=1.d0
      return
      end

C =============================
      function lscc2(m)
      implicit none
      integer m
      double precision lscc2,hsum
      lscc2=hsum(m,1)
      return
      end

C =============================
      function lscc3(m)
      implicit none
      integer m
      double precision lscc3,hsum
      lscc3=hsum(m,1)
      lscc3=lscc3**2-hsum(m,2)
      lscc3=1.d0/2.d0*lscc3
      return
      end

C =============================
      function lscc4(m)
      implicit none
      integer m
      double precision lscc4,hsum
      lscc4=hsum(m,1)
      lscc4=lscc4**3- 3*hsum(m,1)*hsum(m,2) + 2*hsum(m,3)
      lscc4=1.d0/6.d0*lscc4 
      return
      end


C =============================
      function ssum(m,a)
      implicit none
      integer m,a,l
      double precision ssum, tmp, ta
      ssum=0.d0
      ta=2*a
      tmp=1.d0
      if (m.lt.1) then
         return
      else
         do l=1,m-1
            tmp=(2*l)**ta
            ssum=ssum+1.d0/tmp
         enddo
      endif
      return
      end

C =============================
      function hsum(m,a)
      implicit none
      integer m,a,l
      double precision hsum, tmp, ta
      hsum=0.d0
      ta=2*a
      tmp=1.d0
      if (m.lt.1) then
         return
      else
         do l=1,m
            tmp=(2*l-1)**ta
            hsum=hsum+1.d0/tmp
         enddo
      endif
      return
      end

C =============================
      function beta(n)
      implicit none
      integer n,j
      double precision beta,tmp
      beta=1.d0
      do j=1,n
         beta=beta*(2.d0*j-1.d0)/(2.d0*j)
      enddo
      return
      end



C =============================
      function mybinomial(n,m)
      implicit none
      integer m,n
      double precision mybinomial,myfactorial
      if (m.eq.0) then
         mybinomial=1.d0
      else
         mybinomial=myfactorial(n)/myfactorial(m)/myfactorial(n-m)
      endif
      return
      end

C =============================
      function myfactorial(n)
      implicit none
      integer m,n
      double precision myfactorial
      integer l
      myfactorial=1.d0
C      write(6,*) 'in myfactorial, argument n= ',n
      do l=1,n
C         write(6,*) l
         myfactorial=myfactorial*l
      enddo
C      write(6,*) myfactorial
      return
      end

C =============================
      subroutine beta2(tmp)
      implicit none
      doubleprecision tmp(1:1001)
      tmp(1) = 1.D0
      tmp(2) = 0.5D0
      tmp(3) = 0.375D0
      tmp(4) = 0.3125D0
      tmp(5) = 0.2734375D0
      tmp(6) = 0.24609375D0
      tmp(7) = 0.2255859375D0
      tmp(8) = 0.20947265625D0
      tmp(9) = 0.196380615234375D0
      tmp(10) = 0.1854705810546875D0
      tmp(11) = 0.1761970520019531D0
      tmp(12) = 0.1681880950927734D0
      tmp(13) = 0.1611802577972412D0
      tmp(14) = 0.1549810171127319D0
      tmp(15) = 0.1494459807872772D0
      tmp(16) = 0.144464448094368D0
      tmp(17) = 0.139949934091419D0
      tmp(18) = 0.1358337595593184D0
      tmp(19) = 0.1320605995715596D0
      tmp(20) = 0.1285853206354659D0
      tmp(21) = 0.1253706876195793D0
      tmp(22) = 0.1223856712476845D0
      tmp(23) = 0.119604178719328D0
      tmp(24) = 0.1170040878776035D0
      tmp(25) = 0.1145665027134868D0
      tmp(26) = 0.112275172659217D0
      tmp(27) = 0.1101160347234629D0
      tmp(28) = 0.1080768488952506D0
      tmp(29) = 0.1061469051649783D0
      tmp(30) = 0.1043167861104097D0
      tmp(31) = 0.1025781730085695D0
      tmp(32) = 0.100923686347141D0
      tmp(33) = 0.993467537479669D-1
      tmp(34) = 0.9784149990330073D-1
      tmp(35) = 0.9640265431648749D-1
      tmp(36) = 0.9502547354053766D-1
      tmp(37) = 0.9370567529691909D-1
      tmp(38) = 0.9243938238750126D-1
      tmp(39) = 0.9122307472450782D-1
      tmp(40) = 0.9005354812547567D-1
      tmp(41) = 0.8892787877390722D-1
      tmp(42) = 0.8784339244739616D-1
      tmp(43) = 0.8679763777540335D-1
      tmp(44) = 0.8578836291754982D-1
      tmp(45) = 0.8481349515712312D-1
      tmp(46) = 0.8387112298871064D-1
      tmp(47) = 0.82959480347529D-1
      tmp(48) = 0.8207693268425742D-1
      tmp(49) = 0.8122196463546307D-1
      tmp(50) = 0.8039316907795834D-1
      tmp(51) = 0.7958923738717876D-1
      tmp(52) = 0.7880895074612799D-1
      tmp(53) = 0.7805117237356907D-1
      tmp(54) = 0.7731484055872408D-1
      tmp(55) = 0.7659896240540256D-1
      tmp(56) = 0.7590260820171708D-1
      tmp(57) = 0.7522490634277318D-1
      tmp(58) = 0.7456503874327517D-1
      tmp(59) = 0.7392223668514348D-1
      tmp(60) = 0.7329577705221854D-1
      tmp(61) = 0.7268497891011672D-1
      tmp(62) = 0.7208920039446002D-1
      tmp(63) = 0.7150783587514986D-1
      tmp(64) = 0.7094031336820423D-1
      tmp(65) = 0.7038609217001513D-1
      tmp(66) = 0.6984466069178425D-1
      tmp(67) = 0.6931553447442224D-1
      tmp(68) = 0.6879825436640417D-1
      tmp(69) = 0.6829238484900414D-1
      tmp(70) = 0.6779751249502585D-1
      tmp(71) = 0.673132445486328D-1
      tmp(72) = 0.6683920761519173D-1
      tmp(73) = 0.6637504645119734D-1
      tmp(74) = 0.6592042284536722D-1
      tmp(75) = 0.6547501458289853D-1
      tmp(76) = 0.650385144856792D-1
      tmp(77) = 0.6461062952195763D-1
      tmp(78) = 0.6419107997960725D-1
      tmp(79) = 0.6377959869768669D-1
      tmp(80) = 0.633759303514988D-1
      tmp(81) = 0.6297983078680194D-1
      tmp(82) = 0.6259106639922908D-1
      tmp(83) = 0.6220941355533135D-1
      tmp(84) = 0.6183465805198598D-1
      tmp(85) = 0.6146659461120035D-1
      tmp(86) = 0.6110502640760505D-1
      tmp(87) = 0.6074976462616549D-1
      tmp(88) = 0.6040062804785419D-1
      tmp(89) = 0.6005744266121865D-1
      tmp(90) = 0.5972004129795338D-1
      tmp(91) = 0.5938826329074253D-1
      tmp(92) = 0.5906195415178241D-1
      tmp(93) = 0.5874096527052272D-1
      tmp(94) = 0.5842515362928335D-1
      tmp(95) = 0.5811438153551057D-1
      tmp(96) = 0.5780851636953419D-1
      tmp(97) = 0.575074303467762D-1
      tmp(98) = 0.5721100029344231D-1
      tmp(99) = 0.5691910743480229D-1
      tmp(100) = 0.5663163719523259D-1
      tmp(101) = 0.5634847900925642D-1
      tmp(102) = 0.5606952614287396D-1
      tmp(103) = 0.5579467552452654D-1
      tmp(104) = 0.5552382758508709D-1
      tmp(105) = 0.5525688610631264D-1
      tmp(106) = 0.5499375807723496D-1
      tmp(107) = 0.5473435355800272D-1
      tmp(108) = 0.5447858555072233D-1
      tmp(109) = 0.5422636987687639D-1
      tmp(110) = 0.5397762506092742D-1
      tmp(111) = 0.5373227221974139D-1
      tmp(112) = 0.534902349574903D-1
      tmp(113) = 0.5325143926571579D-1
      tmp(114) = 0.5301581342825687D-1
      tmp(115) = 0.5278328793076451D-1
      tmp(116) = 0.525537953745438D-1
      tmp(117) = 0.5232727039448111D-1
      tmp(118) = 0.5210364958082948D-1
      tmp(119) = 0.5188287140463953D-1
      tmp(120) = 0.5166487614663684D-1
      tmp(121) = 0.5144960582935919D-1
      tmp(122) = 0.5123700415237836D-1
      tmp(123) = 0.5102701643044239D-1
      tmp(124) = 0.5081958953438368D-1
      tmp(125) = 0.5061467183464826D-1
      tmp(126) = 0.5041221314730967D-1
      tmp(127) = 0.5021216468243939D-1
      tmp(128) = 0.5001447899471325D-1
      tmp(129) = 0.4981910993614015D-1
      tmp(130) = 0.4962601261080627D-1
      tmp(131) = 0.4943514333153394D-1
      tmp(132) = 0.4924645957836015D-1
      tmp(133) = 0.4905991995874515D-1
      tmp(134) = 0.4887548416942656D-1
      tmp(135) = 0.4869311295983915D-1
      tmp(136) = 0.4851276809702493D-1
      tmp(137) = 0.4833441233196233D-1
      tmp(138) = 0.4815800936724714D-1
      tmp(139) = 0.4798352382606147D-1
      tmp(140) = 0.478109212223706D-1
      tmp(141) = 0.476401679322907D-1
      tmp(142) = 0.4747123116657336D-1
      tmp(143) = 0.4730407894415585D-1
      tmp(144) = 0.4713868006672873D-1
      tmp(145) = 0.4697500409427481D-1
      tmp(146) = 0.4681302132153593D-1
      tmp(147) = 0.4665270275536629D-1
      tmp(148) = 0.4649402009293307D-1
      tmp(149) = 0.4633694570072721D-1
      tmp(150) = 0.4618145259434893D-1
      tmp(151) = 0.4602751441903444D-1
      tmp(152) = 0.4587510543089194D-1
      tmp(153) = 0.4572420047881664D-1
      tmp(154) = 0.455747749870558D-1
      tmp(155) = 0.4542680493839653D-1
      tmp(156) = 0.4528026685795009D-1
      tmp(157) = 0.4513513779750794D-1
      tmp(158) = 0.4499139532044581D-1
      tmp(159) = 0.4484901748715326D-1
      tmp(160) = 0.4470798284096724D-1
      tmp(161) = 0.4456827039458922D-1
      tmp(162) = 0.4442985961696627D-1
      tmp(163) = 0.4429273042061761D-1
      tmp(164) = 0.4415686314938872D-1
      tmp(165) = 0.440222385666162D-1
      tmp(166) = 0.4388883784368706D-1
      tmp(167) = 0.4375664254897716D-1
      tmp(168) = 0.4362563463715387D-1
      tmp(169) = 0.4349579643882901D-1
      tmp(170) = 0.4336711065054845D-1
      tmp(171) = 0.4323956032510566D-1
      tmp(172) = 0.4311312886216676D-1
      tmp(173) = 0.4298779999919534D-1
      tmp(174) = 0.4286355780266587D-1
      tmp(175) = 0.4274038665955477D-1
      tmp(176) = 0.4261827126909889D-1
      tmp(177) = 0.4249719663481168D-1
      tmp(178) = 0.4237714805674724D-1
      tmp(179) = 0.4225811112400357D-1
      tmp(180) = 0.4214007170745607D-1
      tmp(181) = 0.4202301595271314D-1
      tmp(182) = 0.4190693027328576D-1
      tmp(183) = 0.4179180134396354D-1
      tmp(184) = 0.4167761609438987D-1
      tmp(185) = 0.4156436170282903D-1
      tmp(186) = 0.4145202559011868D-1
      tmp(187) = 0.4134059541380116D-1
      tmp(188) = 0.4123005906242736D-1
      tmp(189) = 0.4112040465002728D-1
      tmp(190) = 0.410116205107415D-1
      tmp(191) = 0.4090369519360797D-1
      tmp(192) = 0.4079661745749905D-1
      tmp(193) = 0.4069037626620348D-1
      tmp(194) = 0.4058496078364854D-1
      tmp(195) = 0.404803603692577D-1
      tmp(196) = 0.4037656457343909D-1
      tmp(197) = 0.4027356313320072D-1
      tmp(198) = 0.4017134596788803D-1
      tmp(199) = 0.4006990317503983D-1
      tmp(200) = 0.3996922502635882D-1
      tmp(201) = 0.3986930196379293D-1
      tmp(202) = 0.3977012459572379D-1
      tmp(203) = 0.3967168369325913D-1
      tmp(204) = 0.3957397018662549D-1
      tmp(205) = 0.3947697516165827D-1
      tmp(206) = 0.3938068985638593D-1
      tmp(207) = 0.3928510565770538D-1
      tmp(208) = 0.3919021409814571D-1
      tmp(209) = 0.3909600685271747D-1
      tmp(210) = 0.3900247573584494D-1
      tmp(211) = 0.3890961269837864D-1
      tmp(212) = 0.388174098246858D-1
      tmp(213) = 0.3872585932981626D-1
      tmp(214) = 0.3863495355674157D-1
      tmp(215) = 0.3854468497366508D-1
      tmp(216) = 0.3845504617140074D-1
      tmp(217) = 0.3836602986081879D-1
      tmp(218) = 0.3827762887035608D-1
      tmp(219) = 0.381898361435892D-1
      tmp(220) = 0.3810264473686868D-1
      tmp(221) = 0.3801604781701216D-1
      tmp(222) = 0.3793003865905512D-1
      tmp(223) = 0.3784461064405725D-1
      tmp(224) = 0.3775975725696295D-1
      tmp(225) = 0.3767547208451437D-1
      tmp(226) = 0.3759174881321545D-1
      tmp(227) = 0.375085812273455D-1
      tmp(228) = 0.3742596320702096D-1
      tmp(229) = 0.373438887263038D-1
      tmp(230) = 0.3726235185135554D-1
      tmp(231) = 0.371813467386352D-1
      tmp(232) = 0.3710086763314032D-1
      tmp(233) = 0.3702090886668959D-1
      tmp(234) = 0.3694146485624605D-1
      tmp(235) = 0.3686253010227971D-1
      tmp(236) = 0.3678409918716848D-1
      tmp(237) = 0.3670616677363634D-1
      tmp(238) = 0.3662872760322783D-1
      tmp(239) = 0.3655177649481768D-1
      tmp(240) = 0.3647530834315489D-1
      tmp(241) = 0.3639931811743998D-1
      tmp(242) = 0.3632380085993492D-1
      tmp(243) = 0.3624875168460447D-1
      tmp(244) = 0.3617416577578841D-1
      tmp(245) = 0.361000383869036D-1
      tmp(246) = 0.3602636483917523D-1
      tmp(247) = 0.3595314052039642D-1
      tmp(248) = 0.3588036088371545D-1
      tmp(249) = 0.358080214464499D-1
      tmp(250) = 0.357361177889269D-1
      tmp(251) = 0.3566464555334905D-1
      tmp(252) = 0.3559360044268501D-1
      tmp(253) = 0.3552297821958444D-1
      tmp(254) = 0.3545277470531649D-1
      tmp(255) = 0.3538298577873122D-1
      tmp(256) = 0.3531360737524351D-1
      tmp(257) = 0.3524463548583874D-1
      tmp(258) = 0.3517606615609975D-1
      tmp(259) = 0.351078954852546D-1
      tmp(260) = 0.3504011962524446D-1
      tmp(261) = 0.3497273477981129D-1
      tmp(262) = 0.3490573720360476D-1
      tmp(263) = 0.348391232013078D-1
      tmp(264) = 0.347728891267806D-1
      tmp(265) = 0.3470703138222231D-1
      tmp(266) = 0.3464154641735019D-1
      tmp(267) = 0.3457643072859577D-1
      tmp(268) = 0.345116808583175D-1
      tmp(269) = 0.344472933940296D-1
      tmp(270) = 0.3438326496764664D-1
      tmp(271) = 0.3431959225474359D-1
      tmp(272) = 0.3425627197383078D-1
      tmp(273) = 0.3419330088564359D-1
      tmp(274) = 0.3413067579244644D-1
      tmp(275) = 0.3406839353735074D-1
      tmp(276) = 0.3400645100364646D-1
      tmp(277) = 0.339448451141471D-1
      tmp(278) = 0.3388357283054756D-1
      tmp(279) = 0.3382263115279478D-1
      tmp(280) = 0.3376201711847077D-1
      tmp(281) = 0.3370172780218779D-1
      tmp(282) = 0.3364176031499528D-1
      tmp(283) = 0.3358211180379848D-1
      tmp(284) = 0.3352277945078824D-1
      tmp(285) = 0.3346376047288192D-1
      tmp(286) = 0.3340505212117511D-1
      tmp(287) = 0.3334665168040382D-1
      tmp(288) = 0.3328855646841706D-1
      tmp(289) = 0.3323076383565939D-1
      tmp(290) = 0.3317327116466344D-1
      tmp(291) = 0.3311607586955195D-1
      tmp(292) = 0.3305917539554928D-1
      tmp(293) = 0.3300256721850211D-1
      tmp(294) = 0.329462488444091D-1
      tmp(295) = 0.3289021780895943D-1
      tmp(296) = 0.3283447167707984D-1
      tmp(297) = 0.3277900804249017D-1
      tmp(298) = 0.3272382452726713D-1
      tmp(299) = 0.3266891878141601D-1
      tmp(300) = 0.3261428848245043D-1
      tmp(301) = 0.3255993133497968D-1
      tmp(302) = 0.3250584507030363D-1
      tmp(303) = 0.3245202744601505D-1
      tmp(304) = 0.3239847624560908D-1
      tmp(305) = 0.3234518927809986D-1
      tmp(306) = 0.3229216437764396D-1
      tmp(307) = 0.3223939940317068D-1
      tmp(308) = 0.3218689223801894D-1
      tmp(309) = 0.321346407895806D-1
      tmp(310) = 0.3208264298895021D-1
      tmp(311) = 0.3203089679058093D-1
      tmp(312) = 0.3197940017194656D-1
      tmp(313) = 0.3192815113320946D-1
      tmp(314) = 0.3187714769689443D-1
      tmp(315) = 0.3182638790756817D-1
      tmp(316) = 0.3177586983152441D-1
      tmp(317) = 0.3172559155647453D-1
      tmp(318) = 0.316755511912435D-1
      tmp(319) = 0.316257468654711D-1
      tmp(320) = 0.3157617672931833D-1
      tmp(321) = 0.3152683895317877D-1
      tmp(322) = 0.31477731727395D-1
      tmp(323) = 0.3142885326197979D-1
      tmp(324) = 0.3138020178634205D-1
      tmp(325) = 0.3133177554901745D-1
      tmp(326) = 0.3128357281740357D-1
      tmp(327) = 0.3123559187749958D-1
      tmp(328) = 0.3118783103365019D-1
      tmp(329) = 0.3114028860829402D-1
      tmp(330) = 0.3109296294171606D-1
      tmp(331) = 0.3104585239180437D-1
      tmp(332) = 0.3099895533381071D-1
      tmp(333) = 0.3095227016011521D-1
      tmp(334) = 0.3090579527999492D-1
      tmp(335) = 0.3085952911939613D-1
      tmp(336) = 0.3081347012071046D-1
      tmp(337) = 0.3076761674255464D-1
      tmp(338) = 0.3072196745955382D-1
      tmp(339) = 0.3067652076212844D-1
      tmp(340) = 0.3063127515628459D-1
      tmp(341) = 0.3058622916340771D-1
      tmp(342) = 0.305413813200596D-1
      tmp(343) = 0.3049673017777881D-1
      tmp(344) = 0.3045227430288409D-1
      tmp(345) = 0.3040801227628106D-1
      tmp(346) = 0.3036394269327196D-1
      tmp(347) = 0.3032006416336839D-1
      tmp(348) = 0.3027637531010705D-1
      tmp(349) = 0.3023287477086839D-1
      tmp(350) = 0.3018956119669809D-1
      tmp(351) = 0.3014643325213138D-1
      tmp(352) = 0.3010348961502008D-1
      tmp(353) = 0.3006072897636238D-1
      tmp(354) = 0.3001815004013524D-1
      tmp(355) = 0.299757515231294D-1
      tmp(356) = 0.2993353215478696D-1
      tmp(357) = 0.2989149067704148D-1
      tmp(358) = 0.2984962584416046D-1
      tmp(359) = 0.2980793642259041D-1
      tmp(360) = 0.2976642119080407D-1
      tmp(361) = 0.2972507893915018D-1
      tmp(362) = 0.2968390846970537D-1
      tmp(363) = 0.2964290859612843D-1
      tmp(364) = 0.2960207814351668D-1
      tmp(365) = 0.295614159482646D-1
      tmp(366) = 0.2952092085792451D-1
      tmp(367) = 0.2948059173106942D-1
      tmp(368) = 0.2944042743715788D-1
      tmp(369) = 0.2940042685640087D-1
      tmp(370) = 0.2936058887963068D-1
      tmp(371) = 0.2932091240817172D-1
      tmp(372) = 0.2928139635371327D-1
      tmp(373) = 0.2924203963818409D-1
      tmp(374) = 0.2920284119362888D-1
      tmp(375) = 0.291637999620866D-1
      tmp(376) = 0.2912491489547048D-1
      tmp(377) = 0.2908618495544991D-1
      tmp(378) = 0.2904760911333393D-1
      tmp(379) = 0.290091863499565D-1
      tmp(380) = 0.2897091565556342D-1
      tmp(381) = 0.2893279602970083D-1
      tmp(382) = 0.2889482648110543D-1
      tmp(383) = 0.2885700602759613D-1
      tmp(384) = 0.2881933369596741D-1
      tmp(385) = 0.2878180852188412D-1
      tmp(386) = 0.2874442954977778D-1
      tmp(387) = 0.2870719583274439D-1
      tmp(388) = 0.2867010643244368D-1
      tmp(389) = 0.2863316041899981D-1
      tmp(390) = 0.2859635687090341D-1
      tmp(391) = 0.2855969487491507D-1
      tmp(392) = 0.2852317352597017D-1
      tmp(393) = 0.28486791927085D-1
      tmp(394) = 0.2845054918926428D-1
      tmp(395) = 0.2841444443140989D-1
      tmp(396) = 0.2837847678023089D-1
      tmp(397) = 0.2834264537015484D-1
      tmp(398) = 0.2830694934324029D-1
      tmp(399) = 0.2827138784909049D-1
      tmp(400) = 0.2823596004476832D-1
      tmp(401) = 0.2820066509471236D-1
      tmp(402) = 0.2816550217065411D-1
      tmp(403) = 0.2813047045153638D-1
      tmp(404) = 0.2809556912343274D-1
      tmp(405) = 0.280607973794681D-1
      tmp(406) = 0.2802615441974036D-1
      tmp(407) = 0.2799163945124314D-1
      tmp(408) = 0.2795725168778953D-1
      tmp(409) = 0.2792299034993684D-1
      tmp(410) = 0.2788885466491247D-1
      tmp(411) = 0.2785484386654063D-1
      tmp(412) = 0.2782095719517014D-1
      tmp(413) = 0.2778719389760318D-1
      tmp(414) = 0.2775355322702497D-1
      tmp(415) = 0.2772003444293436D-1
      tmp(416) = 0.276866368110754D-1
      tmp(417) = 0.2765335960336979D-1
      tmp(418) = 0.2762020209785016D-1
      tmp(419) = 0.2758716357859436D-1
      tmp(420) = 0.2755424333566047D-1
      tmp(421) = 0.2752144066502278D-1
      tmp(422) = 0.274887548685085D-1
      tmp(423) = 0.2745618525373539D-1
      tmp(424) = 0.2742373113405012D-1
      tmp(425) = 0.2739139182846752D-1
      tmp(426) = 0.2735916666161049D-1
      tmp(427) = 0.2732705496365086D-1
      tmp(428) = 0.272950560702508D-1
      tmp(429) = 0.2726316932250518D-1
      tmp(430) = 0.2723139406688454D-1
      tmp(431) = 0.2719972965517886D-1
      tmp(432) = 0.27168175444442D-1
      tmp(433) = 0.2713673079693686D-1
      tmp(434) = 0.2710539508008127D-1
      tmp(435) = 0.2707416766639454D-1
      tmp(436) = 0.2704304793344466D-1
      tmp(437) = 0.2701203526379622D-1
      tmp(438) = 0.2698112904495892D-1
      tmp(439) = 0.2695032866933682D-1
      tmp(440) = 0.2691963353417812D-1
      tmp(441) = 0.2688904304152565D-1
      tmp(442) = 0.2685855659816791D-1
      tmp(443) = 0.268281736155908D-1
      tmp(444) = 0.2679789350992986D-1
      tmp(445) = 0.2676771570192318D-1
      tmp(446) = 0.2673763961686484D-1
      tmp(447) = 0.2670766468455894D-1
      tmp(448) = 0.266777903392742D-1
      tmp(449) = 0.2664801601969911D-1
      tmp(450) = 0.2661834116889767D-1
      tmp(451) = 0.2658876523426556D-1
      tmp(452) = 0.2655928766748699D-1
      tmp(453) = 0.2652990792449199D-1
      tmp(454) = 0.2650062546541418D-1
      tmp(455) = 0.2647143975454919D-1
      tmp(456) = 0.2644235026031342D-1
      tmp(457) = 0.2641335645520343D-1
      tmp(458) = 0.2638445781575572D-1
      tmp(459) = 0.2635565382250708D-1
      tmp(460) = 0.2632694395995533D-1
      tmp(461) = 0.2629832771652059D-1
      tmp(462) = 0.2626980458450701D-1
      tmp(463) = 0.2624137406006491D-1
      tmp(464) = 0.2621303564315339D-1
      tmp(465) = 0.2618478883750344D-1
      tmp(466) = 0.2615663315058139D-1
      tmp(467) = 0.2612856809355287D-1
      tmp(468) = 0.2610059318124714D-1
      tmp(469) = 0.2607270793212188D-1
      tmp(470) = 0.2604491186822836D-1
      tmp(471) = 0.2601720451517705D-1
      tmp(472) = 0.2598958540210361D-1
      tmp(473) = 0.2596205406163528D-1
      tmp(474) = 0.2593461002985766D-1
      tmp(475) = 0.2590725284628186D-1
      tmp(476) = 0.2587998205381209D-1
      tmp(477) = 0.2585279719871354D-1
      tmp(478) = 0.2582569783058072D-1
      tmp(479) = 0.2579868350230605D-1
      tmp(480) = 0.2577175377004895D-1
      tmp(481) = 0.2574490819320515D-1
      tmp(482) = 0.2571814633437645D-1
      tmp(483) = 0.2569146775934079D-1
      tmp(484) = 0.2566487203702264D-1
      tmp(485) = 0.2563835873946373D-1
      tmp(486) = 0.2561192744179418D-1
      tmp(487) = 0.2558557772220385D-1
      tmp(488) = 0.2555930916191412D-1
      tmp(489) = 0.2553312134514986D-1
      tmp(490) = 0.2550701385911188D-1
      tmp(491) = 0.2548098629394952D-1
      tmp(492) = 0.2545503824273368D-1
      tmp(493) = 0.2542916930143009D-1
      tmp(494) = 0.2540337906887286D-1
      tmp(495) = 0.2537766714673837D-1
      tmp(496) = 0.2535203313951944D-1
      tmp(497) = 0.2532647665449977D-1
      tmp(498) = 0.2530099730172864D-1
      tmp(499) = 0.2527559469399598D-1
      tmp(500) = 0.2525026844680761D-1
      tmp(501) = 0.252250181783608D-1
      tmp(502) = 0.2519984350952012D-1
      tmp(503) = 0.2517474406379351D-1
      tmp(504) = 0.2514971946730862D-1
      tmp(505) = 0.2512476934878947D-1
      tmp(506) = 0.2509989333953324D-1
      tmp(507) = 0.2507509107338746D-1
      tmp(508) = 0.2505036218672731D-1
      tmp(509) = 0.2502570631843329D-1
      tmp(510) = 0.2500112310986901D-1
      tmp(511) = 0.2497661220485934D-1
      tmp(512) = 0.2495217324966867D-1
      tmp(513) = 0.2492780589297954D-1
      tmp(514) = 0.2490350978587138D-1
      tmp(515) = 0.2487928458179952D-1
      tmp(516) = 0.2485512993657447D-1
      tmp(517) = 0.2483104550834135D-1
      tmp(518) = 0.2480703095755959D-1
      tmp(519) = 0.247830859469828D-1
      tmp(520) = 0.2475921014163888D-1
      tmp(521) = 0.2473540320881038D-1
      tmp(522) = 0.2471166481801498D-1
      tmp(523) = 0.2468799464098623D-1
      tmp(524) = 0.246643923516545D-1
      tmp(525) = 0.2464085762612812D-1
      tmp(526) = 0.2461739014267466D-1
      tmp(527) = 0.2459398958170254D-1
      tmp(528) = 0.2457065562574267D-1
      tmp(529) = 0.2454738795943041D-1
      tmp(530) = 0.2452418626948766D-1
      tmp(531) = 0.2450105024470512D-1
      tmp(532) = 0.244779795759248D-1
      tmp(533) = 0.2445497395602262D-1
      tmp(534) = 0.2443203307989126D-1
      tmp(535) = 0.244091566444232D-1
      tmp(536) = 0.2438634434849383D-1
      tmp(537) = 0.2436359589294486D-1
      tmp(538) = 0.2434091098056782D-1
      tmp(539) = 0.2431828931608774D-1
      tmp(540) = 0.2429573060614703D-1
      tmp(541) = 0.2427323455928948D-1
      tmp(542) = 0.2425080088594448D-1
      tmp(543) = 0.2422842929841132D-1
      tmp(544) = 0.2420611951084373D-1
      tmp(545) = 0.2418387123923449D-1
      tmp(546) = 0.2416168420140033D-1
      tmp(547) = 0.2413955811696682D-1
      tmp(548) = 0.241174927073535D-1
      tmp(549) = 0.240954876957592D-1
      tmp(550) = 0.240735428071474D-1
      tmp(551) = 0.2405165776823181D-1
      tmp(552) = 0.2402983230746209D-1
      tmp(553) = 0.2400806615500968D-1
      tmp(554) = 0.2398635904275379D-1
      tmp(555) = 0.2396471070426755D-1
      tmp(556) = 0.2394312087480425D-1
      tmp(557) = 0.2392158929128374D-1
      tmp(558) = 0.23900115692279D-1
      tmp(559) = 0.2387869981800276D-1
      tmp(560) = 0.2385734141029435D-1
      tmp(561) = 0.2383604021260659D-1
      tmp(562) = 0.2381479596999286D-1
      tmp(563) = 0.2379360842909429D-1
      tmp(564) = 0.2377247733812706D-1
      tmp(565) = 0.2375140244686986D-1
      tmp(566) = 0.2373038350665139D-1
      tmp(567) = 0.2370942027033809D-1
      tmp(568) = 0.2368851249232192D-1
      tmp(569) = 0.2366765992850826D-1
      tmp(570) = 0.2364686233630395D-1
      tmp(571) = 0.2362611947460543D-1
      tmp(572) = 0.2360543110378704D-1
      tmp(573) = 0.2358479698568932D-1
      tmp(574) = 0.2356421688360757D-1
      tmp(575) = 0.2354369056228039D-1
      tmp(576) = 0.235232177878784D-1
      tmp(577) = 0.2350279832799309D-1
      tmp(578) = 0.2348243195162568D-1
      tmp(579) = 0.2346211842917618D-1
      tmp(580) = 0.234418575324325D-1
      tmp(581) = 0.2342164903455972D-1
      tmp(582) = 0.2340149271008935D-1
      tmp(583) = 0.2338138833490887D-1
      tmp(584) = 0.2336133568625114D-1
      tmp(585) = 0.2334133454268414D-1
      tmp(586) = 0.2332138468410065D-1
      tmp(587) = 0.2330148589170808D-1
      tmp(588) = 0.2328163794801838D-1
      tmp(589) = 0.2326184063683809D-1
      tmp(590) = 0.2324209374325843D-1
      tmp(591) = 0.232223970536455D-1
      tmp(592) = 0.2320275035563057D-1
      tmp(593) = 0.2318315343810048D-1
      tmp(594) = 0.2316360609118808D-1
      tmp(595) = 0.2314410810626284D-1
      tmp(596) = 0.2312465927592144D-1
      tmp(597) = 0.2310525939397855D-1
      tmp(598) = 0.2308590825545763D-1
      tmp(599) = 0.2306660565658183D-1
      tmp(600) = 0.2304735139476499D-1
      tmp(601) = 0.2302814526860268D-1
      tmp(602) = 0.2300898707786341D-1
      tmp(603) = 0.229898766234798D-1
      tmp(604) = 0.2297081370753994D-1
      tmp(605) = 0.2295179813327873D-1
      tmp(606) = 0.2293282970506941D-1
      tmp(607) = 0.2291390822841506D-1
      tmp(608) = 0.2289503350994025D-1
      tmp(609) = 0.2287620535738274D-1
      tmp(610) = 0.2285742357958521D-1
      tmp(611) = 0.2283868798648719D-1
      tmp(612) = 0.2281999838911691D-1
      tmp(613) = 0.2280135459958332D-1
      tmp(614) = 0.2278275643106816D-1
      tmp(615) = 0.227642036978181D-1
      tmp(616) = 0.2274569621513695D-1
      tmp(617) = 0.2272723379937791D-1
      tmp(618) = 0.2270881626793595D-1
      tmp(619) = 0.2269044343924021D-1
      tmp(620) = 0.2267211513274648D-1
      tmp(621) = 0.2265383116892975D-1
      tmp(622) = 0.2263559136927683D-1
      tmp(623) = 0.2261739555627901D-1
      tmp(624) = 0.2259924355342486D-1
      tmp(625) = 0.2258113518519295D-1
      tmp(626) = 0.2256307027704479D-1
      tmp(627) = 0.2254504865541776D-1
      tmp(628) = 0.2252707014771806D-1
      tmp(629) = 0.2250913458231383D-1
      tmp(630) = 0.2249124178852821D-1
      tmp(631) = 0.2247339159663255D-1
      tmp(632) = 0.2245558383783966D-1
      tmp(633) = 0.2243781834429706D-1
      tmp(634) = 0.224200949490804D-1
      tmp(635) = 0.224024134861868D-1
      tmp(636) = 0.2238477379052839D-1
      tmp(637) = 0.2236717569792577D-1
      tmp(638) = 0.2234961904510165D-1
      tmp(639) = 0.2233210366967446D-1
      tmp(640) = 0.2231462941015202D-1
      tmp(641) = 0.2229719610592534D-1
      tmp(642) = 0.2227980359726237D-1
      tmp(643) = 0.2226245172530188D-1
      tmp(644) = 0.2224514033204737D-1
      tmp(645) = 0.22227869260361D-1
      tmp(646) = 0.2221063835395762D-1
      tmp(647) = 0.2219344745739883D-1
      tmp(648) = 0.2217629641608708D-1
      tmp(649) = 0.2215918507625986D-1
      tmp(650) = 0.2214211328498385D-1
      tmp(651) = 0.2212508089014924D-1
      tmp(652) = 0.2210808774046403D-1
      tmp(653) = 0.2209113368544834D-1
      tmp(654) = 0.2207421857542885D-1
      tmp(655) = 0.2205734226153326D-1
      tmp(656) = 0.2204050459568476D-1
      tmp(657) = 0.2202370543059659D-1
      tmp(658) = 0.2200694461976661D-1
      tmp(659) = 0.2199022201747195D-1
      tmp(660) = 0.219735374787637D-1
      tmp(661) = 0.2195689085946161D-1
      tmp(662) = 0.2194028201614886D-1
      tmp(663) = 0.2192371080616687D-1
      tmp(664) = 0.2190717708761018D-1
      tmp(665) = 0.2189068071932132D-1
      tmp(666) = 0.2187422156088574D-1
      tmp(667) = 0.2185779947262682D-1
      tmp(668) = 0.2184141431560086D-1
      tmp(669) = 0.2182506595159218D-1
      tmp(670) = 0.2180875424310818D-1
      tmp(671) = 0.2179247905337451D-1
      tmp(672) = 0.2177624024633027D-1
      tmp(673) = 0.2176003768662318D-1
      tmp(674) = 0.2174387123960489D-1
      tmp(675) = 0.2172774077132625D-1
      tmp(676) = 0.2171164614853267D-1
      tmp(677) = 0.216955872386595D-1
      tmp(678) = 0.216795639098274D-1
      tmp(679) = 0.2166357603083785D-1
      tmp(680) = 0.216476234711686D-1
      tmp(681) = 0.2163170610096922D-1
      tmp(682) = 0.2161582379105661D-1
      tmp(683) = 0.2159997641291067D-1
      tmp(684) = 0.2158416383866989D-1
      tmp(685) = 0.21568385941127D-1
      tmp(686) = 0.2155264259372472D-1
      tmp(687) = 0.2153693367055145D-1
      tmp(688) = 0.2152125904633708D-1
      tmp(689) = 0.2150561859644875D-1
      tmp(690) = 0.2149001219688674D-1
      tmp(691) = 0.214744397242803D-1
      tmp(692) = 0.2145890105588357D-1
      tmp(693) = 0.2144339606957152D-1
      tmp(694) = 0.2142792464383589D-1
      tmp(695) = 0.2141248665778126D-1
      tmp(696) = 0.2139708199112098D-1
      tmp(697) = 0.2138171052417334D-1
      tmp(698) = 0.2136637213785758D-1
      tmp(699) = 0.2135106671369006D-1
      tmp(700) = 0.2133579413378041D-1
      tmp(701) = 0.2132055428082771D-1
      tmp(702) = 0.213053470381167D-1
      tmp(703) = 0.2129017228951406D-1
      tmp(704) = 0.2127502991946461D-1
      tmp(705) = 0.2125991981298772D-1
      tmp(706) = 0.2124484185567355D-1
      tmp(707) = 0.2122979593367944D-1
      tmp(708) = 0.2121478193372635D-1
      tmp(709) = 0.2119979974309518D-1
      tmp(710) = 0.2118484924962333D-1
      tmp(711) = 0.2116993034170106D-1
      tmp(712) = 0.2115504290826807D-1
      tmp(713) = 0.2114018683881002D-1
      tmp(714) = 0.2112536202335503D-1
      tmp(715) = 0.2111056835247033D-1
      tmp(716) = 0.2109580571725881D-1
      tmp(717) = 0.210810740093557D-1
      tmp(718) = 0.2106637312092519D-1
      tmp(719) = 0.2105170294465713D-1
      tmp(720) = 0.2103706337376377D-1
      tmp(721) = 0.2102245430197643D-1
      tmp(722) = 0.2100787562354233D-1
      tmp(723) = 0.2099332723322132D-1
      tmp(724) = 0.2097880902628271D-1
      tmp(725) = 0.2096432089850213D-1
      tmp(726) = 0.2094986274615834D-1
      tmp(727) = 0.2093543446603013D-1
      tmp(728) = 0.2092103595539324D-1
      tmp(729) = 0.2090666711201729D-1
      tmp(730) = 0.2089232783416268D-1
      tmp(731) = 0.2087801802057764D-1
      tmp(732) = 0.2086373757049516D-1
      tmp(733) = 0.2084948638363007D-1
      tmp(734) = 0.2083526436017603D-1
      tmp(735) = 0.2082107140080261D-1
      tmp(736) = 0.208069074066524D-1
      tmp(737) = 0.207927722793381D-1
      tmp(738) = 0.2077866592093964D-1
      tmp(739) = 0.2076458823400133D-1
      tmp(740) = 0.2075053912152907D-1
      tmp(741) = 0.2073651848698749D-1
      tmp(742) = 0.2072252623429722D-1
      tmp(743) = 0.2070856226783206D-1
      tmp(744) = 0.2069462649241629D-1
      tmp(745) = 0.2068071881332193D-1
      tmp(746) = 0.20666839136266D-1
      tmp(747) = 0.2065298736740792D-1
      tmp(748) = 0.2063916341334673D-1
      tmp(749) = 0.2062536718111856D-1
      tmp(750) = 0.2061159857819392D-1
      tmp(751) = 0.2059785751247512D-1
      tmp(752) = 0.2058414389229371D-1
      tmp(753) = 0.2057045762640788D-1
      tmp(754) = 0.205567986239999D-1
      tmp(755) = 0.2054316679467365D-1
      tmp(756) = 0.2052956204845201D-1
      tmp(757) = 0.2051598429577446D-1
      tmp(758) = 0.2050243344749456D-1
      tmp(759) = 0.2048890941487747D-1
      tmp(760) = 0.2047541210959758D-1
      tmp(761) = 0.20461941443736D-1
      tmp(762) = 0.2044849732977823D-1
      tmp(763) = 0.2043507968061171D-1
      tmp(764) = 0.2042168840952349D-1
      tmp(765) = 0.2040832343019789D-1
      tmp(766) = 0.203949846567141D-1
      tmp(767) = 0.2038167200354392D-1
      tmp(768) = 0.2036838538554943D-1
      tmp(769) = 0.2035512471798072D-1
      tmp(770) = 0.2034188991647358D-1
      tmp(771) = 0.203286808970473D-1
      tmp(772) = 0.2031549757610239D-1
      tmp(773) = 0.2030233987041838D-1
      tmp(774) = 0.2028920769715162D-1
      tmp(775) = 0.2027610097383305D-1
      tmp(776) = 0.2026301961836606D-1
      tmp(777) = 0.2024996354902433D-1
      tmp(778) = 0.2023693268444966D-1
      tmp(779) = 0.2022392694364989D-1
      tmp(780) = 0.2021094624599671D-1
      tmp(781) = 0.2019799051122364D-1
      tmp(782) = 0.2018505965942388D-1
      tmp(783) = 0.2017215361104829D-1
      tmp(784) = 0.201592722869033D-1
      tmp(785) = 0.201464156081489D-1
      tmp(786) = 0.2013358349629658D-1
      tmp(787) = 0.2012077587320733D-1
      tmp(788) = 0.2010799266108966D-1
      tmp(789) = 0.200952337824976D-1
      tmp(790) = 0.2008249916032872D-1
      tmp(791) = 0.2006978871782218D-1
      tmp(792) = 0.200571023785568D-1
      tmp(793) = 0.2004444006644913D-1
      tmp(794) = 0.2003180170575149D-1
      tmp(795) = 0.2001918722105014D-1
      tmp(796) = 0.2000659653726332D-1
      tmp(797) = 0.1999402957963941D-1
      tmp(798) = 0.1998148627375507D-1
      tmp(799) = 0.1996896654551336D-1
      tmp(800) = 0.1995647032114195D-1
      tmp(801) = 0.1994399752719124D-1
      tmp(802) = 0.1993154809053257D-1
      tmp(803) = 0.1991912193835643D-1
      tmp(804) = 0.1990671899817065D-1
      tmp(805) = 0.1989433919779865D-1
      tmp(806) = 0.1988198246537766D-1
      tmp(807) = 0.1986964872935696D-1
      tmp(808) = 0.1985733791849614D-1
      tmp(809) = 0.1984504996186341D-1
      tmp(810) = 0.1983278478883383D-1
      tmp(811) = 0.1982054232908763D-1
      tmp(812) = 0.1980832251260854D-1
      tmp(813) = 0.1979612526968206D-1
      tmp(814) = 0.1978395053089381D-1
      tmp(815) = 0.1977179822712791D-1
      tmp(816) = 0.1975966828956525D-1
      tmp(817) = 0.1974756064968194D-1
      tmp(818) = 0.1973547523924762D-1
      tmp(819) = 0.1972341199032387D-1
      tmp(820) = 0.1971137083526263D-1
      tmp(821) = 0.1969935170670454D-1
      tmp(822) = 0.1968735453757744D-1
      tmp(823) = 0.1967537926109472D-1
      tmp(824) = 0.1966342581075384D-1
      tmp(825) = 0.1965149412033469D-1
      tmp(826) = 0.1963958412389813D-1
      tmp(827) = 0.1962769575578439D-1
      tmp(828) = 0.196158289506116D-1
      tmp(829) = 0.1960398364327428D-1
      tmp(830) = 0.1959215976894178D-1
      tmp(831) = 0.1958035726305688D-1
      tmp(832) = 0.1956857606133422D-1
      tmp(833) = 0.195568160997589D-1
      tmp(834) = 0.1954507731458498D-1
      tmp(835) = 0.1953335964233403D-1
      tmp(836) = 0.1952166301979371D-1
      tmp(837) = 0.1950998738401632D-1
      tmp(838) = 0.1949833267231738D-1
      tmp(839) = 0.1948669882227423D-1
      tmp(840) = 0.1947508577172461D-1
      tmp(841) = 0.1946349345876525D-1
      tmp(842) = 0.1945192182175052D-1
      tmp(843) = 0.1944037079929105D-1
      tmp(844) = 0.1942884033025233D-1
      tmp(845) = 0.1941733035375336D-1
      tmp(846) = 0.1940584080916534D-1
      tmp(847) = 0.1939437163611028D-1
      tmp(848) = 0.1938292277445968D-1
      tmp(849) = 0.1937149416433323D-1
      tmp(850) = 0.1936008574609747D-1
      tmp(851) = 0.1934869746036447D-1
      tmp(852) = 0.1933732924799057D-1
      tmp(853) = 0.1932598105007509D-1
      tmp(854) = 0.1931465280795898D-1
      tmp(855) = 0.1930334446322364D-1
      tmp(856) = 0.1929205595768959D-1
      tmp(857) = 0.1928078723341524D-1
      tmp(858) = 0.1926953823269563D-1
      tmp(859) = 0.1925830889806119D-1
      tmp(860) = 0.1924709917227652D-1
      tmp(861) = 0.1923590899833915D-1
      tmp(862) = 0.1922473831947833D-1
      tmp(863) = 0.1921358707915381D-1
      tmp(864) = 0.1920245522105464D-1
      tmp(865) = 0.1919134268909802D-1
      tmp(866) = 0.1918024942742802D-1
      tmp(867) = 0.1916917538041449D-1
      tmp(868) = 0.1915812049265185D-1
      tmp(869) = 0.1914708470895793D-1
      tmp(870) = 0.191360679743728D-1
      tmp(871) = 0.1912507023415764D-1
      tmp(872) = 0.191140914337936D-1
      tmp(873) = 0.1910313151898065D-1
      tmp(874) = 0.1909219043563644D-1
      tmp(875) = 0.1908126812989523D-1
      tmp(876) = 0.1907036454810672D-1
      tmp(877) = 0.1905947963683497D-1
      tmp(878) = 0.190486133428573D-1
      tmp(879) = 0.1903776561316319D-1
      tmp(880) = 0.190269363949532D-1
      tmp(881) = 0.1901612563563788D-1
      tmp(882) = 0.1900533328283673D-1
      tmp(883) = 0.1899455928437707D-1
      tmp(884) = 0.1898380358829305D-1
      tmp(885) = 0.1897306614282456D-1
      tmp(886) = 0.1896234689641618D-1
      tmp(887) = 0.1895164579771617D-1
      tmp(888) = 0.1894096279557541D-1
      tmp(889) = 0.1893029783904637D-1
      tmp(890) = 0.1891965087738211D-1
      tmp(891) = 0.1890902186003527D-1
      tmp(892) = 0.1889841073665702D-1
      tmp(893) = 0.1888781745709612D-1
      tmp(894) = 0.1887724197139785D-1
      tmp(895) = 0.1886668422980311D-1
      tmp(896) = 0.1885614418274736D-1
      tmp(897) = 0.1884562178085966D-1
      tmp(898) = 0.1883511697496175D-1
      tmp(899) = 0.18824629716067D-1
      tmp(900) = 0.1881415995537953D-1
      tmp(901) = 0.1880370764429321D-1
      tmp(902) = 0.1879327273439072D-1
      tmp(903) = 0.1878285517744261D-1
      tmp(904) = 0.1877245492540637D-1
      tmp(905) = 0.1876207193042551D-1
      tmp(906) = 0.1875170614482859D-1
      tmp(907) = 0.1874135752112835D-1
      tmp(908) = 0.1873102601202078D-1
      tmp(909) = 0.1872071157038421D-1
      tmp(910) = 0.1871041414927838D-1
      tmp(911) = 0.1870013370194362D-1
      tmp(912) = 0.1868987018179985D-1
      tmp(913) = 0.1867962354244579D-1
      tmp(914) = 0.1866939373765803D-1
      tmp(915) = 0.1865918072139017D-1
      tmp(916) = 0.1864898444777192D-1
      tmp(917) = 0.1863880487110829D-1
      tmp(918) = 0.1862864194587868D-1
      tmp(919) = 0.1861849562673604D-1
      tmp(920) = 0.1860836586850605D-1
      tmp(921) = 0.1859825262618621D-1
      tmp(922) = 0.1858815585494506D-1
      tmp(923) = 0.1857807551012134D-1
      tmp(924) = 0.1856801154722312D-1
      tmp(925) = 0.18557963921927D-1
      tmp(926) = 0.1854793259007731D-1
      tmp(927) = 0.1853791750768526D-1
      tmp(928) = 0.1852791863092815D-1
      tmp(929) = 0.1851793591614855D-1
      tmp(930) = 0.1850796931985353D-1
      tmp(931) = 0.1849801879871383D-1
      tmp(932) = 0.1848808430956307D-1
      tmp(933) = 0.1847816580939699D-1
      tmp(934) = 0.1846826325537266D-1
      tmp(935) = 0.1845837660480769D-1
      tmp(936) = 0.1844850581517945D-1
      tmp(937) = 0.1843865084412433D-1
      tmp(938) = 0.1842881164943697D-1
      tmp(939) = 0.1841898818906946D-1
      tmp(940) = 0.1840918042113066D-1
      tmp(941) = 0.1839938830388538D-1
      tmp(942) = 0.1838961179575366D-1
      tmp(943) = 0.1837985085531006D-1
      tmp(944) = 0.1837010544128285D-1
      tmp(945) = 0.1836037551255336D-1
      tmp(946) = 0.1835066102815518D-1
      tmp(947) = 0.1834096194727349D-1
      tmp(948) = 0.1833127822924431D-1
      tmp(949) = 0.1832160983355378D-1
      tmp(950) = 0.1831195671983747D-1
      tmp(951) = 0.1830231884787966D-1
      tmp(952) = 0.1829269617761264D-1
      tmp(953) = 0.1828308866911599D-1
      tmp(954) = 0.1827349628261593D-1
      tmp(955) = 0.1826391897848458D-1
      tmp(956) = 0.182543567172393D-1
      tmp(957) = 0.18244809459542D-1
      tmp(958) = 0.1823527716619846D-1
      tmp(959) = 0.1822575979815765D-1
      tmp(960) = 0.1821625731651106D-1
      tmp(961) = 0.1820676968249204D-1
      tmp(962) = 0.1819729685747514D-1
      tmp(963) = 0.1818783880297541D-1
      tmp(964) = 0.181783954806478D-1
      tmp(965) = 0.1816896685228647D-1
      tmp(966) = 0.1815955287982414D-1
      tmp(967) = 0.1815015352533148D-1
      tmp(968) = 0.1814076875101642D-1
      tmp(969) = 0.1813139851922354D-1
      tmp(970) = 0.1812204279243343D-1
      tmp(971) = 0.1811270153326207D-1
      tmp(972) = 0.1810337470446019D-1
      tmp(973) = 0.1809406226891263D-1
      tmp(974) = 0.1808476418963775D-1
      tmp(975) = 0.180754804297868D-1
      tmp(976) = 0.1806621095264332D-1
      tmp(977) = 0.180569557216225D-1
      tmp(978) = 0.180477147002706D-1
      tmp(979) = 0.1803848785226432D-1
      tmp(980) = 0.1802927514141026D-1
      tmp(981) = 0.1802007653164423D-1
      tmp(982) = 0.1801089198703075D-1
      tmp(983) = 0.1800172147176241D-1
      tmp(984) = 0.1799256495015927D-1
      tmp(985) = 0.1798342238666834D-1
      tmp(986) = 0.1797429374586292D-1
      tmp(987) = 0.179651789924421D-1
      tmp(988) = 0.1795607809123012D-1
      tmp(989) = 0.1794699100717586D-1
      tmp(990) = 0.1793791770535221D-1
      tmp(991) = 0.1792885815095557D-1
      tmp(992) = 0.1791981230930524D-1
      tmp(993) = 0.1791078014584288D-1
      tmp(994) = 0.1790176162613199D-1
      tmp(995) = 0.1789275671585727D-1
      tmp(996) = 0.1788376538082418D-1
      tmp(997) = 0.178747875869583D-1
      tmp(998) = 0.1786582330030486D-1
      tmp(999) = 0.1785687248702816D-1
      tmp(1000) = 0.1784793511341103D-1
      tmp(1001) = 0.1783901114585432D-1

        return
      end

C =============================
      subroutine lslslsccoeff(co)
      implicit none
      doubleprecision co(0:60,0:1)
      co(0,0) = 0.D0
      co(0,1) = 0.D0
      co(1,0) = -0.5479809610733532D0
      co(1,1) = 0.D0
      co(2,0) = 0.7871721686916723D0
      co(2,1) = -0.3125D0
      co(3,0) = 0.4875354567060364D0
      co(3,1) = -0.2196026152894707D0
      co(4,0) = 0.301906632822785D0
      co(4,1) = -0.1475694444444444D0
      co(5,0) = 0.1943496130598485D0
      co(5,1) = -0.1003897669894724D0
      co(6,0) = 0.1292007589501331D0
      co(6,1) = -0.6944444444444444D-1
      co(7,0) = 0.8802288738309138D-1
      co(7,1) = -0.4875531161961657D-1
      co(8,0) = 0.6110899736459466D-1
      co(8,1) = -0.3465553130731702D-1
      co(9,0) = 0.4305571067127435D-1
      co(9,1) = -0.2488664812333832D-1
      co(10,0) = 0.3069726698609974D-1
      co(10,1) = -0.1802456284249001D-1
      co(11,0) = 0.2209901452765446D-1
      co(11,1) = -0.1314874335855613D-1
      co(12,0) = 0.1603767845451614D-1
      co(12,1) = -0.9650780894286271D-2
      co(13,0) = 0.1171815017444808D-1
      co(13,1) = -0.7120804824842238D-2
      co(14,0) = 0.861184739295683D-2
      co(14,1) = -0.5278180901697179D-2
      co(15,0) = 0.6360780576178906D-2
      co(15,1) = -0.3928098777865704D-2
      co(16,0) = 0.471871072115472D-2
      co(16,1) = -0.2933729542916685D-2
      co(17,0) = 0.3514040992315146D-2
      co(17,1) = -0.2197997284371893D-2
      co(18,0) = 0.2625857084907464D-2
      co(18,1) = -0.1651429269649709D-2
      co(19,0) = 0.1968143747958109D-2
      co(19,1) = -0.124393080783512D-2
      co(20,0) = 0.1479207045530762D-2
      co(20,1) = -0.9391412257765185D-3
      co(21,0) = 0.1114479554871409D-2
      co(21,1) = -0.7105150411859307D-3
      co(22,0) = 0.8415633287707858D-3
      co(22,1) = -0.5385727587134736D-3
      co(23,0) = 0.6367761922650827D-3
      co(23,1) = -0.4089546182367798D-3
      co(24,0) = 0.4827217572176512D-3
      co(24,1) = -0.3110316979903576D-3
      co(25,0) = 0.3665649031857753D-3
      co(25,1) = -0.236907697816267D-3
      co(26,0) = 0.2787985455894174D-3
      co(26,1) = -0.1806969599709341D-3
      co(27,0) = 0.212355736192762D-3
      co(27,1) = -0.1379993260019689D-3
      co(28,0) = 0.1619665195831772D-3
      co(28,1) = -0.1055162248303935D-3
      co(29,0) = 0.1236895477359346D-3
      co(29,1) = -0.8076872575832363D-4
      co(30,0) = 0.9456930165562304D-4
      co(30,1) = -0.6188961931101486D-4
      co(31,0) = 0.7238413117289548D-4
      co(31,1) = -0.4746951468603535D-4
      co(32,0) = 0.5546031222841853D-4
      co(32,1) = -0.3644253222313644D-4
      co(33,0) = 0.4253432676682258D-4
      co(33,1) = -0.2800114011810101D-4
      co(34,0) = 0.3265052220202196D-4
      co(34,1) = -0.2153252407678062D-4
      co(35,0) = 0.2508483196690033D-4
      co(35,1) = -0.1657092127554611D-4
      co(36,0) = 0.1928776325170918D-4
      co(36,1) = -0.1276182749486053D-4
      co(37,0) = 0.148416768178944D-4
      co(37,1) = -0.9835060327608147D-5
      co(38,0) = 0.1142870097641226D-4
      co(38,1) = -0.7584445669196417D-5
      co(39,0) = 0.880658035862342D-5
      co(39,1) = -0.5852469821461792D-5
      co(40,0) = 0.6790460718840457D-5
      co(40,1) = -0.4518662893901082D-5
      co(41,0) = 0.5239125165571207D-5
      co(41,1) = -0.3490791914940496D-5
      co(42,0) = 0.4044575946571609D-5
      co(42,1) = -0.2698173829111734D-5
      co(43,0) = 0.312413583854687D-5
      co(43,1) = -0.2086590830529041D-5
      co(44,0) = 0.2414450744955096D-5
      co(44,1) = -0.1614418893688862D-5
      co(45,0) = 0.186692944610055D-5
      co(45,1) = -0.1249676323889086D-5
      co(46,0) = 0.1444271527495081D-5
      co(46,1) = -0.9677710325950709D-6
      co(47,0) = 0.1117820536921602D-5
      co(47,1) = -0.7497792090034692D-6
      co(48,0) = 0.8655438746354518D-6
      co(48,1) = -0.581128657544166D-6
      co(49,0) = 0.6704893184055123D-6
      co(49,1) = -0.4505906721684663D-6
      co(50,0) = 0.5196044993434828D-6
      co(50,1) = -0.3495074254388054D-6
      co(51,0) = 0.4028330945438647D-6
      co(51,1) = -0.2711993283051712D-6
      co(52,0) = 0.3124222318522643D-6
      co(52,1) = -0.2105100573297128D-6
      co(53,0) = 0.2423912820761579D-6
      co(53,1) = -0.1634569920327645D-6
      co(54,0) = 0.1881240930492645D-6
      co(54,1) = -0.1269624375508549D-6
      co(55,0) = 0.1460557326867901D-6
      co(55,1) = -0.9864681432530845D-7
      co(56,0) = 0.1134316555847338D-6
      co(56,1) = -0.7666941939622498D-7
      co(57,0) = 0.8812241679000986D-7
      co(57,1) = -0.5960573672216133D-7
      co(58,0) = 0.684810239477711D-7
      co(58,1) = -0.4635284890009195D-7
      co(59,0) = 0.5323304450155529D-7
      co(59,1) = -0.3605647052909551D-7
      co(60,0) = 0.4139189420985157D-7
      co(60,1) = -0.2805462886294721D-7
      return
      end
