* cuba.F
* Fortran chooser for the Cuba routines
* last modified 3 Feb 05 th

#define VEGAS 1
#define SUAVE 2
#define DIVONNE 3
#define CUHRE 4


	subroutine Cuba(method, AdaptToFirst, ndim, ncomp, integrand,
     &    epsrel,epsabs,thermsteps,maxeval,nstart,nincrease,ncadapt,
     &    integral, error, prob)
	implicit none
	integer AdaptToFirst
	integer method, ndim, ncomp
	external integrand
	double precision integral(*), error(*), prob(*)

	character*7 name(4)
	data name /"Vegas", "Suave", "Divonne", "Cuhre"/

	integer mineval
	integer thermsteps, maxeval, verbose, last

	integer ncadapt

	double precision epsrel, epsabs
C	parameter (epsrel = 1D-3)
C	parameter (epsabs = 1D-12)
	parameter (verbose = 1)
	parameter (last = 0)
C	parameter (mineval = 0)
C	parameter (maxeval = 50000)

	integer nstart, nincrease
C	parameter (nstart = 1000)
C	parameter (nincrease = 500)

CCC     iprng=0: Sobol quasi-random number generator, 
CCC     iprng=8: Mersenne-Twister pseudo-random generator
	integer iprng
	parameter (iprng=0)

CCC     ismoothing= 0: additional smoothing to importance function applied, 
CCC     ismoothing=16: no additional smoothing
      	integer ismoothing
	parameter(ismoothing=16)

	integer nnew
	double precision flatness
	parameter (nnew = 1000)
	parameter (flatness = 25D0)

	integer key1, key2, key3, maxpass
	double precision border, maxchisq, mindeviation
	integer ngiven, ldxgiven, nextra
	parameter (key1 = 47)
	parameter (key2 = 1)
	parameter (key3 = 1)
	parameter (maxpass = 5)
	parameter (border = 0D0)
	parameter (maxchisq = 10D0)
	parameter (mindeviation = .25D0)
	parameter (ngiven = 0)
C	parameter (ldxgiven = ndim)
	parameter (nextra = 0)

	integer key
	parameter (key = 0)

	integer nregions, neval, fail
	integer c

	integer adap1st

	integer seed
	common/mersenneseed/seed

CCC     long long integer versions
	integer llthermsteps, llmaxeval, llnstart, llnincrease
	integer llneval,llnnew,llngiven,llnextra,llmineval

	if (AdaptToFirst.eq.1) then
	   adap1st=32
	else
	   adap1st=0
	endif

	call zufalli(seed)

	ldxgiven=ndim

	llthermsteps=thermsteps
	llmaxeval=maxeval
	llnstart=nstart
	llnincrease=nincrease
	llneval=neval
	llnnew=nnew
	llngiven=ngiven
	llnextra=nextra


	if( method .eq. VEGAS ) then

	  call vegas(ndim, ncomp, integrand,
     &      epsrel,epsabs,adap1st+(verbose+iprng),llthermsteps,
     &	    llmaxeval,llnstart, llnincrease,ncadapt,
     &      llneval, fail, integral, error, prob)
	  nregions = 1

	else if( method .eq. SUAVE ) then

	   llmineval=llthermsteps*llnstart
	   call suave(ndim, ncomp, integrand,
     &      epsrel,epsabs,adap1st+(verbose+last+iprng),llmineval,
     &	    llmaxeval,
     &      llnnew, flatness,
     &      nregions, llneval, fail, integral, error, prob)

	else if( method .eq. DIVONNE ) then

	   llmineval=thermsteps*nstart
	   call divonne(ndim, ncomp, integrand,
     &      epsrel,epsabs, adap1st+(verbose+iprng),llmineval,
     &      llmaxeval,
     &      key1, key2, key3, maxpass,
     &      border, maxchisq, mindeviation,
     &      llngiven, ldxgiven, 0, llnextra, 0,
     &      nregions, llneval, fail, integral, error, prob)

	else if( method .eq. CUHRE ) then

	   llmineval=nstart
	   call cuhre(ndim, ncomp, integrand,
     $	    epsrel,epsabs,adap1st+(verbose+last+iprng),llmineval,
     &      llmaxeval,
     &      key,
     &      nregions, llneval, fail, integral, error, prob)

	else

	  print *, "invalid method ", method
	  return

	endif

	print *, "method   =", name(method)
	print *, "nregions =", nregions
	print *, "neval    =", llneval
	print *, "fail     =", fail
	print '(G20.12," +- ",G20.12,"   p = ",F8.3)',
     &    (integral(c), error(c), prob(c), c = 1, ncomp)
	end

