/*
	ChiSquare.c
		the chi-square cdf
		after W.J. Kennedy and J.E. Gentle,
		Statistical computing, p. 116
		last modified 9 Feb 05 th
*/

#ifdef HAVE_ERF
#define NErf Nerf
#else
#include "NErf.c"
#endif

static inline double NNormal(double x)
{
  return .5*NErf(x/1.414213562373095048801689) + .5;
}

static inline double Max(double a, double b)
{
  return (a > b) ? a : b;
}

/*********************************************************************/

static double NChiSquare(double x, int df)
{
  double y;

  if( df <= 0 ) return -999;

  if( x <= 0 ) return 0;
  if( x > 1000*df ) return 1;

  if( df > 1000 ) {
    if( x < 2 ) return 0;
    y = 2./(9*df);
    y = (pow(x/df, 1/3.) - (1 - y))/sqrt(y);
    if( y > 5 ) return 1;
    if( y < -18.8055 ) return 0;
    return NNormal(y);
  }

  y = .5*x;

  if( df & 1 ) {
    double sqrty = sqrt(y);
    double h = NErf(sqrty);
    int i;

    if( df == 1 ) return h;

    y = sqrty*exp(-y)/.8862269254527579825931;
    for( i = 3; i < df; i += 2 ) {
      h -= y;
      y *= x/i;
    }
    y = h - y;
  }
  else {
    double term = exp(-y), sum = term;
    int i;

    for( i = 1; i < df/2; ++i )
      sum += term *= y/i;
    y = 1 - sum;
  }

  return Max(0., y);
}

