#!/usr/bin/env python 

#########################################################################################
#								user input
#
#
#								physics input
#
results_dir = 'results'                 # top directory with results from different runs
project = 'ZZ'                          # directory with results of current run
type = 'exclusive'                      # inclusive (fast, no decay) or exclusive
m_higgs	= ['220']                       # mass value or scan set
#m_higgs = range(100,625,25)            #  mass scan: eg. from 100 to 600 with a step of 25
factorization_scale = ['0.5']           # factorization scale value or scan range
vary_a_s = 'no'                        # vary the a_s within the pdf sets ? (yes or no)
perturbative_order='NLO'               # LO, NLO, NNLO
collider = 'LHC'                        # LHC, Tevatron
sqrtS = '7000'                          # in GeV
electroweak_corrections = 'virtual-real'# 'off', 'only-virtual', 'virtual-real'
decay = 'ZZ->l1 l1 l2 l2'               #'none' , gamma gamma, WW, ZZ, WW-ZZ , ZZ->l1 l1 l2 l2
massive_corrections = 'top-bottom'		#none, top-only, top-bottom
pdfset= '4.0'                           #-- 4.0= MSTW 2008; 3.0= MRST 2006; 2.0= MRST 2004; 1.0= MRST 2001
calculate_pdf_error = 'no'             # yes or no
#
ifcluster = 'no'						# are you running on a cluster ? (yes or no)
walltime = '480'                        # time limit in minutes for the job (in case you run on a cluster with time limited queues)
submitts='1'                            #-- For cluster: Number of submissions of the same job. 
#
accRel='0.1'                           # in percentage, i.e. 0.8 means 0.8%
accAbs = '0.0'                          # requires knowledge of the cross section (per sector in the exclusive mode case)
ncalls = '10000'                        # points per iteration in thermalization stage
nincr= '0'                              # increase in points from iteration to iteration in thermalization stage
ntherm = '10' 
WilsonFile = 'xxxxx'                    # file for redefinition of Wilson coefficients
higgs_width_flag = 'no'                 # finite higgs width effects, 'yes' or 'no'


#----------------------     cuts

#########  Cuts on Higgs directly  #########
#-- PtHiggsCut>0.5: Cut on Higgs pT, PtHiggsMin < pT < PtHiggsMax
#PtHiggsCut= '0.0'
#PtHiggsMin= '0.0'
#PtHiggsMax= '0.0'

#-- EtaHiggsCut>0.5: Cut on Higgs rapidity, EtaHiggsMin < y < EtaHiggsMax
#EtaHiggsCut= '0.0'
#EtaHiggsMin= '0.0'
#EtaHiggsMax= '0.0'

#########  General cuts  #########
#-- UseSISCone=0.0: simple cone algorithm
#-- UseSISCone>0.5: SISCone jet algorithm
#UseSISCone= '0.0'

#-- Cone size for jet definition. In the case of SISCone, this gets rescaled.  
#JetConeSize= '0.4'

#-- Minimum pT for jet definition 
#JetDefPtMin= '0.0'

#-- Maximum rapidity for jet definition
#JetDefEtaMax='0.0'

#-- Jet veto
#-- JetVeto>0.5: No jet with pt_jet>JetPtMax and |eta_jet|<JetEtaMax
#JetVeto= '1.0'
#JetPtMax= '35.0'
#JetEtaMax= '3.0'

#-- NumVeto>0.5: cut on number of jets <= JetNmax
#NumVeto= '0.0'
#JetNmax= '0'

#-- Special Tevatron cut on number of jets.
#-- If number of Jets =1, require pTjet < TeVJetPtMax1
#-- if number of Jets =2, require pTjet1,pTjet2 < TeVJetPtMax2
#TeVVeto= '0.0'
#TeVJetPtMax1= '0.0'
#TeVJetPtMax2= '0.0'

#########  Specific cuts for ZZ decay  #########
#-- Cuts on vector boson
#-- MassV1Cut>0.5: Cut on mass of V1,  MassV1Min < m_V1 < MassV1Max
#MassV1Cut=  '1.0'
#MassV1Min= '85.0'
#MassV1Max= '95.0'

#-- MassV2Cut>0.5: Cut on mass of V2,  MassV2Min < m_V2 < MassV2Max
#MassV2Cut=  '0.0'
#MassV2Min= '0.0'
#MassV2Max= '0.0'

#-- PtV1Cut>0.5: Cut on pT of V1,  PtV1Min < pT_V1 < PtV1Max
#PtV1Cut= '0.0'
#PtV1Min= '0.0'
#PtV1Max= '0.0'

#-- PtV2Cut>0.5: Cut on pT of V2,  PtV2Min < pT_V2 < PtV2Max
#PtV2Cut= '0.0'
#PtV2Min= '0.0'
#PtV2Max= '0.0'

#-- EtaV1Cut>0.5: Cut on rapidity of V1,  EtaV1Min < y_V1 < EtaV1Max
#EtaV1Cut= '0.0'
#EtaV1Min= '0.0'
#EtaV1Max= '0.0'

#-- EtaV2Cut>0.5: Cut on rapidity of V2,  EtaV2Min < y_V2 < EtaV2Max
#EtaV2Cut= '0.0'
#EtaV2Min= '0.0'
#EtaV2Max= '0.0'

#-- cut_on_pt_of_all_leptons = 1.0 : cut on the PT of all leptons
#cut_on_pt_of_all_leptons=1.0
#all_leptons_minimum_pt=20.0

#-- cut_on_rapidity_of_all_leptons = 1.0 : cut on the rapidity of all leptons
#cut_on_rapidity_of_all_leptons=1.0
#all_leptons_maximum_y=2.5

#
#
#								end of user input
#				
#								(begining of the "I really know what I am doing" section)
#
#########################################################################################

#-----------------------------------------------------------------------------------------------------

gridpath = '~/work/mstw_pdfs'
real_run = 'yes'
import os,sys,time,copy
module_dir = os.getcwd()
sys.path.insert(0,module_dir)
import ClassesFehip 

jobdefs = {}
myfixed = {}

#-- Do the QCD calculation
possible_flags = ['perturbative_order','type','massive_corrections','electroweak_corrections',
'muFrel','higgs_width_flag','collider','sqrtS','decay','pdfset','calculate_pdf_error',
'vary_a_s','accRel','accAbs','ntherm','ncalls','nincr','WilsonFile',
'PtHiggsCut','PtHiggsMin','PtHiggsMax','EtaHiggsCut','EtaHiggsMin','EtaHiggsMax',
'UseSISCone','JetConeSize','JetDefPtMin','JetDefEtaMax','JetVeto','JetPtMax','JetEtaMax',
'NumVeto','JetNmax','TeVVeto','TeVJetPtMax1','TeVJetPtMax2',
'MassV1Cut','MassV1Min','MassV1Max','MassV2Cut','MassV2Min','MassV2Max',
'PtV1Cut','PtV1Min','PtV1Max','PtV2Cut','PtV2Min','PtV2Max',
'EtaV1Cut','EtaV1Min','EtaV1Max','EtaV2Cut','EtaV2Min','EtaV2Max',
'cut_on_pt_of_all_leptons','all_leptons_minimum_pt',
'cut_on_rapidity_of_all_leptons','all_leptons_maximum_y',
'real_run'
]


job_flags=['results_dir','project','ifcluster','walltime','submitts','gridpath']
for xname in dir():
    if (xname in possible_flags):
        myfixed[xname]=eval(xname)
for xname in dir():
    if (xname in job_flags):
        jobdefs[xname]=eval(xname)
#myfixed['real_run']='no'  # the directory structure and the input files are produced but the jobs are not submitted
fehipjob=ClassesFehip.FeHiProProject(m_higgs,factorization_scale,myfixed,jobdefs)

