#!/usr/bin/env python 

#########################################################################################
#								user input
#
#
#								physics input
#
results_dir = 'results'                 # top directory with results from different runs
project = 'gamma_gamma'                          # directory with results of current run
type = 'exclusive'                      # inclusive (fast, no decay) or exclusive
m_higgs	= ['120']                       # mass value or scan set
#m_higgs = range(100,625,25)            #  mass scan: eg. from 100 to 600 with a step of 25
factorization_scale = ['0.5']           # factorization scale value or scan range
vary_a_s = 'no'                         # vary the a_s within the pdf sets ? (yes or no)
perturbative_order='NNLO'               # LO, NLO, NNLO
collider = 'LHC'                        # LHC, Tevatron
sqrtS = '7000'                          # in GeV
electroweak_corrections = 'virtual-real'# 'off', 'only-virtual', 'virtual-real'
decay = 'gamma gamma'                   # 'none' , 'gamma gamma', 'WW', 'ZZ', 'WW-ZZ' , 'ZZ->l1 l1 l2 l2'
massive_corrections = 'top-bottom'		# 'none', 'top-only', 'top-bottom'
pdfset= '4.0'                           # 4.0= MSTW 2008; 3.0= MRST 2006; 2.0= MRST 2004; 1.0= MRST 2001
calculate_pdf_error = 'yes'             # yes or no
#
ifcluster = 'yes'						# are you running on a cluster ? (yes or no)
walltime = '480'                        # time limit in minutes for the job (in case you run on a cluster with time limited queues)
submitts='1'                            # For cluster: Number of submissions of the same job. 
#
accRel='0.01'                           # in percentage, i.e. 0.8 means 0.8%
accAbs = '0.0'                          # requires knowledge of the cross section (per sector in the exclusive mode case)
ncalls = '50000'                        # points per iteration in thermalization stage
nincr= '0'                              # increase in points from iteration to iteration in thermalization stage
ntherm = '10'                           # number of thermalization iterations in vegas
WilsonFile = 'xxxxx'                    # file for redefinition of Wilson coefficients
higgs_width_flag = 'no'                 # finite higgs width effects, 'yes' or 'no'


#----------------------     cuts

#########  Cuts on Higgs directly  #########
#-- PtHiggsCut>0.5: Cut on Higgs pT, PtHiggsMin < pT < PtHiggsMax
#PtHiggsCut= '0.0'
#PtHiggsMin= '0.0'
#PtHiggsMax= '0.0'

#-- EtaHiggsCut>0.5: Cut on Higgs rapidity, EtaHiggsMin < y < EtaHiggsMax
#EtaHiggsCut= '0.0'
#EtaHiggsMin= '0.0'
#EtaHiggsMax= '0.0'

#########  General cuts  #########
#-- UseSISCone=0.0: simple cone algorithm
#-- UseSISCone>0.5: SISCone jet algorithm
#UseSISCone= '0.0'

#-- Cone size for jet definition. In the case of SISCone, this gets rescaled.  
#JetConeSize= '0.0'

#-- Minimum pT for jet definition 
#JetDefPtMin= '0.0'

#-- Maximum rapidity for jet definition
#JetDefEtaMax='0.0'

#-- Jet veto
#-- JetVeto>0.5: No jet with pt_jet>JetPtMax and |eta_jet|<JetEtaMax
#JetVeto= '0.0'
#JetPtMax= '0.0'
#JetEtaMax= '0.0'

#-- NumVeto>0.5: cut on number of jets <= JetNmax
#NumVeto= '0.0'
#JetNmax= '0'

#-- Special Tevatron cut on number of jets.
#-- If number of Jets =1, require pTjet < TeVJetPtMax1
#-- if number of Jets =2, require pTjet1,pTjet2 < TeVJetPtMax2
#TeVVeto= '0.0'
#TeVJetPtMax1= '0.0'
#TeVJetPtMax2= '0.0'

#########  Cuts for di-photon final state  #########
#-- Basic Photon cut
#-- PhotBasicCut>0.5:
#-- Harder photon must have pT > PhotPtMin1, Softer must have pT > PhotPtMin2
#-- Both photons must have pseudo-rapidity in central region: |eta| < PhotEtaMax

#PhotBasicCut= '1.0'
#PhotPtMin1= '40.0'
#PhotPtMin2= '25.0'
#PhotEtaMax= '2.5'



#-- Photon isolation cut (requires basic photon cut)
#-- PhotIsolCut>0.5:
#-- No jet with pt_jet > PhotEtmax in a cone of radius PhotConeIso around photon
#PhotIsolCut= '1.0'
#PhotConeIso= '0.4'
#PhotEtmax= '15.0'

#-- Cut on pT of softer photon
#-- PtSoftCut>0.5: pT_soft > PtSoftMin
#PtSoftCut= '0.0'
#PtSoftMin= '0.0'

#-- Cut on average pT of photons
#-- PtmCut>0.5: PtmMin < (pT_photon1+pT_photon2)/2 < PtmMax
#PtmCut= '0.0'
#PtmMin= '0.0'
#PtmMax= '0.0'

#-- Cut on pseudo-rapidity of higher energetic photon
#-- PhotEtaHardCut>0.5: PhotEtaHardMin < |eta_hard| < PhotEtaHardMax
#PhotEtaHardCut= '0.0'
#PhotEtaHardMin= '0.0'
#PhotEtaHardMax= '0.0'

#-- Cut on ystar (pseudo-rapidity difference) of photons
#-- YstarCut>0.5: YstarMin < (|eta_photon1 - eta_photon2|)/2 < YstarMax
#YstarCut= '0.0'
#YstarMin= '0.0'
#YstarMax= '0.0'






#
#
#								end of user input
#				
#								(begining of the "I really know what I am doing" section)
#
#########################################################################################

#-----------------------------------------------------------------------------------------------------

import os,sys,time,copy
module_dir = os.getcwd()
sys.path.insert(0,module_dir)
import ClassesFehip 

jobdefs = {}
myfixed = {}

#-- Do the QCD calculation
possible_flags = ['perturbative_order','type','massive_corrections','electroweak_corrections',
'muFrel','higgs_width_flag','collider','sqrtS','decay','pdfset','calculate_pdf_error',
'vary_a_s','accRel','accAbs','ntherm','ncalls','nincr','WilsonFile',
'PtHiggsCut','PtHiggsMin','PtHiggsMax','EtaHiggsCut','EtaHiggsMin','EtaHiggsMax',
'UseSISCone','JetConeSize','JetDefPtMin','JetDefEtaMax','JetVeto','JetPtMax','JetEtaMax',
'NumVeto','JetNmax','TeVVeto','TeVJetPtMax1','TeVJetPtMax2','EtaV2Cut','EtaV2Min','EtaV2Max',
'PhotBasicCut','PhotPtMin1','PhotPtMin2','PhotEtaMax',
'PhotIsolCut','PhotConeIso','PhotEtmax','PtSoftCut','PtSoftMin',
'PtmCut','PtmMin','PtmMax','PhotEtaHardCut','PhotEtaHardMin','PhotEtaHardMax',
'YstarCut','YstarMin','YstarMax'
]


job_flags=['results_dir','project','ifcluster','walltime','submitts']
for xname in dir():
    if (xname in possible_flags):
        myfixed[xname]=eval(xname)
for xname in dir():
    if (xname in job_flags):
        jobdefs[xname]=eval(xname)

#myfixed['real_run']='no'  # the directory structure and the input files are produced but the jobs are not submitted
fehipjob=ClassesFehip.FeHiProProject(m_higgs,factorization_scale,myfixed,jobdefs)

    
    

