CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     SMEggh.f
C
C     Evaluates the born squared matrix element (po=0,1;icon=0) 
C     and the interference of virtual and born contribution (po=1,icon=1)
C     as well as the scheme dependent part proportional to mass derivative
C     of born contribution (po=1,icon=2)



      function SMEggh(porder,icon,s12)
CCC   porder=0: Leading order 
CCC   porder=1: NLO
CCC   icon=0: Born contribution
CCC   icon=1: interference Born x virtual
CCC   icon=2: scheme dependent part proportional to derivative of born
      implicit none
      double precision SMEggh
      double precision s12
      integer porder,icon

      double precision mer,mei,merb,meib,smer,smei,smerb,smeib
      double precision tmerb,tmeib,tmer,tmei
      double precision dtmerb,dtmeib,dtmer,dtmei
      
      double precision bmerb,bmeib,bmer,bmei
      double precision stmer1b,stmei1b,stmer1,stmei1
      double precision stmer2b,stmei2b,stmer2,stmei2
      double precision sbmer1b,sbmei1b,sbmer1,sbmei1
      double precision sbmer2b,sbmei2b,sbmer2,sbmei2
      double precision mesquared
      double precision prefac
      double precision mebornmtop,mevirtualmtop
      double precision smebornmscalar,smevirtualmscalar
      integer po,ic
      
      external mebornmtop,mevirtualmtop
      external smebornmscalar,smevirtualmscalar

      double precision rnorm
      common/hpronorm/rnorm

      include 'model.f'
      include 'xsection.f'
      include 'hprocontrol.f'


      SMEggh=0d0
      tmer=0d0
      tmei=0d0
      dtmer=0d0
      dtmei=0d0
      stmer1=0d0
      stmei1=0d0
      stmer2=0d0
      stmei2=0d0
      tmerb=0d0
      tmeib=0d0
      dtmerb=0d0
      dtmeib=0d0
      stmer1b=0d0
      stmei1b=0d0
      stmer2b=0d0
      stmei2b=0d0
      bmer=0d0
      bmei=0d0
      sbmer1=0d0
      sbmei1=0d0
      sbmer2=0d0
      sbmei2=0d0
      bmerb=0d0
      bmeib=0d0
      sbmer1b=0d0
      sbmei1b=0d0
      sbmer2b=0d0
      sbmei2b=0d0
      rnorm=0d0
      

C     C----------------------------------------------------
      if (porder.eq.0) then
         linfty=linftyLO
      else
         linfty=linftyNLO
      endif


C      write(6,*)'SMEggh: ',s12
C      write(6,*)'SMEggh: ',porder,icon,linftyLO,linftyNLO
C      write(6,*)'        ',ltop,lbot,ltopbot


C     C----------------------------------------------------
      
      prefac=as**2*mh**4/(32d0*PI**2)*(Nc**2-1d0)

      po=0
      ic=0
      if ((ltop.ne.0d0).or.(ltopbot.ne.0d0).or.(linfty.ne.0d0)) then
         if ((ytth).ne.0d0) then
            call MEgghfermion(po,ic,s12,mt2,tmerb,tmeib)
            tmer=tmerb
            tmei=tmeib
         endif
      endif
      if ((lbot.ne.0d0).or.(ltopbot.ne.0d0)) then
         if ((ybbh).ne.0d0) then
            call MEgghfermion(po,ic,s12,mb2,bmerb,bmeib)
            bmer=bmerb
            bmei=bmeib
         endif
      endif
C      if (linfty.ne.0d0) then
      rnorm=(tmerb**2+tmeib**2)/mebornmtop()**2
C     write(6,*)'rnorm= ',rnorm
C     rnorm=1d0
      dtmerb=mebornmtop()
      dtmeib=0d0
      dtmer=dtmerb
      dtmei=dtmeib
C      endif

CCC   --------------------------------
CCC   SUSY contribution
      if (imodel.eq.2) then
         call MEgghscalar(po,ic,s12,ms12,stmer1b,stmei1b)
         stmer1=stmer1b
         stmei1=stmei1b
         call MEgghscalar(po,ic,s12,ms22,stmer2b,stmei2b)
         stmer2=stmer2b
         stmei2=stmei2b
         call MEgghscalar(po,ic,s12,msb12,sbmer1b,sbmei1b)
         sbmer1=sbmer1b
         sbmei1=sbmei1b
         call MEgghscalar(po,ic,s12,msb22,sbmer2b,sbmei2b)
         sbmer2=sbmer2b
         sbmei2=sbmei2b
      endif

CCC   --------------------------------
CCC   virtual corrections
      if (porder.eq.1) then

         if (icon.eq.1) then
            po=1
            ic=1
            prefac=2d0*prefac*as/(4d0*PI)
            
            tmer=0d0
            tmei=0d0
            dtmer=0d0
            dtmei=0d0
            bmer=0d0
            bmei=0d0
            stmer1=0d0
            stmei1=0d0
            stmer2=0d0
            stmei2=0d0
            sbmer1=0d0
            sbmei1=0d0
            sbmer2=0d0
            sbmei2=0d0


C     smer=0d0
C     smei=0d0

            if ((ltop.ne.0d0).or.(ltopbot.ne.0d0)
     $           .or.(linfty.ne.0d0)) then
               if ((ytth).ne.0d0) then
                  call MEgghfermion(po,ic,s12,mt2,tmer,tmei)
               endif
            endif
            if ((lbot.ne.0d0).or.(ltopbot.ne.0d0)) then
               if ((ybbh).ne.0d0) then
                  call MEgghfermion(po,ic,s12,mb2,bmer,bmei)
               endif
            endif
C            if (linfty.ne.0d0) then
            dtmer=mevirtualmtop()
            dtmei=0d0
C            endif

CCC   --------------------------------
CCC   SUSY contribution
            if (imodel.eq.2) then
               call MEgghscalar(po,ic,s12,ms12,stmer1,stmei1)
               call MEgghscalar(po,ic,s12,ms22,stmer2,stmei2)
               call MEgghscalar(po,ic,s12,msb12,sbmer1,sbmei1)
               call MEgghscalar(po,ic,s12,msb22,sbmer2,sbmei2)
            endif
CCC   Higgs coupling renormalization ????
            
         elseif (icon.eq.2) then
            po=1
            ic=2
            prefac=as/(2d0*PI)*prefac

            tmer=0d0
            tmei=0d0
            dtmer=0d0
            dtmei=0d0
            bmer=0d0
            bmei=0d0
            stmer1=0d0
            stmei1=0d0
            stmer2=0d0
            stmei2=0d0
            sbmer1=0d0
            sbmei1=0d0
            sbmer2=0d0
            sbmei2=0d0
c     smer=0d0
C     smei=0d0

            if ((ltop.ne.0d0).or.(ltopbot.ne.0d0)
     $           .or.(linfty.ne.0d0)) then
               if (ytth.ne.0d0) then
                  call MEgghfermion(po,ic,s12,mt2,tmer,tmei)
                  tmer=(-dlog(mh**2/muR**2)*3d0*CF+ Fmtscheme)*tmer
                  tmei=(-dlog(mh**2/muR**2)*3d0*CF+ Fmtscheme)*tmei

               endif
            endif
            if ((lbot.ne.0d0).or.(ltopbot.ne.0d0)) then
               if (ybbh.ne.0d0) then
                  call MEgghfermion(po,ic,s12,mb2,bmer,bmei)
                  bmer=(-dlog(mh**2/muR**2)*3d0*CF+ Fmbscheme)*bmer
                  
                  bmei=(-dlog(mh**2/muR**2)*3d0*CF+ Fmbscheme)*bmei
                  
               endif
            endif

            if (imodel.eq.2) then
               call MEgghscalar(po,ic,s12,ms12,stmer1,stmei1)
               stmer1=0.5d0*(-dlog(mh**2/muR**2)*3d0*CF
     $              +Fms1scheme)*stmer1
C     $              + (-2*beta0*dlog(mh2/muR**2)
C     $              + 2d0*Fgscheme)*stmer1b
               stmei1=0.5d0*(-dlog(mh**2/muR**2)*3d0*CF
     $              +Fms1scheme)*stmei1
C     $              + (-2*beta0*dlog(mh2/muR**2)
C     $              + 2d0*Fgscheme)*stmei1b
               call MEgghscalar(po,ic,s12,ms22,stmer2,stmei2)
               stmer2=0.5d0*(-dlog(mh**2/muR**2)*3d0*CF
     $              +Fms2scheme)*stmer2
C     $              + (-2*beta0*dlog(mh2/muR**2)
C     $              + 2d0*Fgscheme)*stmer2b
               stmei2=0.5d0*(-dlog(mh**2/muR**2)*3d0*CF
     $              +Fms2scheme)*stmei2
C     $              + (-2*beta0*dlog(mh2/muR**2)
C     $              + 2d0*Fgscheme)*stmei2b
               call MEgghscalar(po,ic,s12,msb12,sbmer1,sbmei1)
               sbmer1=0.5d0*(-dlog(mh**2/muR**2)*3d0*CF
     $              +Fmsb1scheme)*sbmer1
C     $              + (-2*beta0*dlog(mh2/muR**2)
C     $              + 2d0*Fgscheme)*sbmer1b
               sbmei1=0.5d0*(-dlog(mh**2/muR**2)*3d0*CF
     $              +Fmsb1scheme)*sbmei1
C     $              + (-2*beta0*dlog(mh2/muR**2)
C     $              + 2d0*Fgscheme)*sbmei1b
               call MEgghscalar(po,ic,s12,msb22,sbmer2,sbmei2)
               sbmer2=0.5d0*(-dlog(mh**2/muR**2)*3d0*CF
     $              + Fmsb2scheme)*sbmer2
C     $              + (-2*beta0*dlog(mh2/muR**2)
C     $              + 2d0*Fgscheme)*sbmer2b
               sbmei2=0.5d0*(-dlog(mh**2/muR**2)*3d0*CF 
     $              + Fmsb2scheme)*sbmei2
C     $              + (-2*beta0*dlog(mh2/muR**2)
C     $              + 2d0*Fgscheme)*sbmei2b
            endif

            
            tmer=tmer+ (-2*beta0*dlog(mh2/muR**2) + 2d0*Fgscheme)*tmerb
            tmei=tmei+ (-2*beta0*dlog(mh2/muR**2) + 2d0*Fgscheme)*tmeib
            bmer=bmer+ (-2*beta0*dlog(mh2/muR**2) + 2d0*Fgscheme)*bmerb
            bmei=bmei+ (-2*beta0*dlog(mh2/muR**2) + 2d0*Fgscheme)*bmeib

            dtmer=dtmer+ (-2*beta0*dlog(mh2/muR**2) 
     $           + 2d0*Fgscheme)*dtmerb
            dtmei=dtmei+ (-2*beta0*dlog(mh2/muR**2) 
     $           + 2d0*Fgscheme)*dtmeib

            if (imodel.eq.2) then
               stmer1=stmer1
     $              + (-2*beta0*dlog(mh2/muR**2)
     $              + 2d0*Fgscheme)*stmer1b
               stmei1=stmei1
     $              + (-2*beta0*dlog(mh2/muR**2)
     $              + 2d0*Fgscheme)*stmei1b
               stmer2=stmer2
     $              + (-2*beta0*dlog(mh2/muR**2)
     $              + 2d0*Fgscheme)*stmer2b
               stmei2=stmei2
     $              + (-2*beta0*dlog(mh2/muR**2)
     $              + 2d0*Fgscheme)*stmei2b
               sbmer1=sbmer1
     $              + (-2*beta0*dlog(mh2/muR**2)
     $              + 2d0*Fgscheme)*sbmer1b
               sbmei1=sbmei1
     $              + (-2*beta0*dlog(mh2/muR**2)
     $              + 2d0*Fgscheme)*sbmei1b
               sbmer2=sbmer2
     $              + (-2*beta0*dlog(mh2/muR**2)
     $              + 2d0*Fgscheme)*sbmer2b
               sbmei2=sbmei2
     $              + (-2*beta0*dlog(mh2/muR**2)
     $              + 2d0*Fgscheme)*sbmei2b
            endif
            
         endif

      endif

      smer=relt(1)*yssh(1,1)*stmer1+relt(2)*yssh(2,2)*stmer2
     $     +relb(1)*ysbsbh(1,1)*sbmer1+relb(2)*ysbsbh(2,2)*sbmer2
      smei=relt(1)*yssh(1,1)*stmei1+relt(2)*yssh(2,2)*stmei2
     $     +relb(1)*ysbsbh(1,1)*sbmei1+relb(2)*ysbsbh(2,2)*sbmei2
      smerb=relt(1)*yssh(1,1)*stmer1b+relt(2)*yssh(2,2)*stmer2b
     $     +relb(1)*ysbsbh(1,1)*sbmer1b+relb(2)*ysbsbh(2,2)*sbmer2b
      smeib=relt(1)*yssh(1,1)*stmei1b+relt(2)*yssh(2,2)*stmei2b
     $     +relb(1)*ysbsbh(1,1)*sbmei1b+relb(2)*ysbsbh(2,2)*sbmei2b

c$$$  write(6,*)'+++++++++++++++++++++++++++++'
c$$$  write(6,*)'SMEggh: ',porder,icon,s12,as
c$$$  write(6,*)tmer,tmei,bmer,bmei
c$$$  write(6,*)tmerb,tmeib,bmerb,bmeib
c$$$  write(6,*)stmer1,stmei1,stmer2,stmei2
c$$$  write(6,*)sbmer1,sbmei1,sbmer2,sbmei2
c$$$  C      write(6,*)'++ ',relt(1)*yssh(1,1)*stmer1,relt(2)*yssh(2,2)*stmer2
c$$$  C     $     ,relb(1)*ysbsbh(1,1)*sbmer1,relb(2)*ysbsbh(2,2)*sbmer2
c$$$  C      write(6,*)'++ ',mer,mei,merb,meib
c$$$  C      write(6,*)'++ ',smer,smei,smerb,smeib
c$$$  write(6,*)'++ Couplings: ',ytth,ybbh
c$$$  write(6,*)'++            ',yssh
c$$$  write(6,*)'++            ',ysbsbh
c$$$  write(6,*)'++            ',relt,relb



CCC   lbot*bot^2
      mesquared=lbot*(bmer*bmerb+bmei*bmeib)*ybbh**2
CCC   ltopbot*(top x bot)
      mesquared=mesquared  + ltopbot*(tmer*bmerb+bmer*tmerb
     $     +tmei*bmeib+bmei*tmeib)*ybbh*ytth
CCC   ltop*top^2+linfty*infty^2
      mesquared=mesquared  
     $     + (ltop*(tmer*tmerb+tmei*tmeib)*ytth**2
     $     + linfty*ytth**2*rnorm*(dtmerb*dtmer+dtmeib*dtmei))
CCC   top*susy
      mesquared=mesquared + (smer*tmerb+tmer*smerb
     $     +smei*tmeib+tmei*smeib)*ytth
CCC   bot*susy
      mesquared=mesquared + (smer*bmerb+bmer*smerb
     $     +smei*bmeib+bmei*smeib)*ybbh
CCC   susy^2
      mesquared=mesquared + (smer*smerb+smei*smeib)

      SMEggh=prefac*mesquared



c$$$      write(6,*)prefac,ltop,ltopbot,lbot
c$$$      write(6,*)rnorm,tmer,bmer,dtmer
c$$$      write(6,*)'SMEggh= ',SMEggh,mesquared,prefac
C      write(6,*)'as= ',as
      return
      end

C==================================================================
C     icon=0: Born, full mass dependence
C     icon=1: Virtual, full mass dependence
C     icon=2: derivative of Born term
      subroutine MEgghfermion(porder,icon,s,m2,mer,mei)
      implicit none 
      double precision s,m2,mer,mei
      integer porder,icon
      

      include 'auxini.f'

C      write(6,*)'MEgghfermion: ',porder,icon

C      write(6,*)'MEgghfermion: (porder,s,m2)= ',porder,s,m2
      mer=0d0
      mei=0d0

      rho=s/(4d0*m2)

C     write(6,*)'rho= ',rho
      if ((rho.lt.1.D0).and.(rho.ge.0.D0)) then
         if (rho.eq.0.D0) then
            theta=0.D0
         elseif ((rho.gt.0.D0).and.(rho.lt.0.5D0)) then
            tmp=dsqrt(rho-rho**2)
            tmp=tmp/(1.D0-2.D0*rho)
            theta=datan(2.D0*tmp)
         elseif (rho.eq.0.5D0) then
            theta=PI/2.d0
         elseif (rho.gt.0.5D0) then
            tmp=dsqrt(rho-rho**2)
            tmp=tmp/(2.D0*rho-1.D0)
            theta=PI-datan(2.D0*tmp)
         endif
         if (icon.eq.2) then
            call mederbornac(theta,mer,mei)
         elseif (icon.eq.0) then
            call mebornac(theta,mer,mei)
         elseif (icon.eq.1) then
CCC   Initialize Ls functions
            call filllsfuncs(theta,acl1,acl2,als1,als2,als3,
     $           alsc1,alsc2,alsc3,alslsc1,alslsc2,alslsc3)
C     write(6,*)'theta= ',theta
            call mevirtac(theta,acl1,acl2,als1,als2,als3,alsc1,
     $           alsc2,alsc3,alslsc1,alslsc2,alslsc3,mer,mei)

CCC   mevirtac computes matrix elements renormalized in msbar-scheme!!
         endif
      elseif (rho.ge.1.D0) then
C     write(6,*)' '
         root1=dsqrt(rho-1.D0)
         root2=dsqrt(rho)
         x=(root1-root2)/(root1+root2)
         call hplog(x,nw,Hc1,Hc2,Hc3,Hc4,
     $        Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,n1,n2)
         if (icon.eq.2) then
            call mederbornsp(x,Hr1(0),Hi1(0),mer,mei)
         elseif (icon.eq.0) then
            call mebornsp(x,Hr1(0),Hi1(0),mer,mei)
         elseif (icon.eq.1) then
            call mevirtsp(x,Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,mer,mei)
         endif
         
      endif

      
      mer=-mer
      mei=-mei

C      write(6,*)'(mer,mei)= ',mer,mei
      return
      end

C==================================================================
C     porder=0: Born, full mass dependence
C     porder=1: Virtual, full mass dependence
      subroutine MEgghscalar(porder,icon,s,m2,mer,mei)
      implicit none 
      double precision s,m2,mer,mei
      integer porder,icon
      

      include 'auxini.f'

      mer=0d0
      mei=0d0

      rho=s/(4d0*m2)

      if ((rho.lt.1.D0).and.(rho.ge.0.D0)) then
         if (rho.eq.0.D0) then
            theta=0.D0
         elseif ((rho.gt.0.D0).and.(rho.lt.0.5D0)) then
            tmp=dsqrt(rho-rho**2)
            tmp=tmp/(1.D0-2.D0*rho)
            theta=datan(2.D0*tmp)
         elseif (rho.eq.0.5D0) then
            theta=PI/2.d0
         elseif (rho.gt.0.5D0) then
            tmp=dsqrt(rho-rho**2)
            tmp=tmp/(2.D0*rho-1.D0)
            theta=PI-datan(2.D0*tmp)
         endif
         if (icon.eq.2) then
            call smederbornac(theta,mer,mei)
         elseif (icon.eq.0) then
            call smebornac(theta,mer,mei)
         elseif (icon.eq.1) then
CCC   Initialize Ls functions
            call filllsfuncs(theta,acl1,acl2,als1,als2,als3,
     $           alsc1,alsc2,alsc3,alslsc1,alslsc2,alslsc3)
C     write(6,*)'theta= ',theta
            call smevirtac(theta,acl1,acl2,als1,als2,als3,alsc1,
     $           alsc2,alsc3,alslsc1,alslsc2,alslsc3,mer,mei)
         endif
      elseif (rho.ge.1.D0) then
C     write(6,*)' '
         root1=dsqrt(rho-1.D0)
         root2=dsqrt(rho)
         x=(root1-root2)/(root1+root2)
         call hplog(x,nw,Hc1,Hc2,Hc3,Hc4,
     $        Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,n1,n2)
         if (icon.eq.2) then
            call smederbornsp(x,Hr1(0),Hi1(0),mer,mei)
         elseif (icon.eq.0) then
            call smebornsp(x,Hr1(0),Hi1(0),mer,mei)
         elseif (icon.eq.1) then
            call smevirtsp(x,Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,mer,mei)
         endif

      endif


      mer=-mer
      mei=-mei

      return
      end
C==================================================================

